/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editdialog.h"
#include "editdialog.moc"

#include "detailswidget.h"
#include <epushbutton.h>

#include <QSettings>
#include <QAbstractItemView>
#include <QSortFilterProxyModel>
#include <QAbstractItemModel>
#include <QLabel>
#include <QLineEdit>
#include <QSplitter>
#include <QVBoxLayout>
#include <QHBoxLayout>

EditDialog::EditDialog ( QWidget* parent, Qt::WindowFlags flags ) : EDialog ( parent, flags )
{
  sortingColumn = -1;
  model = 0;
  proxyModel = 0;
  detailsWidget = 0;
  existingView = 0;
	editMode = false;
}

void EditDialog::setEditMode()
{
	editMode = true;
}

void EditDialog::addNew()
{
  // make curIndex invalid, since the new genres isn't at the model
  selectionChanged ( QModelIndex() );

	detailsWidget->setItem(QModelIndex());
  detailsWidget->setEnabled ( true, filterEdit->text() );
  saveButton->setEnabled ( true );
}

void EditDialog::editSelected()
{
  detailsWidget->setEnabled ( true );
  detailsWidget->setItem ( curIndex );
  saveButton->setEnabled ( true );
}

void EditDialog::finish()
{
	if(editMode){
		accept();
		return;
	}
	
  if ( !curIndex.isValid() ) {
    emit error ( finishFailedText );
    return;
  }

  accept();
}

void EditDialog::cancel()
{
  // make index invalid, because no selection should be made
  selectionChanged ( QModelIndex() );
  reject();
}

void EditDialog::selectionChanged ( QModelIndex index )
{
  if ( !proxyModel )
    return;

  if ( !existingView )
    return;

  if ( !detailsWidget )
    return;

  curIndex = proxyModel->mapToSource ( index );

  existingView->setCurrentIndex ( index );

  // setup the buttons
  editButton->setEnabled ( false );

  saveButton->setEnabled ( false );

  detailsWidget->setEnabled ( false );

  if ( curIndex.isValid() ){
    editButton->setEnabled ( true );
  // setup the details widget
    detailsWidget->setItem ( curIndex );
	}
}

void EditDialog::filter ( QString string )
{
  if ( !proxyModel )
    return;

  selectionChanged ( QModelIndex() );

  filterEdit->setText ( string );

  if ( string.isEmpty() ) {
    proxyModel->setFilterRegExp ( "" );
  } else {
    proxyModel->setFilterRegExp ( QRegExp::escape(string) );
  }

  if ( proxyModel->rowCount() == 1 ) {
		QModelIndex index = proxyModel->index ( 0, sortingColumn );
    existingView->setCurrentIndex ( index );
    selectionChanged ( index );
  }
}

void EditDialog::closeEvent ( QCloseEvent* event )
{
  QSettings settings;
  settings.beginGroup ( "geometries" );
  settings.setValue ( splitter->objectName(), splitter->saveState() );
  settings.endGroup();

  EDialog::closeEvent ( event );
}

void EditDialog::setupUi ( QAbstractItemView* existingView, DetailsWidget* detailsWidget, QString filterString )
{
  this->existingView = existingView;
  this->detailsWidget = detailsWidget;

  if ( !existingView )
    return;

  if ( !detailsWidget )
    return;
	
	connectByPass(detailsWidget);

  QVBoxLayout *mlayout = new QVBoxLayout ( this );
  setLayout ( mlayout );

  splitter = new QSplitter ( this );
  splitter->setObjectName ( objectName() +"Splitter" );
  mlayout->addWidget ( splitter );

  // creating the left (existing stuff) part of the splitter
  QWidget *widget = new QWidget ( splitter );
  splitter->addWidget ( widget );
  QVBoxLayout *widgetLayout = new QVBoxLayout ( widget );
  widget->setLayout ( widgetLayout );
  existingLabel = new QLabel ( existingLabelText, widget );
  widgetLayout->addWidget ( existingLabel );
	
  // filter row
  QHBoxLayout *filterLayout = new QHBoxLayout();
  widgetLayout->addLayout ( filterLayout );

  filterLabel = new QLabel ( tr ( "Filter:" ), widget );
  filterLayout->addWidget ( filterLabel );
	
  filterEdit = new QLineEdit ( widget );
  connect ( filterEdit, SIGNAL ( textChanged ( QString ) ), this, SLOT ( filter ( QString ) ) );
  filterLayout->addWidget ( filterEdit );
	
  widgetLayout->addWidget ( existingView );
  connect ( existingView, SIGNAL ( clicked ( QModelIndex ) ), this, SLOT ( selectionChanged ( QModelIndex ) ) );

  // adding the buttons
  QHBoxLayout *buttonLayout = new QHBoxLayout();
  mlayout->addLayout ( buttonLayout );

	helpButton = new EPushButton ( tr ( "&Help" ), widget );
	buttonLayout->addWidget( helpButton );
	setHelp("keyword:Edit Dialog", helpButton);
	
	buttonLayout->addStretch();
	
  okButton = new EPushButton ( tr ( "Ok" ), widget );
  connect ( okButton, SIGNAL ( clicked() ), this, SLOT ( finish() ) );
  buttonLayout->addWidget ( okButton );

  cancelButton = new EPushButton ( tr ( "Cancel" ), widget );
  connect ( cancelButton, SIGNAL ( clicked() ), this, SLOT ( cancel() ) );
  buttonLayout->addWidget ( cancelButton );

  // create the right (the details stuff) of the splitter
  widget = new QWidget ( splitter );
  splitter->addWidget ( widget );
  widgetLayout = new QVBoxLayout ( widget );
  widget->setLayout ( widgetLayout );

  detailsLabel = new QLabel ( tr ( "Details" ), widget );
  widgetLayout->addWidget ( detailsLabel );

  widgetLayout->addWidget ( detailsWidget );
  detailsWidget->setEnabled ( false );
  connectByPass ( detailsWidget );
	
  buttonLayout = new QHBoxLayout();
  widgetLayout->addLayout ( buttonLayout );
  buttonLayout->addStretch();

  newButton = new EPushButton ( tr ( "New" ), widget );
  connect ( newButton, SIGNAL ( clicked() ), this, SLOT ( addNew() ) );
  buttonLayout->addWidget ( newButton );

  editButton = new EPushButton ( tr ( "Edit" ), widget );
  connect ( editButton, SIGNAL ( clicked() ), this, SLOT ( editSelected() ) );
  buttonLayout->addWidget ( editButton );

  saveButton = new EPushButton ( tr ( "Save" ), widget );
  connect ( saveButton, SIGNAL ( clicked() ), this, SLOT ( store() ) );
  buttonLayout->addWidget ( saveButton );

  QSettings settings;
  settings.beginGroup ( "geometries" );
  splitter->restoreState ( settings.value ( splitter->objectName() ).toByteArray() );
  settings.endGroup();
	
  if ( ( sortingColumn < 0 ) || ( model->columnCount() <= sortingColumn ) )
    filterEdit->setEnabled ( false );

  // creating the models
  proxyModel = new QSortFilterProxyModel ( this );
  // apply changes off the original model to the proxy model
  proxyModel->setDynamicSortFilter ( true );
  proxyModel->setSourceModel ( model );
  proxyModel->sort ( sortingColumn );
  proxyModel->setFilterKeyColumn ( sortingColumn );
  proxyModel->setFilterCaseSensitivity ( Qt::CaseInsensitive );
  existingView->setModel ( proxyModel );

  selectionChanged ( QModelIndex() );

  if ( !filterString.isEmpty() )
    filter ( filterString );

	setTabOrder(filterEdit, existingView);
	setTabOrder(existingView, detailsWidget);
	setTabOrder(detailsWidget, newButton);
	setTabOrder(newButton, editButton);
	setTabOrder(editButton, saveButton);
	setTabOrder(saveButton, helpButton);
	setTabOrder(helpButton, okButton);
	setTabOrder(okButton, cancelButton);
  filterEdit->setFocus();
}

void EditDialog::postStore ( QString filterString )
{
  // select the entered genres
  filter ( filterString );

  QModelIndex index;

  if ( proxyModel->rowCount() == 1 )
    index = proxyModel->index ( 0, sortingColumn );

// 	filter ( "" );
	
  // select the item
  if ( index.isValid() )
    selectionChanged ( index );
}
