/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editmedium.h"
#include "editmedium.moc"
#include "ui_editmedium.h"

#include "medium.h"
#include "mediamodel.h"
#include "mediumeditwidget.h"
#include <esqldatabase.h>

#include <QSqlQuery>
#include <QMessageBox>
#include <QLabel>

#include <QDebug>

EditMedium::EditMedium(QWidget* parent, Qt::WindowFlags flags)
	: EDialog(parent, flags), ui(new Ui::EditMedium)
{
	medium = NULL;
	model = NULL;
	editMode = Edit;
	startTransaction = true;
	ui->setupUi(this);
	setHelp("keyword:Edit Medium", ui->helpButton);
	setWindowTitle(tr("Medium"));
	
	connect(ui->okButton, SIGNAL(clicked()), this, SLOT(save()));
	connect(ui->cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));

	editWidgets << ui->titleWidget << ui->manufacturerWidget;
	editWidgets << ui->genreWidget << ui->personsWidget;
	editWidgets << ui->descriptionWidget << ui->minWidget;
	editWidgets << ui->wishWidget << ui->literatureWidget;
	editWidgets << ui->evaluationWidget << ui->boughtPriceWidget;
	editWidgets << ui->listPriceWidget << ui->giftWidget;
	editWidgets << ui->ownerWidget << ui->loanWidget;
	editWidgets << ui->imageWidget << ui->keywordWidget;
	editWidgets << ui->filesWidget;

	foreach(MediumEditWidget *widget, editWidgets)
		connectByPass(widget);
}

void EditMedium::setAddMode(Medium* medium, bool newTransaction)
{
	editMode = Add;
	setWindowTitle(tr("Add Medium to Database"));
	setMedium(medium, newTransaction);
}

void EditMedium::setEditMode(Medium* medium, bool newTransaction)
{
	editMode = Edit;
	setWindowTitle(tr("Edit Medium"));
	setMedium(medium, newTransaction);
}

void EditMedium::setNewMode(MediaModel *model, Medium *medium, bool newTransaction)
{
	editMode = New;
	if(!model)
		return;
	this->model = model;
	
	setWindowTitle(tr("Add Medium to Database"));
	setMedium(medium, newTransaction);
}

void EditMedium::save()
{
	if(!medium)
		return;

	// as some widgets triggers mysql queries within the medium, those queries
	// must be executed immediatly and not queued to the thread.
	// Therefore thread is disabled here and enabled again later on
	medium->disableThread();
	// if startTransaction is false, the existing transaction
	// should be used, therefore startTransaction is inverted here
	foreach(MediumEditWidget *widget, editWidgets){
		if(!widget->store(!startTransaction))
			return;
	}
	medium->enableThread();
	
	switch(editMode){
		case New:
			if(!model)
				return;
			model->addMedium(medium);
			if(!medium->store(true, startTransaction))
				return;
			medium->setSource(Medium::Local);
			medium->setSourceIcon(QIcon ( ":icons/128x128/meganizer.png" ));
			break;
		case Add:
			if(!medium->store(true, startTransaction))
				return;
			medium->setSource(Medium::Local);
			medium->setSourceIcon(QIcon ( ":icons/128x128/meganizer.png" ));
			break;
		default:
			if(!medium->store(false, startTransaction))
				return;
			break;
	}
	accept();
}

void EditMedium::setManufacturerText(QString text)
{
	ui->manufacturerWidget->setLabelText(text);
}

void EditMedium::setPersonsDefaultRole(QString name)
{
	if(name.isEmpty())
		return;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;

	QSqlQuery query;
	// fetching default roleId
	query.prepare("SELECT id FROM materialpersonsroles WHERE name=:name");
	query.bindValue(":name", name);
	if(!query.exec()){
		emit information(tr("Could not execute default role statement."));
		if(closeDB)
			db.close();
		return;
	}
	if(!query.first()){
		emit information(tr("Default role %1 not found.").arg(name));
		if(closeDB)
			db.close();
		return;
	}

	ui->personsWidget->setDefaultRole(query.record().value("id").toUInt());

	if(closeDB)
		db.close();
}

void EditMedium::insertWidget(MediumEditWidget* newWidget, QString intoWidget, QString afterWidget)
{
	QLayoutItem *newItem = new QWidgetItem(newWidget);
	QWidget *into = findChild<QWidget*>(intoWidget);
	QWidget *after = NULL;
	if(!afterWidget.isEmpty())
		after = findChild<QWidget*>(afterWidget);
	if(!into)
		return;
	if(!newItem)
		return;
	// determine target layout
	QLayout *targetLayout = into->layout();
	// handling special case meta data, because it's a QVBox in QHBox
	if(into->objectName() == "metaDataTab")
		targetLayout = ui->metaDataLayout;
	if(!targetLayout)
		return;

	// determine index
	int index = targetLayout->count();
	if(!after){
		// insert it at last position BEFORE stretch
		if(targetLayout->count() > 0)
			if(targetLayout->itemAt(targetLayout->count() - 1)->spacerItem())
				index = targetLayout->count() - 1;
	}
	else{
		// if index is -1 (afterWidget is not found), the widget is appended
		index = targetLayout->indexOf(after) + 1;
	}

	QBoxLayout *la = qobject_cast<QBoxLayout*>(targetLayout);
	if(la){
		if(newItem->widget()){
			la->insertWidget(index, newItem->widget());
		}
		else if(newItem->layout()){
			la->insertLayout(index, newItem->layout());
		}
		else if(newItem->spacerItem()){
			la->insertSpacerItem(index, newItem->spacerItem());
		}
	}
	editWidgets << newWidget;
	connectByPass(newWidget);
// 	tabOrder(0, layout());
}

void EditMedium::addTab(MediumEditWidget* newWidget, QString title, QString tabWidget)
{
	if(!newWidget)
		return;
	
	if(tabWidget.isEmpty())
		return;

	if(title.isEmpty())
		return;
	
	QTabWidget *widget = findChild<QTabWidget*>(tabWidget);
	if(!widget)
		return;
	widget->addTab(newWidget, title);
	
	editWidgets << newWidget;
	connectByPass(newWidget);
	tabOrder(0, layout());
}

void EditMedium::removeTab(QString tabWidget, QString tabbedWidget)
{
	if(tabWidget.isEmpty())
		return;
	if(tabbedWidget.isEmpty())
		return;

	QTabWidget *widget = findChild<QTabWidget*>(tabWidget);
	if(!widget)
		return;
	QWidget *tabbed = findChild<QWidget*>(tabbedWidget);
	if(!tabbed)
		return;
	widget->removeTab(widget->indexOf(tabbed));
}

void EditMedium::cancel()
{
	cancel(true);
}

void EditMedium::cancel(bool cancelButton)
{
	// if the medium should be added and cancel is pressed,
	// the adding failed whether any modifications are done or not
	if(editMode == Add){
		reject();
		return;
	}
	
	// if nothing changed, we accept to avoid
	// missleading error messages
	bool changed = false;
	foreach(MediumEditWidget *widget, editWidgets){
		if(widget->modified()){
			changed = true;
			break;
		}
	}
	if(!changed){
		accept();
		return;
	}
	
	QMessageBox msgBox;
	msgBox.setText(tr("The medium has been modified."));
	msgBox.setInformativeText(tr("Do you want to save your changes?"));
	QMessageBox::StandardButtons buttons = QMessageBox::Save | QMessageBox::Discard;
	if(cancelButton)
		buttons = buttons | QMessageBox::Cancel;
	msgBox.setStandardButtons(buttons);
	msgBox.setDefaultButton(QMessageBox::Save);
	int ret = msgBox.exec();
	switch(ret){
		case QMessageBox::Save:
			save();
			return;
		case QMessageBox::Discard:
			accept();
			return;
		case QMessageBox::Cancel:
			return;
		default:
			reject();
			return;
	}
	reject();
}

void EditMedium::setMedium(Medium* medium, bool newTransaction)
{
	startTransaction = newTransaction;

	if(!medium)
		return;
	
	foreach(MediumEditWidget *widget, editWidgets)
		widget->setMedium(medium);
	
	this->medium = medium;
}

QWidget* EditMedium::tabOrder(QWidget* lastWidget, QLayout* childlayout)
{
	if(!childlayout)
		return lastWidget;
	
	for(int i = 0; i < childlayout->count(); i++){
		if(childlayout->itemAt(i)->widget()){
			QWidget *widget = childlayout->itemAt(i)->widget();
			QLabel *label = qobject_cast<QLabel*>(widget);
			if(!label){
				if(lastWidget){
					setTabOrder(lastWidget, widget);
					qDebug() << lastWidget->objectName() << " to " << widget->objectName();
				}
				lastWidget = widget;

				// handle special container widgets
				QTabWidget *tabWi = qobject_cast<QTabWidget*>(widget);
				if(tabWi){
					for(int j = 0; j < tabWi->count(); j++)
						lastWidget = tabOrder(lastWidget, tabWi->widget(j)->layout());
				}
			}
		}
		else if(childlayout->itemAt(i)->layout()){
			lastWidget = tabOrder(lastWidget, childlayout->itemAt(i)->layout());
		}
	}
	return lastWidget;
}

void EditMedium::closeEvent(QCloseEvent* event)
{
	cancel(false);
	EDialog::closeEvent(event);
}

