/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "connectmediasearch.h"
#include "connectmediasearch.moc"
#include "ui_connectmediasearch.h"

#include "mpluginloader.h"
#include "mediamodel.h"
#include "medium.h"
#include "searchplugin.h"
#include "mediaplugin.h"

#include <QSettings>

ConnectMediaSearch::ConnectMediaSearch(QWidget* parent, Qt::WindowFlags flags)
 : ESettingsWidget(parent, flags), ui(new Ui::ConnectMediaSearch)
{
	medium = NULL;
	searchPlugin = NULL;
	numMediaAttrs = 0;
	numSearchAttrs = 0;
	numMediaModelAttrs = 0;
	numSearchPluginAttrs = 0;
	canceled = false;
	deleteMedium = false;
	saved = false;
	
	setName(tr("Connect Media Attributes"));
  ui->setupUi(this);
  ui->connectionsView->setColumnCount(2);
  
  connect(ui->assignButton, SIGNAL(clicked()), this, SLOT(assign()));
  connect(ui->removeButton, SIGNAL(clicked()), this, SLOT(remove()));
  connect(ui->mediaPluginAttrsView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
  connect(ui->searchPluginAttrsView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
  connect(ui->connectionsView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
}

ConnectMediaSearch::~ConnectMediaSearch()
{
	if(deleteMedium)
		delete medium;
}

void ConnectMediaSearch::setPlugins(Medium *medium, SearchPlugin* searchPlugin)
{
  if(!medium)
    return;
  
  if(!searchPlugin)
    return;
	
	// replace medium with more specific one if needed
	if(medium->group().isEmpty() && medium->type() > 0){
		QList<MediaPlugin*> plugins = MPluginLoader<MediaPlugin>::loadPlugins();
		//load and test plugin
		foreach ( MediaPlugin* plugin, plugins ) {
			if(plugin->isSupportedMaterialType(medium->typeName())){
				medium = plugin->newMedium(medium->typeName());
				if(medium){
					deleteMedium = true;
					break;
				}
			}
		}
	}
	
	this->medium = medium;
	mediaGroup = medium->group();
	mediaName = medium->name();
	mediaMap = medium->supportedAttrs();
	QStringList translations = medium->supportedAttributesTrans();
	// strip out the standard attributes
	Medium standardMed;
	foreach(ColKeyTrans key, standardMed.supportedAttrs()){
		if(mediaMap.contains(key)){
			ColKeyTrans realKey = mediaMap[mediaMap.indexOf(key)];
			mediaMap.removeAll(realKey);
			translations.removeAll(realKey.translation);
		}
	}
	this->searchPlugin = searchPlugin;

	loadSettings();
	
	ui->mediaPluginAttrsView->clear();
	ui->searchPluginAttrsView->clear();
	ui->connectionsView->clear();
	ui->connectionsView->setRowCount(0);
	
  ui->mediaPluginNameLabel->setText(mediaName);
	translations.sort();
  ui->mediaPluginAttrsView->addItems(translations);
	numMediaModelAttrs = mediaMap.size();
  
  ui->searchPluginNameLabel->setText(searchPlugin->name());
  searchGroup = searchPlugin->settingsGroup();
	searchMap = searchPlugin->supportedAttributes();
	QStringList	items = searchMap.values();
	items.sort();
  ui->searchPluginAttrsView->addItems(items);
	numSearchPluginAttrs = items.size();
  
  QStringList header;
  header << mediaName << searchPlugin->name();
  ui->connectionsView->setHorizontalHeaderLabels(header);
	
	if(mediaAttrs.size() == searchAttrs.size() && mediaAttrs.size() > 0){
		// use configured connections and do not auto connect
		for(int i = 0; i < mediaAttrs.size(); i++)
			assign(mediaAttrs[i], searchAttrs[i]);
	}
	else{
		// do auto connect, because nothing is setup correctly
		foreach(ColKeyTrans attr, mediaMap)
			// try to connect the media attribute attr
			// with the same named search attribute
			assign(attr.key, attr.key);
	}
}

bool ConnectMediaSearch::saveSettings(QString& error)
{
  if(mediaGroup.isEmpty())
    return false;
  if(searchGroup.isEmpty())
    return false;
  
  QStringList localMediaAttrs, localSearchAttrs;
  for(int i = 0; i < ui->connectionsView->rowCount(); i++){
    QTableWidgetItem *item = 0;
    item = ui->connectionsView->item(i, 0);
    if(!item)
      return false;
		foreach(ColKeyTrans keyItem, mediaMap){
			if(keyItem.translation == item->text())
				localMediaAttrs << keyItem.key;
		}
    
    item = ui->connectionsView->item(i, 1);
    if(!item)
      return false;
    localSearchAttrs << searchMap.key(item->text());
  }
  
  QSettings settings;
  settings.beginGroup("mapping" + searchGroup + mediaGroup);
	settings.setValue("numMediaAttrs", numMediaModelAttrs);
	settings.setValue("numSearchAttrs", numSearchPluginAttrs);
  settings.setValue("mediaAttrs", localMediaAttrs);
  settings.setValue("searchAttrs", localSearchAttrs);
  settings.endGroup();
  
  return true;
}

void ConnectMediaSearch::assign()
{
	if(!mediaIndex.isValid())
		return;
	if(!searchIndex.isValid())
		return;
  
	assign(
		ui->mediaPluginAttrsView->item(mediaIndex.row()),
		ui->searchPluginAttrsView->item(searchIndex.row())
	);
}

void ConnectMediaSearch::assign(QString mediaAttr, QString searchAttr)
{
	QList<QListWidgetItem*> itemsMediaAttrs = ui->mediaPluginAttrsView->findItems(
		medium->attrs(mediaAttr).translation,
		Qt::MatchExactly
	);
	QList<QListWidgetItem*> itemsSearchAttrs = ui->searchPluginAttrsView->findItems(
		searchMap.value(searchAttr),
		Qt::MatchExactly
	);
	if(itemsMediaAttrs.size() == 1 && itemsSearchAttrs.size() == 1)
		assign(itemsMediaAttrs[0], itemsSearchAttrs[0]);
}

void ConnectMediaSearch::assign(QListWidgetItem* mediaItem, QListWidgetItem* searchItem)
{
	if(!mediaItem)
		return;
	if(!searchItem)
		return;

	ui->connectionsView->setSortingEnabled(false);
	int row = ui->connectionsView->rowCount();
	ui->connectionsView->insertRow(row);
	ui->connectionsView->setItem(row, 0, new QTableWidgetItem(mediaItem->text()));
	ui->connectionsView->setItem(row, 1, new QTableWidgetItem(searchItem->text()));
	ui->connectionsView->setSortingEnabled(true);

	delete searchItem;
	delete mediaItem;
}

void ConnectMediaSearch::remove()
{
  if(!connectionIndex.isValid())
    return;
  
  // add the remove attrs to the list of media model and search plugin
  QTableWidgetItem *item = 0;
  item = ui->connectionsView->item(connectionIndex.row(), 0);
  if(!item)
    return;
  ui->mediaPluginAttrsView->addItem(item->text());
  
  item = ui->connectionsView->item(connectionIndex.row(), 1);
  if(!item)
    return;
  ui->searchPluginAttrsView->addItem(item->text());
  
  ui->connectionsView->removeRow(connectionIndex.row());
}

void ConnectMediaSearch::indexChanged(QModelIndex index)
{
  if(!sender())
    return;
  if(sender()->objectName() == "mediaPluginAttrsView"){
    mediaIndex = index;
    return;
  }
  if(sender()->objectName() == "searchPluginAttrsView"){
    searchIndex = index;
    return;
  }
  if(sender()->objectName() == "connectionsView"){
    connectionIndex = index;
    return;
  }
}

bool ConnectMediaSearch::checkSettings()
{
	if(!medium)
    return false;
  
  if(!searchPlugin)
    return false;
	
	loadSettings();
	// do some checks, if we must display the dialog
	bool result = true;
	// check wether the attribute number has changed
	if(numMediaAttrs != numMediaModelAttrs)
		result = false;
	if(numSearchAttrs != numSearchPluginAttrs)
		result = false;
	
	// check that all listed attributes are supported by the medium and plugin right now
	for(int i = 0; i < mediaAttrs.size(); i++){
		if(medium->supportedAttributes().indexOf(mediaAttrs[i]) == -1)
			result = false;
	}
	for(int i = 0; i < searchAttrs.size(); i++){
		if(searchPlugin->supportedAttributes().keys().indexOf(searchAttrs[i]) == -1)
			result = false;
	}

	if ( mediaAttrs.size() != searchAttrs.size() )
		result = false;
	
	return result;
}

void ConnectMediaSearch::loadSettings()
{
	if(!medium)
		return;
	
	if(!searchPlugin)
		return;
	
	// loading status quo
	QSettings settings;
	settings.beginGroup ( "mapping" + searchPlugin->settingsGroup() + mediaGroup );
	numMediaAttrs = settings.value("numMediaAttrs", 0).toInt();
	numSearchAttrs = settings.value("numSearchAttrs", 0).toInt();
	mediaAttrs = settings.value ( "mediaAttrs" ).toStringList();
	searchAttrs = settings.value ( "searchAttrs" ).toStringList();
	settings.endGroup();
}
