/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "loanitem.h"
#include "loanitem.moc"

#include <esqldatabase.h>

#include <QSqlQuery>
#include <QSqlError>
#include <QVariant>

LoanItem::LoanItem ( QObject* parent ) : EObject ( parent )
{
	materialId = 0;
	ownerId = 0;
	recipientId = 0;
	statusId = 0;
	rowId = 0;
	loggedInUserId = 0;
	setModified(false);
}

bool LoanItem::load ( QSqlRecord record )
{
	if(record.isEmpty())
		return false;
	
	setMaterial(record.value("materialsId").toUInt());
	myMaterialTitle = record.value("title").toString();
	// don't set owneruserName, because we know him already
	// and set him afterwards
	setOwner(record.value("usersId").toUInt(), false);
	setOwnerUserName(record.value("userName").toString());
	setRecipient(record.value("recipientId").toUInt(), true);
	setStatus(record.value("status").toUInt(), true);
	setLoanedUntil(record.value("loanedUntil").toDate(), true);
	setLastChange(record.value("lastChange").toDate());
	setRow(record.value("id").toUInt());
	
	setModified(false);
	
	if(!isValid())
		return false;
	return true;
}

bool LoanItem::store()
{
	if(!modified())
		return true;

	if(!isValid()){
		emit error(tr("The choosen data combination is not valid!"));
		return false;
	}
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(!db.isOpen())
		return false;
	
	QSqlQuery query;
	if(row() == 0){
		// row not present, so we insert it
		query.prepare("INSERT INTO meganizer_loanedmaterials (materialsId, ownerId, recipientId, status, loanedUntil)"
									"  VALUES(:materialsId, :ownerId, :recipientId, :status, :loanedUntil)");
		query.bindValue(":materialsId", material());
		query.bindValue(":ownerId", owner());
		query.bindValue(":recipientId", recipient());
		query.bindValue(":status", status());
		query.bindValue(":loanedUntil", loanedUntil());
	}
	else{
		query.prepare("UPDATE meganizer_loanedmaterials SET status=:status, loanedUntil=:loanedUntil WHERE id=:id");
		query.bindValue(":status", status());
		query.bindValue(":loanedUntil", loanedUntil());
		query.bindValue(":id", row());
	}
	bool result = query.exec();
	if(closeDB)
		db.close();
	
	if(!result){
		emit error(tr("Could not store loan item\nQuery: %1\nError: %2")
							.arg(query.lastQuery())
							.arg(query.lastError().text()));
		return false;
	}
	
	return true;	
}

bool LoanItem::modified() const
{
	return isModified;
}

bool LoanItem::isValid() const
{
	if(material() == 0)
		return false;

	if(owner() == 0)
		return false;

	if(recipient() == 0)
		return false;

	if(owner() == recipient())
		return false;

	return true;
}

bool LoanItem::loanFinished() const
{
	switch(status()){
		case 2:
			return true;
		case 5:
			return true;
		case 6:
			// status 6 can only be reached, if the previous status was loan request
			// therefore status 6 is the end of an transaction
			return true;
	}
	return false;
}

void LoanItem::setLoggedInUser ( unsigned int id )
{
	loggedInUserId = id;
}

unsigned int LoanItem::loggedInUser()
{
	if(loggedInUserId == 0){
		// set the user id, if none is set already
		ESqlDatabase db = ESqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			db.open();
		if(db.isOpen()){
			QSqlQuery query;
			query.prepare("SELECT id FROM meganizer_users WHERE userName = :userName");
			query.bindValue(":userName", db.userName());
			query.exec();
			query.first();
			loggedInUserId = query.record().value("id").toUInt();
			if(closeDB)
				db.close();
		}
	}
	return loggedInUserId;
}

void LoanItem::setMaterial ( unsigned int id )
{
	materialId = id;
	setModified();
}

unsigned int LoanItem::material() const
{
	return materialId;
}

QString LoanItem::materialTitle() const
{
	return myMaterialTitle;
}

void LoanItem::setOwner ( unsigned int id, bool setOwnerName )
{
	ownerId = id;
	if(setOwnerName){
		setOwnerUserName(QString());
		ESqlDatabase db = ESqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			db.open();
		if(db.open()){
			QSqlQuery query;
			query.prepare("SELECT userName FROM meganizer_users WHERE id=:id");
			query.bindValue(":id", ownerId);
			query.exec();
			if(query.size() == 1)
				if(query.first())
					setOwnerUserName(query.record().value("userName").toString());
			if(closeDB)
				db.close();
		}
	}
	setModified();
}

unsigned int LoanItem::owner() const
{
	return ownerId;
}

void LoanItem::setOwnerUserName ( QString userName )
{
	myOwnerUserName = userName;
}

QString LoanItem::ownerUserName() const
{
	return myOwnerUserName;
}

bool LoanItem::setRecipient ( unsigned int id, bool ignoreRights )
{
	// only owner can set the recipient
	// the exception is, if the owner is not a database user
	if(!ignoreRights && (loggedInUser() != owner() && !ownerUserName().isEmpty()))
		return false;
	
	// we cannot lone to ourself
	if(owner() == id && !ignoreRights)
		return false;
	
	recipientId = id;
	setModified();
	return true;
}

unsigned int LoanItem::recipient() const
{
	return recipientId;
}

QList< unsigned int > LoanItem::validStatis()
{
	QList<unsigned int> result;
	// loggedInUser has nothing to do with this loan
	if(loggedInUser() != owner() && loggedInUser() != recipient()){
		if(recipient() != 0){
			// because the recipient is set, it is an active loan
			// and access is denied
			return result;
		}
		else{
			// so recipient is not set, therefore loan request must be available
			switch(status()){
				case 0:
					result << 1;
			}
			if(ownerUserName().isEmpty()){
				// since owner does not use database, also all other should be available
				switch(status()){
					case 0:
						// loan request (1) is added above already
						// but also loan denied and loaned can be accessed
						result << 2 << 3;
						break;
					case 1:
						// loan denied, loaned and cancel request may be used
						result << 2 << 3 << 6;
						break;
					case 3:
						// return request and returned are possible
						result << 4 << 5;
						break;
					case 4:
						// returned and cancel return request are possible
						result << 5 << 6;
						break;
				}
			}
			return result;
		}
	}
	
	// owner does changes
	if(loggedInUser() == owner()){
		switch(status()){
			case 0:
				// the owner can deny loaning (may be some asked him per phone or something)
				// and he can loan it for same reason
				result << 2 << 3;
				return result;
			case 1:
				// the owner can deny loaning and he can loan it
				result << 2 << 3;
				return result;
			case 3:
				// owner can request return and can mark as returned
				result << 4 << 5;
				return result;
			case 4:
				// owner can mark as returned or cancle return request
				result << 5 << 6;
				return result;
			default:
				return result;
		}
	}
	
	// recipient does changes
	if(loggedInUser() == recipient()){
		switch(status()){
			case 0:
				// recipient can ask for loaning
				result << 1;
				break;
			case 1:
				// recip can cancel request
				result << 6;
				break;
		}
		if(ownerUserName().isEmpty()){
			// since owner is not a database user
			// recipient must be able to handle all stuff
			switch(status()){
				case 0:
					result << 2 << 3;
					break;
				case 3:
					result << 4 << 5;
					break;
				case 4:
					result << 5 << 6;
					break;
			}
		}
		return result;
	}
	
	return result;
}

bool LoanItem::setStatus ( unsigned int id, bool ignoreRights )
{
	if(!ignoreRights){
		QList<unsigned int> allowedStati = validStatis();
		if(!allowedStati.contains(id))
			return false;
	}
	// new stati is allowed, so check wether recipient is set
	// and set it to current user, if needed
	if(recipient() == 0)
		if(!setRecipient(loggedInUser()) && !ignoreRights)
			return false;
		
	// if the Return request is canceled, we must go back to state before
	// which was loaned
	if((id == 6) && (status() == 4))
		id = 3;
	
	statusId = id;
	setModified();
	return true;
}

unsigned int LoanItem::status() const
{
	return statusId;
}

bool LoanItem::setLoanedUntil ( QDate until, bool ignoreRights )
{
	if(!ignoreRights && loggedInUser() != owner() && !(ownerUserName().isEmpty() && loggedInUser() == recipient()))
		return false;
	myLoanedUntil = until;
	setModified();
	return true;
}

QDate LoanItem::loanedUntil() const
{
	return myLoanedUntil;
}

void LoanItem::setLastChange ( QDate changeTime
 )
{
	myLastChange = changeTime;
	setModified();
}

QDate LoanItem::lastChange() const
{
	return myLastChange;
}

bool LoanItem::operator== ( const LoanItem& item ) const
{
	if((row() != 0) || (item.row() != 0)){
		if(row() == item.row())
			return true;
		return false;
	}
	
	if(material() == item.material())
		if(owner() == item.owner())
			if(recipient() == item.recipient())
				return true;
	return false;
}

bool LoanItem::operator!= ( const LoanItem& item ) const
{
	return !operator==(item);
}

void LoanItem::setModified(bool mod)
{
	isModified = mod;
	if(isModified)
		emit changed();
}

void LoanItem::setRow ( unsigned int id )
{
	rowId = id;
	setModified();
}

unsigned int LoanItem::row() const
{
	return rowId;
}
