/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "loanmodel.h"
#include "loanmodel.moc"

#include "loanitem.h"
#include "delegateloanstati.h"
#include "delegateuser.h"
#include <esqldatabase.h>
#include "medium.h"

#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QTableView>
#include <QAbstractItemView>

LoanModel::LoanModel ( Medium *medium, unsigned int ownUserId, QObject* parent )
	: EAbstractItemModel ( parent )
{
	init(medium, ownUserId);
}

LoanModel::LoanModel ( unsigned int ownUserId, QObject* parent )
	: EAbstractItemModel ( parent )
{
	init(NULL, ownUserId, true);
}

LoanModel::~LoanModel()
{
	qDeleteAll(items);
}

int LoanModel::indexOf ( QString columnName ) const
{
	return headerList.indexOf(columnName);
}

LoanItem* LoanModel::loanItemFromIndex ( QModelIndex index ) const
{
	if(!index.isValid())
		return 0;
	return static_cast<LoanItem*>(index.internalPointer());
}

bool LoanModel::store()
{
	bool result = true;
	foreach(LoanItem *item, items){
		if(!item->store())
			result = false;
	}
	return result;
}

bool LoanModel::reload()
{
	return true;
}

bool LoanModel::clear()
{
	emit layoutAboutToBeChanged();
	items.clear();
	emit layoutChanged();
	if(items.size() == 0)
		return true;
	return false;
}

void LoanModel::addItemDelegates ( QAbstractItemView* view )
{
	if(!view)
		return;

	int add = 0;
	if(loanSummaryMode)
		add = 1;
	DelegateLoanStati *statusDelegate = new DelegateLoanStati(view);
	view->setItemDelegateForColumn(2 + add, statusDelegate);
	DelegateUser *userDelegate = new DelegateUser(view);
	view->setItemDelegateForColumn(0 + add, userDelegate);
	view->setItemDelegateForColumn(1 + add, userDelegate);
}

void LoanModel::hideAndResizeColumns ( QTableView* view )
{
	if(!view)
		return;
	view->resizeColumnsToContents();
}

QVariant LoanModel::data ( const QModelIndex& index, int role ) const
{
	if(role != Qt::DisplayRole
			&& role != Qt::EditRole
			&& role != Qt::BackgroundRole
		)
		return QVariant();
	
	LoanItem *item = loanItemFromIndex(index);
	if(!item)
		return QVariant();
	
	if(role == Qt::BackgroundRole){
		QDate dueDate = item->loanedUntil();
		if(dueDate.isValid() && dueDate < QDate::currentDate())
			return QBrush(QColor ( 244, 12, 13, 60 ));
		return QBrush();
	}

	int column = index.column();
	if(loanSummaryMode){
		if(column == 0)
			return item->materialTitle();
		column--;
	}
		
	switch(column){
		case 0:
			return item->owner();
		case 1:
			return item->recipient();
		case 2:
			return item->status();
		case 3:
			return item->loanedUntil();
		case 4:
			return item->lastChange();
	}
	return QVariant();
}

int LoanModel::columnCount ( const QModelIndex& parent ) const
{
	if(parent.isValid())
		return 0;

	return headerList.size();
}

int LoanModel::rowCount ( const QModelIndex& parent ) const
{
	if(parent.isValid())
		return 0;
	
	return items.size();
}

QModelIndex LoanModel::parent ( const QModelIndex& child ) const
{
	Q_UNUSED(child);
	return QModelIndex();
}

QModelIndex LoanModel::index ( int row, int column, const QModelIndex& parent ) const
{
	if(parent.isValid())
		return QModelIndex();
	
	if(column < 0)
		return QModelIndex();
	if(columnCount() <= column)
		return QModelIndex();
	
	if((row < 0) || (items.size() <= row))
		return QModelIndex();
	
	return createIndex(row, column, items[row]);
}

bool LoanModel::setData ( const QModelIndex& index, const QVariant& value, int role )
{
  if(role != Qt::EditRole)
		return false;
	
	LoanItem *item = loanItemFromIndex(index);
	if(!item)
		return false;

	int column = index.column();
	if(loanSummaryMode)
		column--;

	switch(column){
		case 1:
			item->setRecipient(value.toUInt());
			return true;
		case 2:
			return item->setStatus(value.toUInt());
		case 3:
			return item->setLoanedUntil(value.toDate());
	}
	return false;
}

QVariant LoanModel::headerData ( int section, Qt::Orientation orientation, int role ) const
{
	if(role != Qt::DisplayRole)
		return QVariant();
	
  if(orientation == Qt::Vertical)
		return section + 1;
	
	if((section < 0) || (headerList.size() <= section))
		return QVariant();
	
	return headerList[section];
}

Qt::ItemFlags LoanModel::flags ( const QModelIndex& index ) const
{
	LoanItem *item = loanItemFromIndex(index);
	if(!item)
		return QAbstractItemModel::flags(index);

	int column = index.column();
	if(loanSummaryMode)
		column--;
	switch(column){
		case 1:
			return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
		case 2:
			return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
		case 3:
			if(item->owner() == myUserId || item->ownerUserName().isEmpty())
				return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
			else
				return QAbstractItemModel::flags(index);
	}
	return QAbstractItemModel::flags(index);
}

void LoanModel::sort ( int column, Qt::SortOrder order )
{
  QAbstractItemModel::sort ( column, order );
}

void LoanModel::itemChanged()
{
	emit dataChanged ( index ( 0, 0 ), index ( rowCount(), columnCount() ) );
}

void LoanModel::init(Medium *medium, unsigned int ownUserId, bool loadWithoutMaterial)
{
	loanSummaryMode = false;
	this->medium = medium;
	myUserId = ownUserId;
	if(myUserId == 0){
		ESqlDatabase db = ESqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			db.open();
		if(db.isOpen()){
			QSqlQuery query;
			query.prepare("SELECT id FROM meganizer_users WHERE userName=:userName");
			query.bindValue(":userName", db.userName());
			query.exec();
			if(query.size() == 1)
				if(query.first())
					myUserId = query.record().value("id").toUInt();
			if(closeDB)
				db.close();
		}
	}

	if(!medium && myUserId > 0 && loadWithoutMaterial)
		loanSummaryMode = true;
	
	setupHeader();
	load();
}

void LoanModel::load()
{
	// a user id is needed in every case
	if(myUserId < 1)
		return;
	
	if(!loanSummaryMode){
		// so we're not in loanSummary mode.
		// Therefore we need a medium with a valid id
		if(!medium)
			return;
		if(medium->id() < 1)
			return;
	}

	QString whereClause;
	if(loanSummaryMode){
		whereClause = QString("WHERE (mats.usersId=%1 OR loaned.recipientId=%1)"
													"  AND loaned.id<>0"
													"  AND (loaned.status<>5 AND loaned.status<>2 AND loaned.status<>6)").arg(myUserId);
	}
	else{
		whereClause = QString("WHERE mats.id=%1").arg(medium->id());
		// this is needed, to get really that medium, we want and the same medium
		// owned by another user
		if(medium->user() > 0)
			whereClause += QString(" AND mats.usersId=%1").arg(medium->user());
	}

	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(!db.isOpen())
		return;
	
	emit layoutAboutToBeChanged();
	QSqlQuery query;
	query.exec(QString("SELECT mats.id AS materialsId, mats.title, mats.usersId, users.userName,"
													" loaned.id, loaned.recipientId, loaned.status, loaned.loanedUntil, loaned.lastChange"
													" FROM viewmaterials mats"
													" LEFT JOIN meganizer_loanedmaterials loaned ON mats.id=loaned.materialsId"
													" LEFT JOIN meganizer_users users ON mats.usersId=users.id"
													" %1").arg(whereClause));
	while(query.next()){
		LoanItem *item = new LoanItem(this);
		item->setLoggedInUser(myUserId);
		connectByPass(this, item);
		connect(item, SIGNAL(changed()), this, SLOT(itemChanged()));

		if(!item->load(query.record())){
			// this is at least the case, if
			// the medium wasn't loaned before
			delete item;
		}
		else{
			// only add the item, if loan is not finished
			if(item->loanFinished())
				delete item;
			else
				items << item;
		}
	}

	if(!loanSummaryMode && items.size() == 0){
		// this medium isn't loaned actually,
		// so add an empty loan item to allow
		// the user to start a new loan process
		LoanItem *item = new LoanItem(this);
		item->setMaterial(medium->id());
		item->setOwner(medium->user());
		item->setLoggedInUser(myUserId);
		// set the recipient to our own user,
		// if the medium isn't our own
		if(medium->user() != myUserId)
			item->setRecipient(myUserId, true);
		else
			item->setRecipient(0, true);
		item->setStatus(0, true);
		item->setLoanedUntil(QDate());
		item->setLastChange(QDate());
		// we set it to not modified, so that
		// this empty entry, doesn't return false
		// in store function
		item->setModified(false);
		items << item;
	}

	emit layoutChanged();
	if(closeDB)
		db.close();
}

void LoanModel::setupHeader()
{
	if(loanSummaryMode)
		headerList << tr ( "Medium" );
	headerList << tr ( "Owner" ) << tr ( "Loaner" );
	headerList << tr ( "Status" ) << tr ( "Due Date" );
	headerList << tr ( "Last Change" );
}
