/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "materialpersonsmodel.h"
#include "materialpersonsmodel.moc"

#include "materialperson.h"
#include <edelegateforeignkey.h>
#include <esqldatabase.h>
#include <equerythread.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QColor>
#include <QBrush>
#include <QTableView>

MaterialPersonsModel::MaterialPersonsModel(EQueryThread *thread, QObject* parent)
	: PersonsModel(thread, parent)
{
	maxCombinedPersons = 3;
}

Person* MaterialPersonsModel::addPerson()
{
	MaterialPerson *newP = new MaterialPerson(thread, this);
	addPerson(newP);
	return newP;
}

bool MaterialPersonsModel::addCopyOfPerson(MaterialPerson* sourcePerson)
{
	MaterialPerson *newPerson = qobject_cast<MaterialPerson*>(addPerson());
	if(!newPerson)
		return false;
	newPerson->setGivenName(sourcePerson->givenName());
	newPerson->setName(sourcePerson->name(), false);
	newPerson->setId(sourcePerson->id(), false);
	newPerson->setRole(sourcePerson->role(), false);
	newPerson->setRole(sourcePerson->roleName(), false);
	return true;
}

MaterialPerson* MaterialPersonsModel::addPerson(QString name, QString givenName, QString role)
{
  MaterialPerson *newPerson = qobject_cast<MaterialPerson*>(PersonsModel::addPerson(name, givenName));
	if(!newPerson)
		return 0;
  newPerson->setRole(role);
	return newPerson;
}

MaterialPerson* MaterialPersonsModel::addPerson(QString name, QString givenName, uint roleId)
{
  MaterialPerson *newPerson = qobject_cast<MaterialPerson*>(PersonsModel::addPerson(name, givenName));
  if(!newPerson)
		return 0;
	newPerson->setRole(roleId);
  return newPerson;
}

bool MaterialPersonsModel::addPerson(uint personId, uint roleId, uint rowId)
{
  MaterialPerson *newPerson = qobject_cast<MaterialPerson*>(PersonsModel::addPerson(personId));
  if(!newPerson)
    return false;
  newPerson->setRole(roleId);
	newPerson->setRowId(rowId);
	if(personId != newPerson->id())
		return false;
  return true;
}

bool MaterialPersonsModel::addPerson(QSqlRecord record)
{
	return addPerson(record.value("materialPersonsId").toUInt(),
						record.value("materialPersonsRolesId").toUInt(),
						record.value("id").toUInt());
}

bool MaterialPersonsModel::store(uint materialId)
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return false;

  bool result = true;
  foreach(Person *person, items){
		MaterialPerson *matP = qobject_cast<MaterialPerson*>(person);
		if(!matP)
			continue;
    if(!matP->store(materialId))
      result = false;
  }
  if(closeDB)
    db.close();
  return result;
}

bool MaterialPersonsModel::loadAllPersons(uint materialId)
{
	QString queryString = QString("SELECT * FROM meganizer_materialhaspersons WHERE materialsId=%1 ORDER BY id").arg(materialId);
	if(thread){
		thread->execQuery(this, "loadAllPersons", queryString, tr("Could not load persons."));
		return true;
	}
	else{
		ESqlDatabase db = ESqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			if(!db.open())
				return false;
		bool result = true;
		QSqlQuery query;
		query.exec(queryString);
		while(query.next()){
			if(!addPerson(query.record()))
				result = false;
		}
		if(closeDB)
			db.close();
		return result;
	}
}

void MaterialPersonsModel::addItemDelegates(QAbstractItemView* view)
{
	if(!view)
		return;

	view->setItemDelegateForColumn(2, new EDelegateForeignKey("meganizer_materialpersonsroles", "id", "name", view));
}

MaterialPersonsModel* MaterialPersonsModel::copyModel()
{
	MaterialPersonsModel *newModel = new MaterialPersonsModel();
	foreach(Person *person, items){
		MaterialPerson *matP = qobject_cast<MaterialPerson*>(person);
		if(!matP)
			continue;
		MaterialPerson *tmp = matP->copyPerson();
		newModel->addPerson(tmp);
	}
	return newModel;
}

QVariant MaterialPersonsModel::data(const QModelIndex& index, int role) const
{
  MaterialPerson *matPerson = qobject_cast<MaterialPerson*>(personFromIndex(index));
	if(!matPerson)
		return QVariant();
  
	if(role != Qt::DisplayRole)
		return PersonsModel::data(index, role);
  
  switch(index.column()){
    case 2:
			if(matPerson->role() > 0)
				return matPerson->role();
      return matPerson->roleName();
  }
  return PersonsModel::data(index, role);
}

bool MaterialPersonsModel::setData(const QModelIndex& index, const QVariant& value, int role){
	if(role != Qt::EditRole)
		return false;
	
	MaterialPerson *person = qobject_cast<MaterialPerson*>(personFromIndex(index));
	if(!person)
		return false;
	
	switch(index.column()){
		case 2:
			person->setRole(value.toUInt(), false);
			return true;
	}
	return false;
}

int MaterialPersonsModel::columnCount(const QModelIndex& parent) const
{
	Q_UNUSED(parent);
  return 3;
}

QVariant MaterialPersonsModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if((role != Qt::DisplayRole) && (role != Qt::EditRole))
      return PersonsModel::headerData(section, orientation, role);
    
    if(orientation == Qt::Vertical)
      return section + 1;
    
    switch(section){
      case 0:
				return tr("Name");
      case 1:
				return tr("Given Name");
      case 2:
				return tr("Role");
    }
    return PersonsModel::headerData(section, orientation, role);
}

Qt::ItemFlags MaterialPersonsModel::flags(const QModelIndex& index) const
{
  if(index.column() == 2)
    return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
  
  return QAbstractItemModel::flags(index);
}

void MaterialPersonsModel::customQueryResult(QVariant requestId, int querySize, QSqlRecord record)
{
	Q_UNUSED(querySize);
	if(requestId.toString() == "loadAllPersons")
		addPerson(record);
}

