/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIAMODEL_H
#define MEDIAMODEL_H

#include "libmeganizer.h"

#include <eabstractitemmodel.h>

#include "medium.h"
#include "sqlconstraint.h"

class EQueryThread;
class SearchPlugin;
class SearchDetailsWidget;

#include <QTimer>

class QAbstractItemView;
class QTableView;
class QSqlRecord;
class QTextTable;
class QMenu;

struct PreparedConstraints {
	QList<SearchPlugin*> lastSearchPlugins;
	QList<SqlConstraint> rawConstraints;
	QStringList strings;
	QList<QVariant> bindValues;
	QString table;
	QString order;
	bool usedDetailsWidget;
	bool wish;
	bool projects;
	bool gifts;
	bool userNotNeeded;
	bool searchExternal;
	unsigned int actualUserId;
};

/**
  * @brief The base class of all models holding media.
	*
	* It is also used to handle the "All" types functionality
	* To create a subclass, you normally want to overload
	* the functions (Functions in () are optional and normally not needed)
	* @li newMedium() to create a new instance of your Medium type
	* @li (loadChilds(Medium*), if this model should be used as a child model)
	* @li createNewModel() to create a new instance of your MediaModel (see note below)
	* @li addItemDelegates() to add your custom item delegates to views, which displays this model
	* @li addCustomContextItems() to add custom items to the context menu
	* @li (doAddMedium() to do some special stuff on new media)
	*
	* @note createNewModel() should be implemented only to support Qt < 4.5. In newer versions,
	*    the MediaModel creates a new instance of the actuall instanciated class with help of the
	*    QMetaObject system. Therefore make sure, you place a Q_INVOKEABLE before your constructor
	*    like done in this class. Summarized, the following code should be used:
	*    @code
	*      Q_INVOKEABLE YourMediaModel(EQueryThread *queryThread, QObject *parent = NULL){ setupModel(queryThread); };
	*      #if QT_VERSION < 0x040500
	*      MediaModel *createNewModel(){ return new YourMediaModel(NULL); };
	*      #endif
	*    @endcode
	*
	* @see Medium
  */
class LIBMEGANIZER_EXPORT MediaModel : public EAbstractItemModel
{
	Q_OBJECT
  public:
		/**
			* @brief Constructor for usage with subclasses.
			*
			* call setupModel() at the constructor of the subclass
			*/
    MediaModel ( QObject *parent = 0 );
		/**
			* @brief This constructor must be used, if this Model should be used directly
			*/
		MediaModel ( EQueryThread *queryThread, QObject *parent = 0 );
    virtual ~MediaModel();

    // own API
		QString group();
		QString name();
		void setThread(EQueryThread *thread);
		/**
			* used by AddMedium to create a new model of the same type
			* to let the user search for a item he wants to add
			* @note should be reimplemented by the subclassed model, to create the right model,
			*       because the default implementation creates a pure MediaModel
			* @see AddMedium
			* @see InsertManualMedia
			*/
		virtual MediaModel *createNewModel();
    /**
    	* @brief Creates a new instance of a medium, which isn't added to the model.
			* 
    	* To allow the user to select a special medium (if it is a plain MediaModel),
			* use newSpecialMedium().
    	*/
    virtual Medium *newMedium() const;
		/**
			* @brief Adds a new medium of a special media type.
			*
			* If it is a subclass of MediaModel it simply calls newMedium(),
			* if not and @p type is empty, the user can decide which type of material the new
			* medium should have. Afterwards a new medium of the selected
			* type is returned. To set the wanted type without a dialog, @p type must
			* contain the translated type name (See MediaPlugin).
			* 
			* If the user should not be able to select a special type,
			* use newMedium() instead.
			*/
		Medium *newSpecialMedium(QString type = QString()) const;
    /**
    	* adds a new medium to the model
    	* @attention Don't forget to emit the layoutAboutToBeChanged() and layoutChanged() signals
    	*/
    Medium *addEmptyMedium();
    bool addMedium ( unsigned int materialsId );
    bool addMedium ( Medium *medium );
    Medium *mediumFromIndex ( QModelIndex index ) const;
		QModelIndex indexFromMedium(Medium *medium);
		bool removeMedium(Medium *medium);
		bool removeMedium(QModelIndex index);
		/**
			* @brief Marks medium to be removed.
			*/
		bool markForRemoval(QModelIndex index);
    bool store();
		/**
			* @brief Used to store all media as childs of @p parent.
			*
			* It is called within Medium::store(bool, bool), if Medium::childs()
			* returns a valid model.
			*
			* It mostly calls Medium::storeChild(Medium*) for each media.
			* But it also handles some standard stuff, like opening database connection,
			* skip media, which are marked for removal and are not in the database already
			* or displaying the progress bar. It also makes sure, that the medium has a valid
			* id (means it is already in the database).
			* To customize the storing procedure, overload
			* Medium::storeChild(Medium*).
			*/
		bool store(Medium* parent);
    /**
      * searches media in local database and with help of SearchPlugin's
      * @param userId > 0 searches local database for media of user userId
      * @param userId == 0 means default order (own media, other media, external)
      * @param userId -1 means all local media
			* @param userId -2 means external
      */
    virtual bool load(QList<SqlConstraint> constraints, QList<SearchPlugin*> plugins,
											int userId, SearchDetailsWidget *searchDetailsWidget = 0);
		/**
			* @brief Loads all media, which have @p parent as a parent.
			*
			* It is called in Medium::select(QSqlRecord), if Medium::childs() returns
			* a valid pointer.
			*
			* To handle your special stuff, do something like that:
			* @code
			* ESqlDatabase db = ESqlDatabase::database();
			* bool closeDB = !db.isOpen();
			* if ( closeDB )
			*  if( !db.open() )
			*    return false;
			* if(!MediaModel::loadChilds(parent)){
			*   if(closeDB)
			*     db.close();
			*   return false;
			* }
			* // load special stuff
			* foreach(Medium *med, media){
			*   // load your stuff here
			* }
			* if(closeDB)
			*  db.close();
			* return true;
			* @endcode
			*/
		virtual bool loadChilds(Medium *parent);
		/**
			* reloads the model with calling load with the parameters stored as last...
			* @todo the reloading isn't working with CD's (Books and Tracks is no problem)
			*/
		bool reload();
		/**
			* removes all items from the model
			*/
		bool clear();
		/**
			* this function set's the selection behavior (SelectRows) and modes (ExtendedSelection)
			* afterwards, it calls addItemDelegates() and if the view is an QTableView,
			* hideAndResizeColums() is called.
			*/
		virtual void configureView(QAbstractItemView *view);
    virtual void addItemDelegates(QAbstractItemView *view);
		void hideAndResizeColumns(QTableView *view);
		/**
			* @brief Called by the viewer class to extend the context menu with type specific stuff.
			* 
			* Just add your EActions and QMenus to the given @p contextMenu and connect them to
			* apropriate slots. You should store the @p selection to know the selected items
			* later on. They are known at this time already and can not be changed, because
			* the context menu is a modal dialog.
			*/
		virtual void addCustomContextItems(QMenu *contextMenu, QModelIndexList selection);

    // QT API
    QVariant data ( const QModelIndex& index, int role = Qt::DisplayRole ) const;
    int columnCount ( const QModelIndex& parent = QModelIndex() ) const;
    int rowCount ( const QModelIndex& parent = QModelIndex() ) const;
    QModelIndex parent ( const QModelIndex& child ) const;
    QModelIndex index ( int row, int column, const QModelIndex& parent = QModelIndex() ) const;
    bool hasChildren ( const QModelIndex& parent = QModelIndex() ) const;
    bool setData ( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
    QVariant headerData ( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
    virtual Qt::ItemFlags flags ( const QModelIndex& index ) const;
    void sort ( int column, Qt::SortOrder order = Qt::AscendingOrder );
		
	public slots:
		bool addMedium ( QSqlRecord record );
		
	protected slots:
		/**
			* this function is connect with the EQueryThread::finished(bool) signal
			* If @p lastStepResult is false or the model does not have any rows,
			* the search should be continued under regards of the lastSearchConstraints
			*/
		void searchOtherUsers(QObject* client, QVariant requestId, bool lastStepResult);
		void searchSearchPlugins(QObject* client, QVariant requestId, bool lastStepResult);
		
		void showStatusWidgetByPass ( QWidget *widget );

  protected:
		void setupModel(EQueryThread *queryThread = NULL);
		void connectMedium(Medium *object);
		/**
			* @brief Used to do all needed stuff to add an medium to the list of media
			*
			* It emits layoutAboutToBeChanged, add it to the QList etc.
			* You may want to subclass it, to do some special stuff on new media
			* (like TracksModel) does. But don't forget to call the base implementation.
			*/
		virtual void doAddMedium(Medium *med);

		
    QList<Medium*> media;
		EQueryThread *thread;
		bool ownThread;
    /**
      * the condition to only load the handled media
      */
    SqlConstraint mediaCondition;
 		/**
			* a list of columns which should be hidden (index of column)
			*/
		QList<int> hideColumns;
		/**
			* the index of the column, which should be use the whole available
			* space after resizing all other columns to content
			*/
		int biggestColumn;
		QStringList headerList;

  private slots:
		void addMedium ( QObject *client, QVariant requestId, int querySize, QSqlRecord record );
    void itemChanged(Medium *med);
		void setupThread();
		void threadFinished();

	private:
		void initMediaModel(bool compatibilityMode);
		
		/**
			* it's the search mode id (this can be a userId, if > 0)
			*/
		int lastUserId;
		PreparedConstraints lastLoad;
		QString myName, myGroup;
		bool compatibilityMode, isLoading;
		QTimer finishedTimer;
    
  signals:
		void showStatusWidget ( QWidget* widget );
		void finished();
};

#endif // MEDIAMODEL_H
