/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIAPLUGIN_H
#define MEDIAPLUGIN_H

#include "libmeganizer.h"
#include "meganizerplugin.h"

class EWidget;
class ESettingsModelItem;
class ESetupWizardObject;
class SearchPlugin;
class MediaModel;
class Medium;
class SearchDetailsWidget;
class EQueryThread;

class QAbstractItemModel;
class QStringList;

/**
	* @brief An interface for all plugins, providing media types.
	*
	* In Meganizer different types of media are handled by a subclass of Medium
	* with a related subclass of MediaModel. But this two plain classes
	* are not enough. To integrate this new media types in Meganizer, create
	* a new MediaPlugin. A MediaPlugin can handle several media types, but they
	* should be splitted up into different plugins, if they are not belongs to each
	* other (like Music devices and tracks does for example).
	*
	* To create your own type, start to implement this interface
	* and create a subclass of Medium and a subclass of MediaModel,
	* which handles your specific Medium.
	* To do so, implement the MediaPlugin with help of
	* the following base code:
	* @code
	* class YourMediaPlugin : public MediaPlugin
	* {
	*   Q_OBJECT
	*   // Both interfaces are needed
	*   Q_INTERFACES ( MediaPlugin MeganizerPlugin )
	*   // Register properties, needed by EPluginLoader to
	*   // access information about this plugin.
	*   Q_PROPERTY(QString name READ name)
	*   Q_PROPERTY(QString description READ description)
	*   Q_PROPERTY(QIcon icon READ icon)
	*   public:
	*     ...
	* };
	* @endcode
	*
	* or copy and adapt for example the Books plugin.
	*
	* @see SearchPlugin
	* @see MeganizerPlugin
	* @see Medium
	* @see MediaModel
	*/
class LIBMEGANIZER_EXPORT MediaPlugin : public MeganizerPlugin
{
  public:
    virtual ~MediaPlugin() {}

		/**
			* @brief Create a new MediaModel for material type @p materialType.
			*
			* The @p materialType is the string, which is returned in supportedMaterialTypes().
			* If the type is not supported by this plugin, return NULL.
			*/
    virtual MediaModel *newModel( QString materialType, EQueryThread *thread = NULL ) = 0;
		/**
			* @brief Create a new Medium of material type @p materialType.
			*
			* The @p materialType is the string, which is returned in supportedMaterialTypes().
			* If the type is not supported by this plugin, return NULL.
			*/
    virtual Medium *newMedium( QString materialType, EQueryThread *thread = NULL ) = 0;
		/**
			* @brief Return the search widget for medium of type @p materialType.
			*
			* The @p materialType is the string, which is returned in supportedMaterialTypes().
			*/
    virtual SearchDetailsWidget *searchWidget( QString materialType ) = 0;
		/**
			* The @p materialType is the string, which is returned in supportedMaterialTypes().
			* * If the type is not supported by this plugin, return an empty list.
			*/
    virtual QStringList supportedAttributes( QString materialType ) __attribute__ ((deprecated)) = 0;
		virtual ESettingsModelItem *settingsModelItem() = 0;
    virtual ESetupWizardObject *wizardObject() = 0;
    /**
    	* @brief A list with untranslated names of supported material types.
			*
			* The list should contain the names, which are inserted into materialtypes table.
			* Normally this is YourCustomMediaClass.group() like Book::group().
    	* For translation and a description, see translateMaterialType().
    	*/
    virtual QStringList supportedMaterialTypes() = 0;
		/**
			* @brief Check whether untranslated material type is supported.
			*
			* Implement it is follows:
			* @code
			* if(supportedMaterialTypes().contains(materialType))
			*   return true;
			* return false;
			* @endcode
			*/
		virtual bool isSupportedMaterialType(QString materialType) = 0;
		/**
			* @brief A list with the translated name and the description for material from type @p materialType.
			*
			* The list should contain the translated name for given @p materialType as first element.
			* As the second element, the list should contain the translated description for the
			* media type.
			* If the type is not supported by this plugin, return empty list.
			*/
		virtual QStringList translateMaterialType(QString materialType) = 0;
		/**
			* @brief Return the unstranslated name of the given @p materialType.
			* This is the reverse function to translateMaterialType().
			* If the type is not supported by this plugin, return empty string.
			*/
		virtual QString untranslateMaterialType(QString materialType) = 0;
};

Q_DECLARE_INTERFACE ( MediaPlugin,
											"Meganizer.MediaPlugin/0.4.0" );

#endif // MEDIAPLUGIN_H
