/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIUM_H
#define MEDIUM_H

#include "libmeganizer.h"

#include <eobject.h>
#include <efileinfo.h>

class EWidget;
class MaterialPersonsModel;
class MINModel;
class MediumMIN;
class EditMedium;
class EQueryThread;
class MediaModel;
class ManufacturerModel;

#include <QString>
#include <QList>
#include <QVariant>
#include <QStringList>
#include <QImage>
#include <QSqlRecord>
#include <QModelIndex>
#include <QIcon>

class QNetworkReply;
class QTextTable;
class QStringListModel;
class QSqlQuery;

class ColKeyTrans {
	public:
		inline ColKeyTrans(int col = -1, QString key = QString(), QString trans = QString()){
			column = col;
			this->key = key;
			translation = trans;
		};
		int column;
		QString key;
		QString translation;

		inline bool operator== (const ColKeyTrans &other) const
		{
			if(key == other.key)
				return true;
			return false;
		};
		inline bool operator!= (const ColKeyTrans &other) const{ return !operator==(other); };
};
		
/**
	* @brief Represents one medium with base properties.
	*
	* This base class provides many basic functions and properties, which are
	* used by many media types (Title, Manufacturer, Genre, MIN, Persons, ...).
	* To extend such features to your specific needs, create your own subclass
	* and also subclass MediaModel to handle your new Medium properly.
	* Normally you want to overload the function
	* @li Medium *newMedium() to create a new instance of your Medium type (see note below)
	* @li select(QSqlRecord) to process result of the select query
	* @li bool prepare() to prepare your medium before storing
	* @li store() to store your specific data
	* @li (storeChild(Medium*) to store this medium as a child of another medium)
	* @li setValue(QString, QVariant) to modifiy your specific data with Qt Model API
	* @li value(QString, int) to give back current data with Qt Model API
	* @li editWidget() to return a widget to edit your Medium (see EditMedium)
	* @li copyMedium() to copy your specific data
	* @li exportTextTable() to export your Medium into another format (like ODF or HTML)
	* @li exportPlainText() to export your Medium into plain text formats (like Bibtex)
	* @li supportedAttrs() to modify your provided attributes in the table view
	* @li customQueryResult() to process query results from QueryThread
	*
	* @note newMedium() should be implemented only to support Qt < 4.5. In newer versions,
	*    the Medium creates a new instance of the actuall instanciated class with help of the
	*    QMetaObject system. Therefore make sure, you place a Q_INVOKEABLE before your constructor
	*    like done in this class. Summarized, the following code should be used:
	*    @code
	*      Q_INVOKABLE YourMedium ( QueryThread *thread = NULL, Medium *parent = NULL )
	*        : Medium ( tr("Your Medium name"), "yourmediumtypename", thread, newParent ){ };
	*      #if QT_VERSION < 0x040500
	*      Medium *newMedium(){ return new YourMedium(); };
	*      #endif
	*    @endcode
	*
	* To add new media types, subclass this class and also the MediaModel class.
	* To do so, please make sure, that you respect the compatibilityMode, so that
	* your subclass can be shown quite well in the pure MediaModel. See compatibilityMode()
	* for details.
	*/
class LIBMEGANIZER_EXPORT Medium : public EObject
{
  Q_OBJECT

  Q_PROPERTY(QString title READ title WRITE setTitle)
  Q_PROPERTY(bool wish READ wish WRITE setWish)
  Q_PROPERTY(bool literatureOnly READ literatureOnly WRITE setLiteratureOnly)
  Q_PROPERTY(QString manufacturer READ manufacturerName WRITE setManufacturer)
	Q_PROPERTY(QString genre READ genreName WRITE setGenre)
	Q_PROPERTY(QString description READ description WRITE setDescription)
  // don't know how to localize that
//   Q_ENUMS(ExportFormat)
  public:
		enum Origin { Local, Manual, SearchPlugin };
		Q_DECLARE_FLAGS(Origins, Origin);
		
		/**
			* @brief Export format of this medium.
			*/
		enum ExportFormat {
			Table,				/**< All media are put into one table
											* (in this case call Medium::exportTextTable() in
											* your implementation of exportTextTable()) */
			TablePerType,	/**< All media of the same type are put into one table. */
			Long,					/**< One Medium per page should be exported. */
			Bibtex				/**< Export this Medium as a Bibtex entry with exportPlainText(). */
		};
		static QString exportFormatToString(ExportFormat format);
		
    Q_INVOKABLE Medium ( EQueryThread *thread = NULL, Medium *newParent = NULL );
		/**
			* @brief This constructor should be used by any subclasses.
			*
			* The medium @p name and @p group are given with the first two arguments.
			* With help of the third (@p selectTable), you can specify a database table name,
			* which is used at the load() function, to select the medium data.
			* This table is also used in some other functions (like prepare()). If empty, viewmaterials
			* is used.
			*/
		Medium ( QString name, QString group, QString selectTable, EQueryThread *thread = NULL, Medium *newParent = NULL );
    virtual ~Medium();
		
		virtual Medium *newMedium();
		QString name();
    QString group();
		void setThread(EQueryThread *thread);
		/**
			* @brief Disables the usage of the EQueryThread.
			* 
			* Disables thread usage with help of moving the current thread (if not NULL) to inactiveThread
			* and setting the current thread to NULL.
			* To enable it again afterwards, use enableThread().
			*/
		void disableThread();
		/**
			* @brief Enables the usage of the EQueryThread.
			* 
			* Enables thread usage with help of moving the inactiveThread (if not NULL) to current thread.
			* To enable thread usage, if no old thread exists, use setThread().
			* To disable the thread use disableThread().
			*/
		void enableThread();
    /**
    	* @brief Fetches neeeded stuff from database for medium with id @p searchId.
			*
    	* @param searchId id of table materials to indicate which item should be fetched
    	* @note A subclass of Medium should implement it's own select function
    	*     to select the additional stuff which the subclass needs.
    	*     The base class implementation should be called at the beginning of the subclassed select
    	*     function to fetch the standard stuff.
			* @todo Make sure @p userId is handled correctly by the subclasses or if it is
			*    useless, strip it out.
    	*/
    bool load ( unsigned int searchId, unsigned int userId = 0 );
    /**
      * @brief Sets all data from the @p record.
			*
			* Should be overloaded, but call the base class
			* implementation to avoid double coding.
      */
    virtual bool select( QSqlRecord record);
		/**
			* @brief Prepare the medium to be stored right now.
			*
			* This means, make sure, that all needed fields are set with a valid value
			* (e.g.: genres() > 0 && manufacturer() > 0)! To achieve this, if you can not
			* call the base class implementation as shown below, use checkKeys() to check
			* such standard stuff.
			* To prepare your custom stuff, overload it like this:
			* @code
			* bool prepare(){
			*   if(Medium::prepare())
			*     return false;
			*   // prepare your custom stuff below
			*   ...
			*   // if anything fails:
			*   if(!result){
			*     EditMedium *edit = editWidget();
			*     edit->setAddMode(this, false);
			*     if(edit->exec() != QDialog::Accepted)
			*       return false;
			*     return prepare();
			*   }
			* }
			* @endcode
			*/
		virtual bool prepare();
    /**
      * @brief The function stores the standard data of this medium into the database.
			*
			* It uses prepare() to prepare the medium before storing. Afterwards the common inserts of
			* this medium are executed and the id of the newly entry is tried to fetch. It uses
			* the genre, the manufacturer, the type and the title to fetch the newly added material.
			* If more than one material is found, the last insert is the highest id.
			* 
      * @param addOwner if called as true, an entry at userownsmaterial with actual user
      * @param addOwner is added, if the actual user doesn't own it already
      * @attention addOwner takes only effect if an update is done, 
      * @attention otherwise the owner is set to actual user
      */
    virtual bool store(bool addOwner = false, bool startTransaction = true);
		/**
			* @brief Is used to store this medium as a child of @p parent
			*
			* It is called within MediaModel::store(Medium*).
			*
			* This base implementation handles the database table materialhasmaterials and
			* sets the materialHasMaterialsId() if the process was successfull. Therefore
			* you should call the base implementation right after preparing your Medium.
			* Afterwards you can process your special stuff with help of materialHasMaterialsId()
			* in your implementation of store(Medium*).
			* @code
			* bool storeChild(Medium *parent){
			*   // prepare your medium here e.g.:
			*   if(genre() < 1){
			*     setGenre(parent->genre(), false);
			*     setGenre(parent->genreName(), false);
			*   }
			*   // remove your special relations, if markForRemovel() is true
			*   if(markForRemovel()){
      *     query.prepare("DELETE FROM yourspecialrelationtable WHERE materialhasmaterialsId=:materialhasmaterialsId");
			*     query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
			*     if(!query.exec())
			*       return false;
			*   }
			*   // Add standard relation entries or if markForRemovel()
			*   // is true, remove the standard relations
			*   if(!Medium::storeChild(parent))
			*     return false;
			*   if(markForRemovel())
			*     return true;
			*   // add your special relations below
			*   ...
			*   return true;
			* }
			* @endcode
			*/
		virtual bool storeChild(Medium* parent);
    /**
    	* @todo check wether QVariant can be converted to needed value type
			* @attention if submit persons like authors (,...),
			* @attention you need this triple as QStringList: Name, givenName (may be empty), roleName or Id (may be empty)
			* @attention if you want to set MINs, you can either give only a min and the type will be gassed,
			* @attention or you can give a QStringList: min, type
			* in the subclass function, only special attributes, which Medium does not support, must be handled
			* to handle standard stuff, call return Medium::setValue(QString, QVariant) at the end
			* Please make sure, that you support the compatibilityMode() like this
			* @code
			*   if(compatibilityMode())
			*     return Medium::supportedAttrs();
			*   // Do what you need, if compatibilityMode() is false
			* @endcode
    	*/
		virtual bool setValue ( QString attribute, QVariant value );
    bool setValue ( int attributeIndex, QVariant value );
		/**
			* Please make sure, that you support the compatibilityMode() like this
			* @code
			*   if(compatibilityMode())
			*     return Medium::supportedAttrs();
			*   // Do what you need, if compatibilityMode() is false
			* @endcode
			*/
		virtual QVariant value ( QString attribute, int role );
    QVariant value ( int attributeIndex, int role );
    /**
    	* a list with supported attributes (not translated)
    	*/
    QStringList supportedAttributes();
		QStringList supportedAttributesTrans();
		/**
			* @brief A list of supported attributes (not translated, translated).
			*
			* Please make sure, that you support the compatibilityMode() like this
			* @code
			*   if(compatibilityMode())
			*     return Medium::supportedAttrs();
			*   // Do what you need, if compatibilityMode() is false
			* @endcode
			*/
		virtual QList<ColKeyTrans> supportedAttrs();
		/**
			* @brief returns the ColKeyTrans belonging to attribut @p key.
			*
			* It can be used to insert translations of this class into the
			* supportedAttrs list of a subclass by
			* attrsSubclass << Medium::attrs(key, newCol);
			* @return if key not an empty ColKeyTrans is returned
			* @note if newCol is greator -1 the base class implementation is used!
			*/
		ColKeyTrans attrs(QString key, int newCol = -1);


		virtual EditMedium* editWidget(EWidget* parent = NULL);
		/**
			* Do a bit more around Medium::editWidget
			* @param editMode is EditMedium::EditMode enum
			*/
		virtual bool edit(int editMode, EWidget *parent = NULL);
		/**
			* checks wether the medium has the minimum needed stuff
			*/
		virtual bool isValid();
		/**
			* The sublass should call this implementation with
			* Medium::copyMedium() and copy specific properties
			* afterwards
			*/
		virtual Medium* copyMedium();
		/**
			* @brief Exports the content of this Medium into a QTextTable.
			*
			* if compatibility mode is activated and ExportFormat is Table,
			* call the base implementation like that:
			* @code
			* if(compatibilityMode() && format == Table)
			*   return Medium::exportTextTable(table, format, titleOnly);
			* @endcode
			* For formats where plain text is needed (e.g. BibTex, XML, ...)
			* see exportPlainText().
			*/
		virtual bool exportTextTable(QTextTable *table, Medium::ExportFormat format, bool titleOnly = false);
		/**
			* @brief Exports the content of this Medium into a plain text format.
			*
			* If @p format is Bibtex, use bibtexStrip() to strip out characters,
			* which are unsupported by bibtex.
			*
			* For formats where a QTextDocument is needed (ODF, html), see
			* exportTextTable().
			*/
		virtual QString exportPlainText(Medium::ExportFormat format);

    void setParent ( Medium *newParent );
    Medium *parent() const;
    void setId ( unsigned int newId );
    unsigned int id() const;
    void setType ( unsigned int typeId, bool setName = true );
		void setType ( QString name, bool setId = true );
		void setType ( QSqlRecord record );
    unsigned int type() const;
		QString typeName() const;
		/**
			* sets the id of the manufacturer at the database to manuId
			* and tries to get the name of the manufacturer and sets it to the result
			* @param manuId the id of the manufacturer at the manufacturers table
			* @param setName wether the corresponding name to the id should be setted or not (if possible)
			*/
    void setManufacturer ( unsigned int manuId, bool setName = true );
		/**
			* the manufacturer name is set to name
			* if setId is true, it tries to find the id corresponding to the name at the db and sets it
			*/
		void setManufacturer ( QString name, bool setId = true );
		void setManufacturer ( QSqlRecord record );
		/**
			* returns the id of the manufacturer
			*/
    unsigned int manufacturer() const;
		/**
			* returns the name of the manufacturer
			*/
		QString manufacturerName() const;
		bool prepareManufacturer();
    void setGenre ( unsigned int genreId, bool setName = true );
		void setGenre ( QString name, bool setId = true );
		void setGenre ( QSqlRecord record );
    unsigned int genre() const;
		QString genreName() const;
		bool prepareGenres();
    void setTitle ( QString newTitle );
    QString title() const;
    void setMIN ( QString newMin, QString minType = QString() );
		void setMIN ( QString newMin, unsigned int minTypeId );
		void setMIN ( MINModel *newMINModel );
    MINModel *mins() const;
		QString minsCombined() const;
		/**
			* @return the min, which's type is into @p minTypes
			* @return if more than one MIN's of @p minTypes are found, 
			* @return the one which lowest type index (index of type at @p minTypes) is returned
			* @return if no MIN with wished type is found or no other type can be converted to, empty QString is returned
			*/
		QString min(QStringList minTypes) const;
		QString min(QString minType) const;
    void setListPrice ( double price );
    double listPrice() const;
    void setListPriceCurrency ( unsigned int curId );
		/**
			* @p shortCode can be the shortcode of a currency or the name
			*/
		void setListPriceCurrency ( QString shortCode );
    unsigned int listPriceCurrency() const;
    void setDescription(QString desc);
    QString description() const;
		/**
			* @brief Sets the image of this medium to @p newImage.
			* 
			* We use QImage only in this class, because QPixmap is not threadsafe.
			*/
    void setImage ( QImage newImage );
		void setImage ( QString url );
    QImage image() const;
		QByteArray imageToByteArray();
    void setUser ( unsigned int userId );
    unsigned int user() const;
    void setBoughtPrice ( double price );
    double boughtPrice() const;
    void setBoughtPriceCurrency ( unsigned int curId );
		/**
			* @see setListPriceCurrency()
			*/
		void setBoughtPriceCurrency ( QString shortCode );
    unsigned int boughtPriceCurrency() const;
    void setEvaluation ( int eval );
    int evaluation() const;
    void setWish ( bool isWish );
    bool wish() const;
		void setLiteratureOnly ( bool isLiterature );
		bool literatureOnly () const;
		void setGiftOnly( bool isGift );
		bool giftOnly() const;
		void setSourceIcon(QIcon icon);
		QIcon sourceIcon() const;
		Medium::Origin source() const;
		void setSource(Medium::Origin newOrigin);
    
    // handle material persons
		void setMaterialPerson(QString name, QString givenName, QString role);
		void setMaterialPerson(QString name, QString givenName, unsigned int role);
		void setMaterialPersons(MaterialPersonsModel *personModel);
    MaterialPersonsModel *materialPersons() const;
		QString materialPersonsCombined() const;
    bool removeMaterialPerson(QModelIndex index);
		void setMarkForRemovel();
    bool markForRemovel();

		// handle keywords
		void addKeyword(QString keyword);
		bool removeKeyword(QString keyword);
		/**
			* @returns the keywords as a QStringListModel
			* You must take care to delete the model of your own
			*/
		QStringListModel *keywords();
		
		// handle files
		bool addFile(EFileInfo info);
		bool removeFile(QString uniqueId);
		/**
			* @brief Returns true, if the file with @p id is listed within myFiles.
			*/
		bool fileListed(QString uniqueId) const;
		QList<EFileInfo> files() const;
		
		virtual MediaModel *childs();
		/**
			* @brief Sets @p childModel as the child model used by childs() and removeChild().
			*/
		void setChildModel(MediaModel *childModel);
		bool removeChild(QModelIndex index);
		void setMaterialHasMaterialsId(uint id);
		uint materialHasMaterialsId() const;
		
    int rowCount() const;
    int columnCount();
    bool hasChildren() const;
		void setCompatibilityMode(bool enable);
		
    bool operator== (const Medium &med) const;
		bool operator!= (const Medium &med) const;

	protected slots:
		/**
			* @brief This function is used to process records from EQueryThread.
			*
			* It is connected to EQueryThread::result() and distributes
			* the given results to the specific processing function for each request id.
			* To distribute your own @p requestId in your subclass, implement the
			* customQueryResult() function.
			*/
		void queryResult(QObject *client, QVariant requestId, int querySize, QSqlRecord record);
		
  protected:
		/**
			* @brief Implement in your subclass to process your own threaded SQL-Requests.
			*
			* The default implementation does nothing.
			*/
		virtual void customQueryResult(QVariant requestId, int querySize, QSqlRecord record);
		/**
			* Use this function to check wether compatibility mode
			* is enabled or not. Compatibility mode is used, to show
			* a subclass of this class in a pure MediaModel. Therefore
			* the column order and hiding is different to the subclasses.
			* You should respect this espacially at this functions:
			*  - value(QString, int), setValue(QString, QVariant)
			*  - supportedAttrs().
			*
			* Usually you should only call the base implementation in your
			* functions, if compatibilityMode() is set to true.
			*/
		bool compatibilityMode() const;
		QString bibtexStrip(QString string);
		/**
			* @brief Checks that all common foreign key relations are valid.
			*/
		bool checkKeys();
		
		bool transactionsSupported;
		QString myGroup, myName, mySelectTable;
		EQueryThread *thread, *inactiveThread;

	private slots:
		/**
			* @todo seems to procude a segfault from time to time
			*/
		void imageDownloaded();
		/**
			* @brief Sets the current manufacturer from a response according to manuserver.xsd.
			*/
		void loadManufacturerRequestFinished(ManufacturerModel *model);
		
  private:
		void init(EQueryThread *thread = NULL, Medium *newParent = NULL, QString selectTable = "viewmaterials");
		bool loadKeywords();
		bool storeKeywords();
		bool loadFiles();
		bool storeFiles();
		/**
			* @brief Tries to set the manufacturer related to the given @p min.
			* 
			* If the manufacturer is already set (manufacturer() > 0), this function
			* does nothing.
			*/
		void setManufacturerByMIN(MediumMIN *min);
		
		Medium *myParent;
		uint myMaterialHasMaterialsId;
		unsigned int myId, myTypeId, myManuId, myGenreId, myCurId, myUserId, myBoughtCurId;
		QString myTypeName, myTitle, myEan, myDescription, myManuName, myGenreName;
		double myListPrice, myBoughtPrice;
		int myEval;
		bool myWish, myLiterature, myGift, shouldBeRemoved;
		bool myCompatibilityMode;
		QImage myImage;
		MaterialPersonsModel *matPersonModel;
		MINModel *minModel;
		QIcon mySourceIcon;
		Medium::Origin myOrigin;
		QStringList keywordList;
		MediaModel *myChilds;
		QList<EFileInfo> myFiles;
    
  signals:
    void changed(Medium *medium);
};

Q_DECLARE_OPERATORS_FOR_FLAGS(Medium::Origins)

#endif // MEDIUM_H
