/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumexport.h"
#include "mediumexport.moc"

#include "mediamodel.h"
#include "medium.h"
#include "selectexportformat.h"
#include <efiledialog.h>

#include <QSettings>
#include <QTextDocument>
#include <QTextCursor>
#if QT_VERSION > 0x040400
	#include <QTextDocumentWriter>
#endif
#include <QTextTable>
#include <QDateTime>
#include <QProgressDialog>
#include <QTextStream>
#include <QTextCodec>
#include <QFile>

#include <QDebug>

MediumExport::MediumExport(QObject* parent)
	: EObject(parent)
{
	QTextTableFormat tableFormat;
	tableFormat.setHeaderRowCount(1);
}

MediumExport::~MediumExport()
{

}

QMap<QString, QStringList> MediumExport::supportedFileFormats()
{
	QMap<QString, QStringList> results;
	QStringList suffixes;
// 	suffixes << "bib";
// 	results.insert(tr("Bibtex"), suffixes);
// 	
// 	suffixes.clear();
// 	suffixes << "tex";
// 	results.insert(tr("Latex"), suffixes);
	
	suffixes.clear();
	suffixes << "odf";
	results.insert(tr("OpenDocument"), suffixes);
	
	suffixes.clear();
	suffixes << "html";
	results.insert(tr("HTML"), suffixes);

	suffixes.clear();
	suffixes << "bib";
	results.insert(tr("Bibtex"), suffixes);
	
// 	suffixes.clear();
// 	suffixes << "pdf";
// 	results.insert(tr("PDF"), suffixes);
	
	return results;
}

bool MediumExport::exportMedium(Medium* medium, Medium::ExportFormat layout, QString format, QString file)
{
	if(!medium)
		return false;
	
	MediaModel *model = new MediaModel(this);
	model->addMedium(medium->copyMedium());
	bool result = exportMedium(model, QModelIndexList(), layout, format, file);
	delete model;
	return result;
}

bool MediumExport::exportMedium(MediaModel* model, QModelIndexList indexes, Medium::ExportFormat layout, QString format, QString file)
{
	if(!model)
		return false;

	// getting file
	QFileInfo fileInfo = selectFile(file, format);
	if(fileInfo.fileName().isEmpty())
		return false;

	if(fileInfo.exists() && !fileInfo.isWritable()){
		emit error(tr("Selected file is not writeable"));
		return false;
	}

	format = selectFormat(format, fileInfo);
	if(format.isEmpty())
		return false;

	if(format == "bib")
		layout = Medium::Bibtex;
	
	bool result = true;
	QString document;
	QTextDocument *textDocument = NULL;
	QTextTable *table = NULL;
	if(layout != Medium::Bibtex){
		// prepare document
		textDocument = new QTextDocument(this);
		if(!addHeader(textDocument))
			result = false;

		QTextCursor cursor(textDocument);
		cursor.movePosition(QTextCursor::End);
		table = cursor.insertTable(1, 1, tableFormat);
		if(!table){
			emit error(tr("Could not create table"));
			return false;
		}
	}
	
	// add media to document
	int totalMedia = indexes.size();
	if(totalMedia < 1)
		totalMedia = 2 * model->rowCount();

	QProgressDialog progDia ("Creating export document...", "Abort Export", 0, totalMedia);
	progDia.setWindowModality(Qt::WindowModal);
	progDia.setMinimumDuration(2000);
	
	if(indexes.size() < 1){
		for(int i = 0; i < model->rowCount(); i++){
			indexes.append(model->index(i, 0));
			progDia.setValue(progDia.value() + 1);
		}
	}

	bool first = true;
	foreach(QModelIndex index, indexes){
		Medium *med = model->mediumFromIndex(index);
		bool mediumResult = true;
		if(!med){
			mediumResult = false;
		}
		else{
			if(layout == Medium::Bibtex){
				document += med->exportPlainText(layout) +"\n";
			}
			else{
				if(first){
					if(!med->exportTextTable(table, layout, true)){
						mediumResult = false;
					}
					else{
						first = false;
					}
				}
				if(!med->exportTextTable(table, layout))
					mediumResult = false;
			}
		}
		if(!mediumResult){
			mediumResult = false;
			emit warning(tr("Could not create internal document structures. The output may be wrong or impossible."));
		}
		progDia.setValue(progDia.value() + 1);
		if(progDia.wasCanceled())
			return false;
	}
	progDia.close();
	
	if(layout == Medium::Bibtex){
		QFile file(fileInfo.absoluteFilePath());
		if(!file.open(QFile::WriteOnly)){
			emit error(tr("Could not open selected file"));
			return false;
		}
		QTextStream stream(&file);
		stream.setCodec(QTextCodec::codecForName(QString("UTF-8").toAscii()));
		stream << document;
		file.close();
	}
	else{
#if QT_VERSION < 0x040500
		emit error(tr("QT greater 4.4 is needed, but you have %1").arg(QT_VERSION));
		return false;
#else
		QTextDocumentWriter writer(fileInfo.absoluteFilePath());
		writer.setFormat(format.toAscii());
		if(!writer.write(textDocument)){
			emit error(tr("Could not write document"));
			return false;
		}
#endif
	}
	emit information(tr("Document exported to %1").arg(fileInfo.absoluteFilePath()));
	return result;
}

QFileInfo MediumExport::selectFile(QString file, QString format)
{
	QFileInfo fileInfo(file);
	if(!fileInfo.fileName().isEmpty()){
		return fileInfo;
	}
	
	QMultiMap<QString, QStringList> formats = supportedFileFormats();
	QStringList keys = formats.keys();
	QString *defaultFilter = 0;
	QStringList filters;
	filters << QString("%1 (*.*)").arg(tr("All Files"));
	foreach(QString key, keys){
		QString filter = QString("%1 (*.%2)").arg(key).arg(formats.value(key).join(" *."));
		if(key == format)
			defaultFilter = &filter;
		filters << filter;
	}
	
	QSettings settings;
	QString lastDir = settings.value("lastExportDir", "").toString();
	QString wTitle = tr("Save File");
	fileInfo = QFileInfo(EFileDialog::getSaveFileName(0, wTitle, lastDir, filters.join(";;"), defaultFilter));
	if(!fileInfo.absolutePath().isEmpty())
		settings.setValue("lastExportDir", fileInfo.absolutePath());
	return fileInfo;
}

QString MediumExport::selectFormat(QString format, QFileInfo file)
{
	QMultiMap<QString, QStringList> formats = supportedFileFormats();
	if(!format.isEmpty()){
		if(formats.value(format).size() < 1)
			return QString();
		return formats.value(format)[0];
	}
		
	// try to get format out of file name
	QString suffix = file.suffix();
	QStringList keys = formats.keys();
	foreach(QString key, keys){
		if(formats.value(key).indexOf(suffix) > -1){
			if(formats.value(key).size() < 1)
			return QString();
		return formats.value(key)[0];
		}
	}

	SelectExportFormat dia(formats);
	connectByPass(&dia);
	if(dia.exec() == QDialog::Accepted)
		format = dia.selectedFormat();
	dia.deleteLater();
	if(formats.value(format).size() < 1)
		return QString();
	return formats.value(format)[0];
}

bool MediumExport::addHeader ( QTextDocument* document )
{
	if(!document)
		return false;
	
	QTextCursor cursor(document);
	QTextTableFormat tableFormat;
	tableFormat.setHeaderRowCount(1);
	// header table
	QTextTable *table = cursor.insertTable(2, 2, tableFormat);
	if(!table){
		emit error(tr("Could not create table"));
		return false;
	}
	QTextTableCell cell = table->cellAt(0, 0);
	cell.firstCursorPosition().insertText(tr("Medialist"));
	table->mergeCells(0, 0, 1, 2);
	cell = table->cellAt(1, 0);
	cell.firstCursorPosition().insertText(tr("Created:"));
	cell = table->cellAt(1, 1);
	cell.firstCursorPosition().insertText(QDateTime::currentDateTime().toString(tr("MM/dd/yyyy hh:mm:ss")));
	return true;
}
