/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "mediumfileswidget.h"
#include "mediumfileswidget.moc"
#include "ui_mediumfileswidget.h"

#include "medium.h"
#include <efileservice.h>
#include <efiledialog.h>

#include <QSettings>

MediumFilesWidget::MediumFilesWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags), ui(new Ui::MediumFilesWidget)
{
	ui->setupUi(this);
	connect(ui->addButton, SIGNAL(clicked(bool)), this, SLOT(addFile()));
	connect(ui->removeButton, SIGNAL(clicked(bool)), this, SLOT(removeFile()));
	connect(ui->filesView, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(openFile(QListWidgetItem*)));
	connect(ui->openButton, SIGNAL(clicked(bool)), this, SLOT(openFile()));
}

bool MediumFilesWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);
	if(!medium){
		emit error(tr("Could not store the files. (No medium)"));
		return false;
	}
	
	if(!modified())
		return true;

	// first remove files, because they may be added by the user after the remove
	foreach(EFileInfo info, removedFiles){
		if(!medium->removeFile(info.id())){
			emit error(tr("Could not store the files. (Could not remove file %1)").arg(info.name()));
			return false;
		}
	}
	// add files
	foreach(QString name, mapFiles.uniqueKeys()){
		EFileInfo info = mapFiles.value(name);
		if(!medium->fileListed(info.id())){
			if(!medium->addFile(info)){
				emit error(tr("Could not store the files. (Could not add file %1)").arg(info.name()));
				return false;
			}
		}
	}

	setModified(false);
	return true;
}

void MediumFilesWidget::setMedium(Medium* newMedium)
{
	setEnabled(false);
	setModified(false);
	mapFiles.clear();
	ui->filesView->clear();

	medium = newMedium;
	if(!medium)
		return;

	foreach(EFileInfo info, medium->files())
		mapFiles.insert(info.name(), info);
	ui->filesView->addItems(mapFiles.uniqueKeys());

	setEnabled(true);
	setModified(false);
}

void MediumFilesWidget::addFile()
{
	EFileService service;
	if(!service.serviceLoaded()){
		emit warning(tr("You have no file service selected right now. Please do that now and add a file again."));
		emit showSettings();
		return;
	}
	
	QSettings settings;
	QString lastDir = settings.value("lastFileDir").toString();
	QString file = EFileDialog::getOpenFileName(this, tr("Select a file to add"), lastDir);
	if(file.isEmpty())
		return;
	
	QFileInfo fileInfo(file);
	settings.setValue("lastFileDir", fileInfo.absolutePath());
	
	EFileInfo info = service.add(fileInfo);
	if(!info.isValid())
		return;
	
	mapFiles.insert(info.name(), info);
	ui->filesView->addItem(info.name());
	setModified();
}

void MediumFilesWidget::removeFile()
{
	QListWidgetItem *item = ui->filesView->currentItem();
	if(!item)
		return;
	
	QString name = item->text();
	EFileInfo info = mapFiles.value(name);
	if(!info.isValid())
		return;
	
	ui->filesView->removeItemWidget(item);
	delete item;
	
	removedFiles << info;
	mapFiles.remove(name);
	setModified();
}

void MediumFilesWidget::openFile(QListWidgetItem* item)
{
	if(!item)
		return;
	
	EFileInfo info = mapFiles.value(item->text());
	if(!info.isValid())
		return;
	
	EFileService service;
	if(!service.serviceLoaded()){
		emit warning(tr("You have no file service selected right now. Please do that now and add a file again."));
		emit showSettings();
		return;
	}
	
	if(!service.open(info))
		emit error(tr("Could not open the file %1.").arg(item->text()));
	else
		emit information(tr("Opened %1").arg(item->text()));
}

void MediumFilesWidget::openFile()
{
	openFile(ui->filesView->currentItem());
}
