/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumsimpleeditwidget.h"
#include "mediumsimpleeditwidget.moc"

#include "medium.h"

#include <esearchbutton.h>

#include <QHBoxLayout>
#include <QLabel>
#include <QSpinBox>
#include <QDateEdit>
#include <QTimeEdit>
#include <QLineEdit>
#include <QCheckBox>
#include <QPlainTextEdit>

MediumSimpleEditWidget::MediumSimpleEditWidget(EditWidgetType type, QString labelText, const char *property,
																							 QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	spinBox = NULL;
	dateEdit = NULL;
	timeEdit = NULL;
	lineEdit = NULL;
	plainEdit = NULL;
	checkBox = NULL;

	medium = NULL;
	myEditWidget = NULL;
	
	// setup the choosen edit widget
	this->type = type;
	switch(type){
		case SpinBox:
			spinBox = new QSpinBox(this);
			myEditWidget = spinBox;
			// Always connect the signal, which is emitted emidiatly on changes
			// and not the one, which is emitted on leaving the widget or so.
			// (Think of rase the number on 2, do not leave the widget, but use
			// a shortcut to accept the dialog. The done change, must be also processed.)
			connect(spinBox, SIGNAL(valueChanged(int)), this, SLOT(setModified()));
			break;
		case DateEdit:
			dateEdit = new QDateEdit(this);
			myEditWidget = dateEdit;
			dateEdit->setDisplayFormat(tr("MM/yyyy"));
			connect(dateEdit, SIGNAL(dateChanged(QDate)), this, SLOT(setModified()));
			break;
		case TimeEdit:
			timeEdit = new QTimeEdit(this);
			myEditWidget = timeEdit;
			timeEdit->setDisplayFormat(tr("HH:mm:ss"));
			connect(timeEdit, SIGNAL(timeChanged(QTime)), this, SLOT(setModified()));
			break;
		case LineEdit:
			lineEdit = new QLineEdit(this);
			myEditWidget = lineEdit;
			connect(lineEdit, SIGNAL(textChanged(QString)), this, SLOT(setModified()));
			break;
		case PlainTextEdit:
			plainEdit = new QPlainTextEdit(this);
			myEditWidget = plainEdit;
			connect(plainEdit, SIGNAL(textChanged()), this, SLOT(setModified()));
			break;
		case CheckBox:
			checkBox = new QCheckBox(this);
			myEditWidget = checkBox;
			connect(checkBox, SIGNAL(stateChanged(int)), this, SLOT(setModified()));
			break;
	}

	setObjectName(QString(property) + "Widget");
	QHBoxLayout *ml = new QHBoxLayout(this);
	setLayout(ml);
	ml->setContentsMargins(0, 0, 0, 0);

	label = new QLabel(this);
	ml->addWidget(label);
	label->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	label->setMinimumSize(150, 23);
	setLabelText(labelText);

	if(myEditWidget){
		ml->addWidget(myEditWidget);
		myEditWidget->setObjectName(QString(property) +"Edit");
		myEditWidget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
		myEditWidget->setMinimumSize(150, 23);
		if(type == PlainTextEdit)
			myEditWidget->setMaximumHeight(5000);
		else
			myEditWidget->setMaximumHeight(30);
	}

	this->property = property;
}

void MediumSimpleEditWidget::addSearchButton(const char * method)
{
	// only add a new one, if there was no search button before
	ESearchButton *button = findChild<ESearchButton*>();
	if(!button){
		button = new ESearchButton(this);
		button->setObjectName(QString(property) +"SearchButton");
		layout()->addWidget(button);
		button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
		button->setMinimumSize(100, 23);
		button->setMaximumHeight(30);
	}
	connect(button, SIGNAL(clicked()), this, method);
}

bool MediumSimpleEditWidget::store(bool useExistingTransaction)
{
	Q_UNUSED(useExistingTransaction);

	// check if everything is setup correctly
	if(QString(property).isEmpty()){
		emit error(tr("Could not store data. (No property specified)"));
		return false;
	}
	if(!medium){
		emit error(tr("Could not store %1. (No medium given)").arg(property));
		return false;
	}
	if(medium->metaObject()->indexOfProperty(property) < 0){
		emit error(tr("Could not store %1. (Property does not exist)").arg(property));
		return false;
	}

	if(!modified())
		return true;
	bool result = false;
	switch(type){
		case SpinBox:
			result = medium->setProperty(property, spinBox->value());
			break;
		case DateEdit:
			result = medium->setProperty(property, dateEdit->date());
			break;
		case TimeEdit:
			result = medium->setProperty(property, timeEdit->time());
			break;
		case LineEdit:
			result = medium->setProperty(property, lineEdit->text());
			break;
		case PlainTextEdit:
			result = medium->setProperty(property, plainEdit->toPlainText());
			break;
		case CheckBox:
			result = medium->setProperty(property, checkBox->isChecked());
			break;
	}
	if(result)
		setModified(false);
	else
		emit error(tr("Could not store %1. (The property could not be set)").arg(property));
	return result;
}

void MediumSimpleEditWidget::setMedium(Medium* newMedium)
{
	setEnabled(false);
	medium = newMedium;
	if(QString(property).isEmpty())
		return;
	if(!medium)
		return;
	// check wether property exists
	if(medium->metaObject()->indexOfProperty(property) < 0)
		return;

	switch(type){
		case SpinBox:
			spinBox->setValue(medium->property(property).toInt());
			break;
		case DateEdit:
			dateEdit->setDate(medium->property(property).toDate());
			break;
		case TimeEdit:
			timeEdit->setTime(medium->property(property).toTime());
			break;
		case LineEdit:
			lineEdit->setText(medium->property(property).toString());
			break;
		case PlainTextEdit:
			plainEdit->setPlainText(medium->property(property).toString());
			break;
		case CheckBox:
			checkBox->setChecked(medium->property(property).toBool());
			break;
	}
	// set it after setting the initialize values, because
	// otherwise it is modified everytime
	setModified(false);
	setEnabled(true);
}

void MediumSimpleEditWidget::setLabelText(QString text)
{
	if(text.isEmpty())
		label->setVisible(false);
	else
		label->setVisible(true);
	text = text.trimmed();
	if(text.right(1) != ":")
		text += ":";
	label->setText(text);
}

QString MediumSimpleEditWidget::labelText() const
{
	return label->text();
}

QWidget* MediumSimpleEditWidget::editWidget() const
{
	return myEditWidget;
}

