/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MEDIUMSIMPLEEDITWIDGET_H
#define MEDIUMSIMPLEEDITWIDGET_H

#include "libmeganizer.h"
#include <mediumeditwidget.h>

class Medium;

class QLabel;
class QSpinBox;
class QDateEdit;
class QTimeEdit;
class QLineEdit;
class QCheckBox;
class QPlainTextEdit;

/**
	* @brief Class to display and edit medium properties.
	*
	* It uses the Q_PROPERTY systemt to read and set the data.
	* You can create it with different edit widget types (see EditWidgetType)
	* and specify which property should be used. It checks wether the property
	* exists or not.
	* To create your own subclass, do something like this:
	* @code
	* class LIBMEGANIZER_EXPORT MediumLiteratureWidget : public MediumSimpleEditWidget
	* {
	*   Q_OBJECT
	*   public:
	*   MediumLiteratureWidget(QWidget *parent = 0, Qt::WindowFlags flags = 0)
	*     : MediumSimpleEditWidget(LineEdit, tr("Title"), "title", parent, flags){
	*     // If you want access to the edit widget:
	*     QLineEdit *titleEdit = qobject_cast<QLineEdit*>(editWidget());
	*   }
	* };
	* @endcode
	* and don't forget to register the needed properties within your medium class.
	*
	* To test your subclass, see MediumSimpleEditWidgetBaseTest.
	* @see MediumTitleWidget
	* @see MediumWishWidget
	*/
class LIBMEGANIZER_EXPORT MediumSimpleEditWidget : public MediumEditWidget
{
	Q_OBJECT
	Q_PROPERTY(QString labelText READ labelText WRITE setLabelText)
	public:
		/**
			* @brief The type of the used edit widget.
			*/
		enum EditWidgetType {
			SpinBox,				/**< The used edit widget is a QSpinBox */
			DateEdit,				/**< The used edit widget is a QDateEdit */
			TimeEdit,				/**< The used edit widget is a QTimeEdit */
			LineEdit,				/**< The used edit widget is a QLineEdit */
			PlainTextEdit,	/**< The used edit widget is a QPlainTextEdit */
			CheckBox				/**< The used edit widget is a QCheckBox */
		};
		/**
			* @brief Create a new medium edit widget.
			*
			* The type of the widget, which can be used to edit the data, can be
			* specified with @p type. The @p labelText is displayed at the label before.
			* The data which is displayed and edited by this widget can be set with
			* @p property. This is the name of the relatd property of the Medium.
			* If it is empty or does not exists, this widget is empty and store
			* returns false, if anythin is changed by the user.
			*
			* @note The label gets a ":" appended to the @p labelText by default. So
			*    it is not needed to specify one. But if the last character is a ":",
			*    a second is not added.\n
			*    Some objectNames are also set (all examples with @p property = "title"):
			*    @li own object name is: @p property +"Widget" (e.g. "titleWidget")
			*    @li edit widget: @p property + "Edit" (e.g. "titleEdit")
			*    @li search button: @p property +"SearchButton (e.g. "titleSearchButton")
			*/
		MediumSimpleEditWidget(EditWidgetType type, QString labelText, const char *property,
													 QWidget* parent = 0, Qt::WindowFlags flags = 0);
		/**
			* @brief Adds a search button and connects it to SLOT @p method.
			*
			* @code
			* addSearchButton(SLOT(myReciever()));
			* @endcode
			*/
		void addSearchButton(const char * method);
    bool store(bool useExistingTransaction = false);
		/**
			* @brief Reimplement this function to do some special reaction on new medium.
			*
			* @note Normally it is not needed to reimplement this function. Only in some
			*   rare special cases, this is needed. If so, don't forget to call the base
			*   implementation MediumSimpleEditWidget::setMedium().
			*/
    virtual void setMedium(Medium* newMedium);
		void setLabelText(QString text);
		QString labelText() const;

	protected:
		QWidget *editWidget() const;

	private:
		const char *property;
		QLabel *label;
		QWidget *myEditWidget;
		Medium *medium;
		EditWidgetType type;
		// to avoid cast it everytime,
		// editWidget is also stored casted
		QSpinBox *spinBox;
		QDateEdit *dateEdit;
		QTimeEdit *timeEdit;
		QLineEdit *lineEdit;
		QCheckBox *checkBox;
		QPlainTextEdit *plainEdit;
};

#endif // MEDIUMSIMPLEEDITWIDGET_H
