/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "minmodel.h"
#include "minmodel.moc"

#include "mediummin.h"
#include <edelegateforeignkey.h>
#include "delegatemin.h"
#include "minvalidator.h"
#include <esqldatabase.h>
#include <equerythread.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QAbstractItemView>
#include <QTableView>

MINModel::MINModel(EQueryThread* thread, QObject* parent): EAbstractItemModel(parent)
{
	this->thread = NULL;
	setThread(thread);
}

MINModel::~MINModel()
{
	qDeleteAll(items);
	items.clear();
}

void MINModel::setThread(EQueryThread* thread)
{
	if(this->thread){
		this->thread->disconnect(this);
		disconnect(this->thread);
	}
	this->thread = thread;
	if(thread){
		connect(thread, SIGNAL(result(QObject*, QVariant, int, QSqlRecord)),
						this, SLOT(queryResult(QObject*, QVariant, int, QSqlRecord)));
		foreach(MediumMIN *min, items)
			min->setThread(thread);
	}
}

MediumMIN* MINModel::addMIN()
{
	MediumMIN *newMIN = new MediumMIN(thread, this);
	connect(newMIN, SIGNAL(itemChanged(MediumMIN*)), this, SLOT(itemChanged(MediumMIN*)));
	emit layoutAboutToBeChanged();
  items << newMIN;
  emit layoutChanged();

	return newMIN;
}

MediumMIN* MINModel::addMIN(QString min, QString type)
{
	MediumMIN *newMIN = addMIN();
	newMIN->setMIN(min);
	newMIN->setType(type);
	return newMIN;
}

MediumMIN* MINModel::addMIN(QString min, unsigned int typeId)
{
	MediumMIN *newMIN = addMIN();
	newMIN->setMIN(min);
	newMIN->setType(typeId);
	return newMIN;
}

bool MINModel::addMIN(QString min, unsigned int typeId, unsigned int rowId)
{
	MediumMIN *newMIN = addMIN();
	newMIN->setMIN(min);
	newMIN->setType(typeId);
	newMIN->setRowId(rowId);
	return newMIN;
}

bool MINModel::addMIN(MediumMIN* min)
{
	min->setParent(this);
	connect(min, SIGNAL(itemChanged(MediumMIN*)), this, SLOT(itemChanged(MediumMIN*)));
	emit layoutAboutToBeChanged();
  items << min;
  emit layoutChanged();
	return true;
}

bool MINModel::addMIN(QSqlRecord record)
{
	return addMIN(record.value("min").toString(),
						record.value("mintypesId").toUInt(),
						record.value("id").toUInt());
}

bool MINModel::removeMIN(MediumMIN* min)
{
	if(items.indexOf(min) < 0)
		return false;
	
	emit layoutAboutToBeChanged();
	items.removeAll(min);
	emit layoutChanged();
	min->deleteLater();
	return true;
}

bool MINModel::removeMIN(QModelIndex index)
{
	return removeMIN(minFromIndex(index));
}

bool MINModel::store(unsigned int materialId)
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return false;

  bool result = true;
  foreach(MediumMIN *min, items){
    if(!min->store(materialId))
      result = false;
  }
  if(closeDB)
    db.close();
  return result;
}

bool MINModel::loadAllMINs(unsigned int materialId)
{
	QString queryString = QString("SELECT * FROM meganizer_materialhasmins WHERE materialsId=%1 ORDER BY id").arg(materialId);
	if(thread){
		thread->execQuery(this, "loadAllMINs", queryString, tr("Could not load MINs"));
		return true;
	}
	else{
		ESqlDatabase db = ESqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			if(!db.open())
				return false;
		bool result = true;
		QSqlQuery query;
		query.exec(queryString);
		while(query.next()){
			if(!addMIN(query.record()))
				result = false;
		}
		if(closeDB)
			db.close();
		return result;
	}
}

void MINModel::setupView(QAbstractItemView* view)
{
	if(!view)
		return;

	view->setModel(this);
	view->setSelectionBehavior(QAbstractItemView::SelectRows);
	view->setSelectionMode(QAbstractItemView::ExtendedSelection);
	
	EDelegateForeignKey *delegate = new EDelegateForeignKey("meganizer_mintypes", "id", "name", view);
	delegates << delegate;
	view->setItemDelegateForColumn(1, delegate);
	
	DelegateMIN *minDelegate = new DelegateMIN(view);
	view->setItemDelegateForColumn(0, minDelegate);
	
	QTableView *tableView = qobject_cast<QTableView*>(view);
	if(tableView)
		hideColumns(tableView);
}

void MINModel::reloadDelegates()
{
	foreach(QPointer<EDelegateForeignKey> delegate, delegates){
		if(delegate)
			delegate->reloadForeignKeys();
	}
}

void MINModel::hideColumns(QTableView* view)
{
	Q_UNUSED(view);
}

MINModel* MINModel::copyModel()
{
	MINModel *newModel = new MINModel();
	foreach(MediumMIN *min, items){
		MediumMIN *tmp = min->copyMIN();
		newModel->addMIN(tmp);
	}
	return newModel;
}

QString MINModel::combineMINs()
{
	int max = 3;
	QStringList result;
	foreach(MediumMIN *min, items){
		if(min->min().isEmpty())
			continue;
		
		QString tmp;
		if(!min->typeName().isEmpty())
			tmp = min->typeName() +": ";
		tmp += min->min();
		result << tmp;
		
		if(result.size() == max)
			break;
	}
	if(rowCount() > max)
		result.append(tr ( "more" ));
	return result.join(tr(",\n"));
}

MediumMIN* MINModel::min(QString minType)
{
	foreach(MediumMIN *min, items){
		if(min->typeName() == minType)
			return min;
	}
	return 0;
}

QStringList MINModel::mins(QString minType)
{
	QStringList results;
	MINValidator *validator = new MINValidator(this);
	foreach(MediumMIN *min, items){
		if(min->typeName() == minType){
			results << min->min();
		}
		else{
			QString result = validator->convert(min->min(), minType);
			if(!result.isEmpty())
				results << result;
		}
	}
	return results;
}

MediumMIN* MINModel::minFromIndex(QModelIndex index) const
{
	if(!index.isValid())
    return 0;
  
  return static_cast<MediumMIN*>(index.internalPointer());
}

QModelIndex MINModel::indexFromMIN(MediumMIN* min)
{
	if(items.indexOf(min) == -1)
		return QModelIndex();
	
	return index(items.indexOf(min), 0);
}

QVariant MINModel::data(const QModelIndex& index, int role) const
{
  MediumMIN *min = qobject_cast<MediumMIN*>(minFromIndex(index));
	if(!min)
		return QVariant();
  
	 if(role == Qt::ForegroundRole){
    if(min->markForRemovel())
      return QBrush(QColor(200, 200, 200));
  }
  
	if((role != Qt::DisplayRole) && (role != Qt::EditRole))
		return QVariant();
	
  switch(index.column()){
		case 0:
			return min->min();
    case 1:
			if(min->type() > 0)
				return min->type();
      return min->typeName();
  }
  return QVariant();
}

bool MINModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
  MediumMIN *min = qobject_cast<MediumMIN*>(minFromIndex(index));
	if(!min)
		return false;
  
	if(role != Qt::EditRole)
		return QAbstractItemModel::setData(index, value, role);
  
  switch(index.column()){
		case 0:
			min->setMIN(value.toString());
			return true;
    case 1:
			bool check = false;
			value.toUInt(&check);
			if(check){
				min->setType(value.toUInt());
				return true;
			}
			min->setType(value.toString());
	}
  return false;
}

int MINModel::columnCount(const QModelIndex& parent) const
{
	Q_UNUSED(parent);
	return 2;
}

QVariant MINModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	if((role != Qt::DisplayRole) && (role != Qt::EditRole))
		return QAbstractItemModel::headerData(section, orientation, role);
    
    if(orientation == Qt::Vertical)
      return section + 1;
    
    switch(section){
      case 0:
				return tr("MIN");
      case 1:
				return tr("Type");
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}

Qt::ItemFlags MINModel::flags(const QModelIndex& index) const
{
	if(!index.isValid())
		return QAbstractItemModel::flags(index);
  return QAbstractItemModel::flags(index) | Qt::ItemIsEditable;
}

int MINModel::rowCount(const QModelIndex& parent) const
{
	if(parent.isValid())
		return 0;
	
	return items.size();
}

QModelIndex MINModel::parent(const QModelIndex& child) const
{
	Q_UNUSED(child);
	return QModelIndex();
}

QModelIndex MINModel::index(int row, int column, const QModelIndex& parent) const
{
  if(parent.isValid())
    return QModelIndex();
  
	if((column < 0) || (columnCount() <= column))
		return QModelIndex();		
	
  if(row < 0)
    return QModelIndex();
  if(items.size() <= row)
    return QModelIndex();
  
  return createIndex(row, column, items[row]);
}

void MINModel::queryResult(QObject* client, QVariant requestId, int querySize, QSqlRecord record)
{
	Q_UNUSED(querySize);
	if(client != this)
		return;
	if(requestId.toString() == "loadAllMINs")
		addMIN(record);
}

void MINModel::itemChanged(MediumMIN* min)
{
	QModelIndex minIndex = indexFromMIN(min);
	emit dataChanged(minIndex, index(minIndex.row(), columnCount()));
}
