/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "libmeganizerconfig.h"

#include "minvalidator.h"
#include "minvalidator.moc"

#include "minvalidatorplugin.h"
#include <emodulelistwidgetitem.h>
#include <epluginloader.h>

MINValidator::MINValidator(QObject* parent): QValidator(parent)
{
  //load and test plugin
	plugins = EPluginLoader<MINValidatorPlugin>::loadPlugins(QString(MINPLUGINDIR), pluginProtocol);
	initPlugins();
}

MINValidator::MINValidator(QList< MINValidatorPlugin* > plugins, QObject* parent)
	: QValidator(parent)
{
	this->plugins = plugins;
	initPlugins();
}

MINValidator::~MINValidator()
{
// 	unloadPlugins();
// 	qDeleteAll(pluginProtocol);
}

EngSaS::ModuleListWidgetItems MINValidator::loadProtocol()
{
	return pluginProtocol;
}

QStringList MINValidator::supportedTypes() const
{
	return types;
}

QString MINValidator::descriptionToType(QString type) const
{
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type)
			return plugin->description();
	}
	return QString();
}

bool MINValidator::isSupported(QString type) const
{
	foreach(MINValidatorPlugin *plugin, plugins)
		if(plugin->type() == type)
			return true;
	return false;
}

void MINValidator::fixup(QString& input) const
{
	if(possibleTypes(input).size() < 1)
		return;
  
	QString result = validate(input, QString());
	if(!result.isEmpty())
		input = result;
}

QValidator::State MINValidator::validate(QString& input, int& pos) const
{
	if(isValid(input))
		return QValidator::Acceptable;
	
	if(possibleTypes(input).size() > 0)
		return QValidator::Intermediate;
	
	return QValidator::Invalid;
}

QString MINValidator::type(QString min) const
{
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->isType(min))
			return plugin->type();
	}
	return QString();
}

QStringList MINValidator::possibleTypes(QString min) const
{
	QStringList result;
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->mayType(min))
			result << plugin->type();
	}
	return result;
}

bool MINValidator::isValid(QString min, QString type) const
{
	if(type.isEmpty())
		type = this->type(min);
	if(type.isEmpty())
		return false;
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type){
			if(plugin->isValid(min))
				return true;
			return false;
		}
	}
	return false;
}

QString MINValidator::validate(QString min, QString type) const
{
	if(type.isEmpty()){
		// try to guess type
		QStringList mayTypes = possibleTypes(min);
		if(mayTypes.size() != 1)
			return QString();
		type = mayTypes[0];
	}

	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type)
			return plugin->validate(min);
	}
	return QString();
}

QString MINValidator::convert(QString min, QString toType, QString fromType) const
{
	if(toType.isEmpty())
		return QString();
	
	if(fromType.isEmpty())
		fromType = this->type(min);
	if(fromType.isEmpty())
		return QString();
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->canConvert(toType)){
			if((fromType.isEmpty()) || (plugin->type() == fromType)){
				QString newMin = plugin->convert(min, toType);
				if(newMin.isEmpty())
					continue;
				newMin = validate(newMin, toType);
				if(newMin.isEmpty())
					continue;
				if(isValid(newMin, toType))
					return newMin;
			}
		}
	}
	return QString();
}

QString MINValidator::strip(QString min, QString type) const
{
	if(type.isEmpty())
		type = this->type(min);
	if(type.isEmpty())
		return QString();
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type)
			return plugin->strip(min);
	}
	return QString();
}

QStringList MINValidator::commonPrefixes(QStringList mins, QString type) const
{
	if(type.isEmpty()){
		foreach(QString min, mins){
			QString tmp = this->type(min);
			// all MINs must be of the same type
			if(type.isEmpty())
				type = tmp;
			else if((!type.isEmpty()) && (tmp != type))
				return QStringList();
		}
	}
	if(type.isEmpty())
		return QStringList();
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type){
			// remove not valid mins and
			// strip valid ones
			foreach(QString min, mins){
				if(!plugin->isValid(min))
					// strip out not valids
					mins.removeAll(min);
				else
					mins.replace(mins.indexOf(min), plugin->strip(min));
			}
			// remove doubles
			mins.removeDuplicates();
			if(mins.size() < 2)
				return QStringList();
			return plugin->commonPrefixes(mins);
		}
	}
	return QStringList();
}

QStringList MINValidator::minMaxPrefixes(QString min, QString type) const
{
	if(type.isEmpty())
		type = this->type(min);
	if(type.isEmpty())
		return QStringList();
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type){
			if(!plugin->isValid(min))
				return QStringList();
			else
				return plugin->minMaxPrefixes(min);
		}
	}
	return QStringList();
}

void MINValidator::initPlugins()
{
	foreach ( MINValidatorPlugin *plugin, plugins ) {
		QString type = plugin->type();
		if(!types.contains(type))
			types << type;
	}
}

void MINValidator::unloadPlugins ( )
{
	foreach(MINValidatorPlugin *object, plugins){
		if ( object ) {
			plugins.removeAll ( object );
			foreach ( EModuleListWidgetItem *item, pluginProtocol ) {
				if ( item->name() == object->type() ) {
					pluginProtocol.removeAll ( item );
					delete item;
					break;
				}
			}
			
			delete object;
		}
	}
}
