/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "personsmodel.h"
#include "personsmodel.moc"

#include "person.h"
#include <equerythread.h>

#include <QTableView>

PersonsModel::PersonsModel ( EQueryThread *thread, QObject* parent ) : EAbstractItemModel(parent)
{
	this->thread = NULL;
	setThread(thread);
	maxCombinedPersons = 0;
}

void PersonsModel::setThread(EQueryThread* thread)
{
	if(this->thread){
		this->thread->disconnect(this);
		disconnect(this->thread);
	}
	this->thread = thread;
	if(thread){
		connect(thread, SIGNAL(result(QObject*, QVariant, int, QSqlRecord)),
						this, SLOT(queryResult(QObject*, QVariant, int, QSqlRecord)));
		foreach(Person *person, items)
			person->setThread(thread);
	}
}

void PersonsModel::addPerson ( Person* person )
{
	person->setParent(this);
	person->setThread(thread);
	connect(person, SIGNAL(changed(Person*)), this, SLOT(itemChanged(Person*)));
	emit layoutAboutToBeChanged();
  items.append(person);
  emit layoutChanged();
}

Person* PersonsModel::addPerson()
{
	Person *newP = new Person(thread, this);
	addPerson(newP);
	return newP;
}

Person* PersonsModel::addPerson ( uint personId )
{
	Person *newP = addPerson();
	newP->setId(personId);
	return newP;
}

Person* PersonsModel::addPerson ( QString name, QString givenName )
{
	Person *newP = addPerson();
	newP->setName(name);
	newP->setGivenName(givenName);
	return newP;
}

Person* PersonsModel::addPerson ( uint personId, QString name, QString givenName )
{
	Person *newP = addPerson();
	newP->setId(personId, false);
	newP->setName(name, false);
	newP->setGivenName(givenName);
	return newP;
}

Person* PersonsModel::personFromIndex ( QModelIndex index ) const
{
	if(!index.isValid())
    return 0;
  
  return static_cast<Person*>(index.internalPointer());
}

QModelIndex PersonsModel::indexFromPerson ( Person* person )
{
	if(items.indexOf(person) == -1)
		return QModelIndex();
	
	return index(items.indexOf(person), 0);
}

void PersonsModel::hideColumns ( QTableView* view )
{
	if(!view)
		return;
	
	view->resizeColumnsToContents();
}

Person* PersonsModel::takePerson ( Person* person )
{
	if(items.indexOf(person) == -1)
		return 0;
	disconnect(person);
	emit layoutAboutToBeChanged();
	items.removeAll(person);
	emit layoutChanged();
	return person;
}

Person* PersonsModel::takePerson ( QModelIndex index )
{
	return takePerson(personFromIndex(index));
}

QString PersonsModel::combinePersons()
{
	
	QStringList persons;
	foreach(Person *person, items){
		if(!person)
			continue;
		persons << person->combine();
		if(maxCombinedPersons > 0 && persons.size() >= maxCombinedPersons)
			break;
	}
	if(maxCombinedPersons > 0 && rowCount() > maxCombinedPersons)
		persons.append(tr ( "more" ));
	return persons.join(tr(" and "));
}

QStringList PersonsModel::persons()
{
	QStringList persons;
	foreach(Person *person, items){
		if(!person)
			continue;
		persons << QString("%1 %2").arg(person->givenName()).arg(person->name()).trimmed();
		
		if(maxCombinedPersons > 0 && persons.size() >= maxCombinedPersons)
			break;
	}
	return persons;
}

QVariant PersonsModel::data ( const QModelIndex& index, int role ) const
{
	Person *person = personFromIndex(index);
  if(!person)
    return QVariant();
  
  if((role != Qt::DisplayRole) && (role != Qt::EditRole) && (role != Qt::ForegroundRole))
    return QVariant();
  
  if(role == Qt::ForegroundRole){
    if(person->markForRemovel())
      return QBrush(QColor(200, 200, 200));
		return QVariant();
  }
  
  switch(index.column()){
    case 0:
      return person->name();
    case 1:
      return person->givenName();
  }
  return QVariant();
}

bool PersonsModel::setData ( const QModelIndex& index, const QVariant& value, int role )
{
	Q_UNUSED(index);
	Q_UNUSED(value);
	Q_UNUSED(role);
	return false;
}

int PersonsModel::columnCount ( const QModelIndex& parent ) const
{
	if(parent.isValid())
		return 0;
	
	return 2;
}

int PersonsModel::rowCount ( const QModelIndex& parent ) const
{
	if(parent.isValid())
		return 0;
	
	return items.size();
}

QModelIndex PersonsModel::parent ( const QModelIndex& child ) const
{
	Q_UNUSED(child);
	return QModelIndex();
}

QModelIndex PersonsModel::index ( int row, int column, const QModelIndex& parent ) const
{
  if(parent.isValid())
    return QModelIndex();
  
	if(column < 0)
		return QModelIndex();
	if(columnCount() <= column)
		return QModelIndex();

  if(row < 0)
    return QModelIndex();
  if(items.size() <= row)
    return QModelIndex();
  
  return createIndex(row, column, items[row]);
}

QVariant PersonsModel::headerData ( int section, Qt::Orientation orientation, int role ) const
{
	if((role != Qt::DisplayRole) && (role != Qt::EditRole))
		return QAbstractItemModel::headerData(section, orientation, role);
	
	if(orientation == Qt::Vertical)
		return section + 1;
	
	switch(section){
		case 0:
			return tr("Name");
		case 1:
			return tr("Given Name");
	}
	return QVariant();
}

void PersonsModel::itemChanged ( Person* person )
{
	QModelIndex pIndex = indexFromPerson(person);
	emit dataChanged(pIndex, index(pIndex.row(), columnCount()));
}

void PersonsModel::queryResult(QObject* client, QVariant requestId, int querySize, QSqlRecord record)
{
	if(client != this)
		return;
	customQueryResult(requestId, querySize, record);
}

void PersonsModel::customQueryResult(QVariant requestId, int querySize, QSqlRecord record)
{
	Q_UNUSED(requestId);
	Q_UNUSED(querySize);
	Q_UNUSED(record);
}
