/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "libmeganizertestconfig.h"

#include "dbtestinit.h"
#include "dbtestinit.moc"

#include <esqldatabase.h>
#include <mediaplugin.h>
#include <searchplugin.h>

#include <QDir>
#include <QDateTime>
#include <QSettings>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QApplication>
#include <QTcpSocket>
#include <QTest>

DBTestInit::DBTestInit(QObject* parent)
	: EObject(parent)
{

}

DBTestInit::~DBTestInit()
{
	cleanupTestDB();
	ESqlDatabase::removeDatabase("defaultConnection");
	ESqlDatabase::removeDatabase(sourceConnName);
}

bool DBTestInit::initTestDB()
{
	QString timeString = QDateTime::currentDateTime().toString("yyyyMMddhhmmsszzz");
// 	connName = "test_"+ timeString;
	QString database = "test_"+ timeString;
	ESqlDatabase db = ESqlDatabase::addDatabase("QMYSQL");
	db.setUserName("test");
	db.setPassword("test");
	db.setHostName("192.168.2.1");
	db.setPort(3306);
	
	// test if server is reachable, because QSqlDatabase has a huge timeout
	// we use 2s as timeout here
	// Somehow, it doesn't work on windows
	QTcpSocket *socket = new QTcpSocket(this);
  socket->connectToHost(db.hostName(),db.port(),QIODevice::ReadOnly); 
  if (!socket->waitForConnected(2000)){
		QTest::qWarn("Database server not reachable");
		return false;
	}
	socket->close();

	if(!db.open()){
		QTest::qWarn("Could not open Database");
		return false;
	}
	
	QSqlQuery query(db);
	// create database
	if(!query.exec("CREATE DATABASE "+ database)){
		db.close();
		QTest::qWarn("Could not create new database");
		return false;
	}
	db.close();
	db.setDatabaseName(database);
	if(!db.open()){
		QTest::qWarn("Could not open connection to created database");
		return false;
	}
	
	sourceConnName = "source";
	ESqlDatabase dbSource = ESqlDatabase::addDatabase("QMYSQL", sourceConnName);
	dbSource.setUserName("test");
	dbSource.setPassword("test");
	dbSource.setHostName("192.168.2.1");
	dbSource.setDatabaseName("test_source");
	if(!dbSource.open()){
		cleanupTestDB();
		db.close();
		QTest::qWarn("Could not connect to source database "+ dbSource.databaseName().toLatin1());
		return false;
	}
	
	// create db structure
	// since db creation with setupwizard is to complex here, we copy an empty db
	QStringList dbTables = dbSource.tables ( QSql::Tables );
	QSqlQuery sourceQuery(dbSource);
	query.exec("SET foreign_key_checks = 0");
	foreach(QString table, dbTables){
		sourceQuery.exec(QString("SHOW CREATE TABLE %1").arg(table));
		if(!sourceQuery.first()){
			cleanupTestDB();
			QTest::qWarn("Could not copy table (SHOW CREATE TABLE has failed)");
			return false;
		}
		
		QString tableCreateStatement = sourceQuery.record().value("Create Table").toString();
		if(tableCreateStatement.isEmpty()){
			QTest::qWarn("Could not copy table (SHOW CREATE TABLE is empty)");
			cleanupTestDB();
			return false;
		}
		
		if(!query.exec(tableCreateStatement)){
			cleanupTestDB();
			QTest::qWarn("Could not copy table (Execution of create statement has failed)");
			return false;
		}
	}

	QStringList dbViews = sortViews(dbSource.tables ( QSql::Views ));
	foreach(QString view, dbViews){
		sourceQuery.exec(QString("SHOW CREATE VIEW %1").arg(view));
		if(!sourceQuery.first()){
			cleanupTestDB();
			QTest::qWarn("Could not copy view (SHOW CREATE VIEW has failed)");
			return false;
		}
		
		QString viewCreateStatement = sourceQuery.record().value("Create View").toString();
		if(viewCreateStatement.isEmpty()){
			cleanupTestDB();
			QTest::qWarn("Could not copy view (SHOW CREATE VIEW is empty)");
			return false;
		}
		
		if(!query.exec(viewCreateStatement)){
			cleanupTestDB();
			QTest::qWarn("Could not copy view (Execution of create statement has failed)");
			return false;
		}
	}
	query.exec("SET foreign_key_checks = 1");
	
	// prepare user entry
	if(!query.exec("INSERT INTO meganizer_users (name, givenName, userName) VALUES('Testing', 'More', 'test')")){
		cleanupTestDB();
		QTest::qWarn("Could not insert test user");
		return false;
	}
	
	return true;
}

bool DBTestInit::cleanupTestDB()
{
	ESqlDatabase db = ESqlDatabase::database();
	if(!db.open())
		return false;
	
	QSqlQuery query(db);
	// create database
	if(!query.exec("DROP DATABASE "+ db.databaseName())){
		db.close();
		return false;
	}
	db.close();
	connName = QString();
	return true;
}

QStringList DBTestInit::sortViews(QStringList views)
{
	ESqlDatabase db = ESqlDatabase::database(sourceConnName);
	
	QRegExp fromRegExp("from [\\(]+`([\\w\\d]+)`[\\s\\.]{1}");
	QRegExp joinRegExp("join `([\\w\\d]+)`[\\s\\.]{1}");
	QMultiMap<QString, QString> dependencies;
	QStringList result;
	
	foreach(QString view, views){
		QSqlQuery query(db);
	
		// dump create statement
		query.exec(QString("SHOW CREATE VIEW %1").arg(view));
		if(!query.first())
			return result;
	
		QString viewCreateStatement = query.record().value("Create View").toString();
		if(viewCreateStatement.isEmpty())
			return result;
		
		QStringList deps;
		QStringList fromList = viewCreateStatement.split(QRegExp("from [\\(]+`"));
		// the first one is not interesting, because it's the part in front of the from
		fromList.removeFirst();
		foreach(QString from, fromList){
			QString subStr = from.left(from.indexOf("`"));
			if(!subStr.isEmpty())
				deps << subStr;
		}
		
		QStringList joinList = viewCreateStatement.split(QRegExp("join `"));
		joinList.removeFirst();
		foreach(QString join, joinList){
			QString subStr = join.left(join.indexOf("`"));
			if(!subStr.isEmpty())
				deps << subStr;
		}
		
		// insert all views to the list
		foreach(QString dep, deps){
			if(views.contains(dep))
				dependencies.insert(view, dep);
		}
		
		QApplication::processEvents();
	}
	
	// sorting views
	// first put all without any dependencies
	foreach(QString view, views){
		if(dependencies.values(view).size() == 0){
			result << view;
			views.removeAll(view);
		}
		QApplication::processEvents();
	}
	
	// try to add all other in some rounds
	// we try 15 times all views, if this is not enough, we give up
	int maxTries = views.size() * 15;
	int i = 0;
	
	QStringList::const_iterator iter;
	iter = views.constBegin();
	while(views.size() > 0){
		if(i >= maxTries){
			break;
		}
		if(iter == views.constEnd())
			iter = views.constBegin();
		
		bool allFound = true;
		foreach(QString dep, dependencies.values((*iter))){
			if(result.indexOf(dep) < 0)
				allFound = false;
		}
		if(allFound){
			result << (*iter);
			views.removeAll((*iter));
		}

		iter++;
		i++;
		
		QApplication::processEvents();
	}
	
	return result;
}

MediumKeys DBTestInit::prepareMediumKeys(QString materialTypeName)
{
	MediumKeys result;
	result.valid = false;
	
	// insert dummy items
	ESqlDatabase db = ESqlDatabase::database();
	if(!db.open()){
		db.close();
		return result;
	}
	
	QSqlQuery query;
	bool success = true;
	
	result.testGenre = "Testgenre";
	if(!query.exec("INSERT INTO meganizer_genres (name, description) VALUES('"+ result.testGenre +"', 'This is only for testing purposes')"))
		success = false;
	query.exec("SELECT id FROM meganizer_genres WHERE name='"+ result.testGenre +"'");
	if(!query.first())
		success = false;
	result.testGenreId = query.record().value("id").toUInt();
	
	result.testManu = "Testmanu";
	if(!query.exec("INSERT INTO meganizer_manufacturers (name) VALUES('"+ result.testManu +"')"))
		success = false;
	query.exec("SELECT id FROM meganizer_manufacturers WHERE name='"+ result.testManu +"'");
	if(!query.first())
		success = false;
	result.testManuId = query.record().value("id").toUInt();
	
	result.testType = materialTypeName;
	if(!query.exec("INSERT INTO meganizer_materialtypes (name) VALUES('"+ result.testType +"')"))
		success = false;
	query.exec("SELECT id FROM meganizer_materialtypes WHERE name='"+ result.testType +"'");
	if(!query.first())
		success = false;
	result.testTypeId = query.record().value("id").toUInt();
	
	result.testCur = "Euro";
	result.testCurShort = "EUR";
	if(!query.exec("INSERT INTO meganizer_currencies (name, shortName) VALUES('"+ result.testCur +"', '"+ result.testCurShort +"')"))
		success = false;
	query.exec("SELECT id FROM meganizer_currencies WHERE name='"+ result.testCur +"'");
	if(!query.first())
		success = false;
	result.testCurId = query.record().value("id").toUInt();
	
	result.testUser = "test";
	query.exec("SELECT id FROM meganizer_users WHERE userName='"+ result.testUser +"'");
	if(!query.first())
		success = false;
	result.testUserId = query.record().value("id").toUInt();
	
	db.close();
	
	result.valid = success;
	result.invalid = "asdasdasdasdasd";
	result.invalidId = 11111111;
	
	return result;
}
