/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumsimpleeditwidgetbasetest.h"
#include "mediumsimpleeditwidgetbasetest.moc"

#include <mediumsimpleeditwidget.h>
#include <medium.h>
#include "testplugindir.h"
#include <epluginloader.h>
#include <mediaplugin.h>
#include <mediamodel.h>

#include <QTest>
#include <QSpinBox>
#include <QDateEdit>
#include <QTimeEdit>
#include <QLineEdit>
#include <QPlainTextEdit>
#include <QCheckBox>
#include <QSignalSpy>

#include <QDebug>

MediumSimpleEditWidgetBaseTest::MediumSimpleEditWidgetBaseTest()
{
	testMed = NULL;
}

MediumSimpleEditWidgetBaseTest::~MediumSimpleEditWidgetBaseTest()
{
	if(testMed)
		delete testMed;
}

void MediumSimpleEditWidgetBaseTest::addTestObject(MediumSimpleEditWidget* widget, const char* property)
{
	if(QString(property).isEmpty())
		return;
	if(!widget)
		return;
	map.insert(property, widget);
}

void MediumSimpleEditWidgetBaseTest::baseTest(QString materialType)
{
	if(!materialType.isEmpty())
		setTestMedium(materialType);
	QVERIFY(testMedium());
	foreach(const char* property, map.uniqueKeys())
		baseTest(map.value(property), property);

	map.clear();
}

void MediumSimpleEditWidgetBaseTest::baseTest(MediumSimpleEditWidget* widget, const char* property)
{
	if(!testMed)
		QSKIP("Please setup a test medium", SkipAll);

	qDebug() << QString(property);
	// test whether property exists
	QVERIFY(testMed->metaObject()->indexOfProperty(property) > -1);
	QVERIFY(widget);

	// test with NULL medium pointer
	QSignalSpy spy(widget, SIGNAL(error(QString)));
	widget->setMedium(NULL);
	QCOMPARE(widget->isEnabled(), false);
	QCOMPARE(widget->store(), false);
	QCOMPARE(spy.count(), 1);

	// test setMedium function
	widget->setMedium(testMed);
	// if setMedium was successfull, it must be enabled
	QCOMPARE(widget->isEnabled(), true);
	QCOMPARE(widget->modified(), false);

	// edit the data
	QVariant value = setValue(widget, property, testMedium());
	QVERIFY(value.isValid());
	QVERIFY(widget->modified());
	QSignalSpy spy1(widget, SIGNAL(error(QString)));
	bool store = widget->store();
	if(spy1.count() > 0)
		QTest::qWarn(spy.takeFirst()[0].toString().toLatin1());
	QCOMPARE(store, true);
	QCOMPARE(testMed->property(property), value);
	delete widget;
}

QVariant MediumSimpleEditWidgetBaseTest::setValue(MediumSimpleEditWidget* editWidget, const char* property, Medium *testedMed)
{
	if(!testedMed){
		if(!testMedium())
			return QVariant();
		else
			testedMed = testMedium();
	}
	
	QWidget *widget = editWidget->findChild<QWidget*>(QString(property) +"Edit");
	if(!widget)
		return QVariant();
	QSpinBox *spinBox = qobject_cast<QSpinBox*>(widget);
	if(spinBox){
		// we must decide between uint and int
		QMetaProperty meta = testedMed->metaObject()->property(testedMed->metaObject()->indexOfProperty(property));
		if(!meta.isValid())
			return QVariant();
		QVariant value = 10;
		value.convert(meta.type());
		QTest::keyClick(spinBox, Qt::Key_Enter);
		QTest::keyClicks(spinBox, value.toString());
		return value;
	}
	QDateEdit *dateEdit = qobject_cast<QDateEdit*>(widget);
	if(dateEdit){
		QDate date = QDate::fromString("11/1911", "MM/yyyy");
		QTest::keyClick(dateEdit, Qt::Key_Enter);
		QTest::keyClicks(dateEdit, date.toString("MMyyyy"));
		return date;
	}
	QTimeEdit *timeEdit = qobject_cast<QTimeEdit*>(widget);
	if(timeEdit){
		QTime time = QTime::currentTime();
		time.addSecs(5600);
		QTest::keyClick(timeEdit, Qt::Key_Enter);
		QTest::keyClicks(timeEdit, time.toString("HHmmss"));
		// to avoid failed compares because of milliseconds
		if(timeEdit->dateTime().toString("HH:mm:ss") == time.toString("HH:mm:ss"))
			time = QTime::fromString(timeEdit->dateTime().toString("HH:mm:ss:zzz"));
		return time;
	}
	QLineEdit *lineEdit = qobject_cast<QLineEdit*>(widget);
	if(lineEdit){
		QString value = "this is a test string";
		QTest::keyClicks(lineEdit, value);
		return value;
	}
	QPlainTextEdit *plainEdit = qobject_cast<QPlainTextEdit*>(widget);
	if(plainEdit){
		QString value = "this is a test string. with a line break into it";
		QTest::keyClicks(plainEdit, value);
		return value;
	}
	QCheckBox *checkBox = qobject_cast<QCheckBox*>(widget);
	if(checkBox){
		QTest::keyClick(checkBox, Qt::Key_Space);
		return true;
	}
	return QVariant();
}

void MediumSimpleEditWidgetBaseTest::setTestMedium(QString materialType)
{
	if(materialType.isEmpty())
		return;

	QList<EModuleListWidgetItem*> log;
	QList<MediaPlugin*> plugins = EPluginLoader<MediaPlugin>::loadPlugins(TestPluginDir::libDir(), log);
	foreach(MediaPlugin *plugin, plugins){
		if(plugin->isSupportedMaterialType(materialType)){
			setTestMedium(plugin->newMedium(materialType));
			break;
		}
	}
}

void MediumSimpleEditWidgetBaseTest::setTestMedium(Medium* med)
{
	if(testMed)
		delete testMed;
	testMed = med;
}

Medium* MediumSimpleEditWidgetBaseTest::testMedium()
{
	return testMed;
}

