/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "book.h"
#include "book.moc"

#include "books.h"
#include "bookedit.h"
#include <materialpersonsmodel.h>
#include <esqldatabase.h>
#include <ewidget.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QTextTable>
#include <QTextCursor>

Book::Book ( EQueryThread *thread, Medium *newParent )
	: Medium ( tr("Book"), "book", Books::group() +"_viewbooks", thread, newParent )
{
  myEdition = 0;
  myPages = 0;
}

#if QT_VERSION < 0x040500
Medium* Book::newMedium()
{
  return new Book();
}
#endif

bool Book::select ( QSqlRecord record )
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = false;
  if ( !db.isOpen() )
    closeDB = true;

  if ( !Medium::select ( record ) )
    return false;

  if ( !record.contains ( "pages" ) ) {
    if ( !db.open() )
      return false;

    QSqlQuery replaceRecord;
    replaceRecord.exec
    (
      QString ( "SELECT * FROM %1_books WHERE materialsId=%2" )
      .arg ( Books::group() )
      .arg ( id() )
    );
    if ( !replaceRecord.first() ) {
      if ( closeDB )
        db.close();
      return false;
    }
    record = replaceRecord.record();
  }

  setPages ( record.value ( "pages" ).toUInt() );

  setEdition ( record.value ( "edition" ).toUInt() );

  setPublicationPlace ( record.value ( "publicationPlace" ).toString() );

  setPublicationDate ( record.value ( "publicationDate" ).toDate() );

	if(closeDB)
		db.close();
	return true;
}

bool Book::store ( bool addOwner, bool startTransaction )
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
  if ( closeDB )
    db.open();
  if ( !db.isOpen() )
    return false;

  bool transaction = false;
  if ( startTransaction )
    transaction = transactionsSupported;
  if ( transaction )
    db.transaction();

  if ( !Medium::store ( addOwner, false ) ) {
    if ( transaction )
      if ( !db.rollback() )
        emit error ( tr ( "Rolling back failed!" ) );
    if ( closeDB )
      db.close();
    return false;
  }

  // id is needed and should be set by Medium::store()
  if ( id() == 0 ) {
    if ( transaction )
      if ( !db.rollback() )
        emit error ( tr ( "Rolling back failed!" ) );
    if ( closeDB )
      db.close();
    return false;
  }



  QSqlQuery query;
  query.exec ( QString ( "SELECT id FROM %1_books WHERE materialsId=%2" ).arg ( Books::group() ).arg ( id() ) );
  bool isUpdate = false;
  if ( query.first() )
    isUpdate = true;

  if ( isUpdate ) {
    if
    (
      !query.prepare
      (
        QString
        (
          "UPDATE %1_books SET "
          "pages=:pages, edition=:edition, "
          "publicationPlace=:publicationPlace, publicationDate=:publicationDate "
          "WHERE materialsId=:materialsId"
        )
        .arg ( Books::group() )
      )
    ) {
      if ( transaction )
        if ( !db.rollback() )
          emit error ( tr ( "Rolling back failed!" ) );
      if ( closeDB )
        db.close();
      return false;
    }
  } else {
    if
    (
      !query.prepare
      (
        QString
        (
          "INSERT INTO %1_books "
          "(materialsId, pages, edition, publicationPlace, publicationDate) "
          "VALUES(:materialsId, :pages, :edition, :publicationPlace, :publicationDate)"
        )
        .arg ( Books::group() )
      )
    ) {
      if ( transaction )
        if ( !db.rollback() )
          emit error ( tr ( "Rolling back failed!" ) );
      if ( closeDB )
        db.close();
      return false;
    }
  }
  query.bindValue ( ":materialsId", id() );
  query.bindValue ( ":pages", pages() );
  query.bindValue ( ":edition", edition() );
  query.bindValue ( ":publicationPlace", publicationPlace() );
  query.bindValue ( ":publicationDate", publicationDate().toString ( "yyyy-MM-dd" ) );

  if ( !query.exec() ) {
    if ( transaction )
      if ( !db.rollback() )
        emit error ( tr ( "Rolling back failed!" ) );
    if ( closeDB )
      db.close();
    return false;
  }

  if ( transaction )
    if ( !db.commit() )
      emit error ( tr ( "Could not store medium! Commit has failed!" ) );
  if ( closeDB )
    db.close();
  return true;
}

bool Book::setValue ( QString attribute, QVariant value )
{
	if(attribute == "edition"){
		setEdition(value.toUInt());
		return true;
	}
	
	if(attribute =="pubPlace"){
		setPublicationPlace(value.toString());
		return true;
	}
	
	if(attribute == "pubDate"){
		setPublicationDate(value.toDate());
		return true;
	}

	if(attribute == "pages"){
		setPages(value.toUInt());
		return true;
	}
	return Medium::setValue(attribute, value);
}

QVariant Book::value ( QString attribute, int role )
{
	if ( role == Qt::DisplayRole ){
		if(attribute == "edition")
			return edition();
		
		if(attribute == "pubPlace")
			return publicationPlace();
		
		if(attribute == "pubDate")
			return publicationDate();
		
		if(attribute == "pages")
			return pages();
	}
	return Medium::value(attribute, role);
}

EditMedium *Book::editWidget(EWidget* parent)
{
  return new BookEdit ( parent );
}

void Book::setPages ( uint newPages )
{
  myPages = newPages;
}

uint Book::pages() const
{
  return myPages;
}

void Book::setEdition ( uint newEdition )
{
  myEdition = newEdition;
}

uint Book::edition() const
{
  return myEdition;
}

void Book::setPublicationPlace ( QString newPlace )
{
  myPublicationPlace = newPlace;
}


QString Book::publicationPlace() const
{
  return myPublicationPlace;
}

void Book::setPublicationDate ( QDate newDate )
{
  myPublicationDate = newDate;
}

QDate Book::publicationDate() const
{
  return myPublicationDate;
}


Medium* Book::copyMedium()
{
	Book *med = qobject_cast<Book*>(Medium::copyMedium());
	if(!med)
		return 0;

	med->setEdition(edition());
	med->setPages(pages());
	med->setPublicationPlace(publicationPlace());
	med->setPublicationDate(publicationDate());
	return med;
}

bool Book::exportTextTable ( QTextTable* table, Medium::ExportFormat format, bool titleOnly )
{
	if(format == Bibtex)
		return false;
	
  if ( !table )
    return false;

	if(compatibilityMode() && format == Table)
		return Medium::exportTextTable(table, format, titleOnly);

  // collecting the data which should be inserted
  QStringList values;
  if ( titleOnly ) {
    values << attrs ( "title" ).translation;
    values << attrs ( "persons" ).translation;
    values << attrs ( "edition" ).translation;
    values << attrs ( "manufacturer" ).translation;
    values << attrs ( "min" ).translation;
    values << attrs ( "pages" ).translation;
    values << tr ( "Wish" );
  } else {
    values << title();
    values << materialPersonsCombined();
    values << QString::number ( edition() );
    values << manufacturerName();
    values << minsCombined();
    values << QString::number ( pages() );
    if ( wish() )
      values << tr ( "Yes" );
  }

  // insert the place at the table we need
  if ( !titleOnly ) {
    // the first row is present at the table, because empty table can not be created
#if QT_VERSION < 0x040500
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
  }
  if ( table->columns() < values.size() ) {
#if QT_VERSION < 0x040500
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
  }
  int row = table->rows() - 1;
  bool result = true;
  for ( int i = 0; i < values.size(); i++ ) {
    QTextTableCell cell = table->cellAt ( row, i );
    QTextCursor cursor = cell.firstCursorPosition();
    cursor.insertText ( values[i] );
  }
  return result;
}

QString Book::exportPlainText(Medium::ExportFormat format)
{
	if(format != Bibtex)
		return QString();

	QStringList personsList = materialPersons()->persons();
	QString key = publicationDate().toString("yyyy");
	if(personsList.size() > 0)
		key = personsList[0].left(3).toUpper() + key;

	// standard format is misc
	QString result;
	result  = "@book{"+ bibtexStrip(key) +",\n";
	result += "	title = {"+ bibtexStrip(title()) +"},\n";
	result += "	author = {"+ bibtexStrip(personsList.join(" and ")) +"},\n";
	result += "	publisher = {"+ bibtexStrip(manufacturerName()) +"},\n";
	result += "	year = {"+ publicationDate().toString("yyyy") +"}\n";
	result += "}\n";
	return result;
}

QList<ColKeyTrans> Book::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();

	QList<ColKeyTrans> result;
  result << Medium::attrs("source", 0);
	result << Medium::attrs("title", 1);
	result << Medium::attrs("persons", 2);
	result << ColKeyTrans(3, "edition", tr ( "Edition" ));
  result << ColKeyTrans(4, "manufacturer", tr ( "Publisher" ));
	result << Medium::attrs("genre", 5);
	result << ColKeyTrans(6, "pubPlace", tr ( "Pub. Place" ));
  result << ColKeyTrans(7, "pubDate",  tr ( "Pub. Date" ));
	result << Medium::attrs("min", 8);
  result << ColKeyTrans(9, "pages",  tr ( "Pages" ));
	result << Medium::attrs("listPrice", 10);
	result << Medium::attrs("listPriceCurrency", 11);
	result << Medium::attrs("description", 12);
	result << Medium::attrs("owner", 13);
  return result;
}
