/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "article.h"
#include "article.moc"

#include "magazines.h"
#include <esqldatabase.h>
#include "articleedit.h"
#include <ewidget.h>
#include "magazine.h"
#include <materialpersonsmodel.h>

#include <QSqlQuery>
#include <QTextTable>
#include <QTextCursor>
#include <QTextTableCell>

Article::Article(EQueryThread* thread, Medium* newParent)
	: Medium(tr("Article"), "article", Magazines::group() +"_viewarticles", thread, newParent)
{
	dbTable = Magazines::group() +"_articles";
	myStartPage = 0;
	myEndPage = 0;
}

#if QT_VERSION < 0x040500
Medium* Article::newMedium()
{
	return Article();
}
#endif

bool Article::select(QSqlRecord record)
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = false;
	if ( !db.isOpen() )
		closeDB = true;

	if ( !Medium::select ( record ) )
		return false;

	if ( !record.contains ( "startPage" ) ) {
		if ( !db.open() )
			return false;

		QSqlQuery replaceRecord;
		replaceRecord.exec
		(
			QString ( "SELECT * FROM %1 WHERE materialsId=%2" )
			.arg (dbTable )
			.arg ( id() )
		);
		if ( !replaceRecord.first() ) {
			if ( closeDB )
				db.close();
			return false;
		}
		record = replaceRecord.record();
	}

	setStartPage( record.value("startPage").toUInt() );
	setEndPage( record.value("endPage").toUInt() );
	setPublicationDate( record.value("publicationDate").toDate() );

	if(closeDB)
		db.close();
	return true;
}

bool Article::store(bool addOwner, bool startTransaction)
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		db.open();
	if ( !db.isOpen() )
		return false;

	bool transaction = false;
	if ( startTransaction )
		transaction = transactionsSupported;
	if ( transaction )
		db.transaction();

	if ( !Medium::store ( addOwner, false ) ) {
		if ( transaction )
			if ( !db.rollback() )
				emit error ( tr ( "Rolling back failed!" ) );
		if ( closeDB )
			db.close();
		return false;
	}

	// id is needed and should be set by Medium::store()
	if ( id() == 0 ) {
		if ( transaction )
			if ( !db.rollback() )
				emit error ( tr ( "Rolling back failed!" ) );
		if ( closeDB )
			db.close();
		return false;
	}

	QSqlQuery query;
	query.exec ( QString ( "SELECT id FROM %1 WHERE materialsId=%2" ).arg ( dbTable ).arg ( id() ) );
	bool isUpdate = false;
	if ( query.first() )
		isUpdate = true;

	if ( isUpdate ) {
		if
		(
			!query.prepare
			(
				QString
				(
					"UPDATE %1 SET"
					" startPage=:startPage, endPage=:endPage, publicationDate=:publicationDate"
					" WHERE materialsId=:materialsId"
				)
				.arg ( dbTable )
			)
		) {
			if ( transaction )
				if ( !db.rollback() )
					emit error ( tr ( "Rolling back failed!" ) );
			if ( closeDB )
				db.close();
			return false;
		}
	} else {
		if
		(
			!query.prepare
			(
				QString(
					"INSERT INTO %1"
					" (materialsId, startPage, endPage, publicationDate) "
					" VALUES(:materialsId, :startPage, :endPage, :publicationDate)"
				)
				.arg ( dbTable )
			)
		) {
			if ( transaction )
				if ( !db.rollback() )
					emit error ( tr ( "Rolling back failed!" ) );
			if ( closeDB )
				db.close();
			return false;
		}
	}
	query.bindValue ( ":materialsId", id() );
	query.bindValue ( ":startPage", startPage() );
	query.bindValue ( ":endPage", endPage() );
	query.bindValue ( ":publicationDate", publicationDate() );

	if ( !query.exec() ) {
		if ( transaction )
			if ( !db.rollback() )
				emit error ( tr ( "Rolling back failed!" ) );
		if ( closeDB )
			db.close();
		return false;
	}

	if ( transaction ){
		if ( !db.commit() ){
			emit error ( tr ( "Could not store medium! Commit has failed!" ) );
			if ( closeDB )
				db.close();
			return false;
		}
	}
	if ( closeDB )
		db.close();
	return true;
}

bool Article::setValue(QString attribute, QVariant value)
{
	if(attribute == "startPage"){
		setStartPage(value.toUInt());
		return true;
	}
	if(attribute == "endPage"){
		setEndPage(value.toUInt());
		return true;
	}
	if(attribute == "pubDate"){
		setPublicationDate(value.toDate());
		return true;
	}
	return Medium::setValue(attribute, value);
}

QVariant Article::value(QString attribute, int role)
{
	if ( role == Qt::DisplayRole ){
		if(attribute == "startPage")
			return startPage();
		if(attribute == "endPage")
			return endPage();
		if(attribute == "pubDate")
			return publicationDate();
		if(attribute == "pages")
			return pages();
	}
	return Medium::value(attribute, role);
}

QList< ColKeyTrans > Article::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();

	QList<ColKeyTrans> result;
  result << Medium::attrs("source", 0);
	result << Medium::attrs("title", 1);
	result << Medium::attrs("persons", 2);
  result << ColKeyTrans(3, "manufacturer", tr ( "Publisher" ));
	result << Medium::attrs("genre", 4);
  result << ColKeyTrans(5, "pubDate",  tr ( "Pub. Date" ));
	result << Medium::attrs("min", 6);
  result << ColKeyTrans(7, "pages",  tr ( "Pages" ));
	result << Medium::attrs("owner", 8);
  return result;
}

EditMedium* Article::editWidget(EWidget* parent)
{
	return new ArticleEdit(parent);
}

Medium* Article::copyMedium()
{
	Article *med = qobject_cast<Article*>(Medium::copyMedium());
	if(!med)
		return 0;

	med->setStartPage(startPage());
	med->setEndPage(endPage());
	med->setPublicationDate(publicationDate());
	return med;
}

bool Article::exportTextTable(QTextTable* table, Medium::ExportFormat format, bool titleOnly)
{
	if(format == Bibtex)
		return false;
	
	if ( !table )
		return false;

	if(compatibilityMode() && format == Table)
		return Medium::exportTextTable(table, format, titleOnly);

	// collecting the data which should be inserted
	QStringList values;
	if ( titleOnly ) {
		values << attrs ( "title" ).translation;
		values << attrs ( "persons" ).translation;
		values << attrs ( "manufacturer" ).translation;
		values << attrs ( "genre" ).translation;
		values << attrs ( "pubDate" ).translation;
		values << attrs ( "min" ).translation;
		values << attrs ( "pages" ).translation;
		values << tr ( "Wish" );
	} else {
		values << title();
		values << materialPersonsCombined();
		values << manufacturerName();
		values << genreName();
		values << publicationDate().toString(tr("MM/yyyy"));
		values << minsCombined();
		values << QString::number ( pages() );
		if ( wish() )
			values << tr ( "Yes" );
	}

	// insert the place at the table we need
	if ( !titleOnly ) {
    // the first row is present at the table, because empty table can not be created
#if QT_VERSION < 0x040500
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
	}
	if ( table->columns() < values.size() ) {
#if QT_VERSION < 0x040500
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
	}
	int row = table->rows() - 1;
	bool result = true;
	for ( int i = 0; i < values.size(); i++ ) {
		QTextTableCell cell = table->cellAt ( row, i );
		QTextCursor cursor = cell.firstCursorPosition();
		cursor.insertText ( values[i] );
	}
	return result;
}

QString Article::exportPlainText(Medium::ExportFormat format)
{
	if(format != Bibtex)
		return QString();

	// determine journal, where this article is published in
	Magazine  parent;
	if(id() > 0){
		ESqlDatabase db = ESqlDatabase::database();
		bool closeDB = !db.isOpen();
		if(closeDB)
			db.open();
		if(db.isOpen()){
			QSqlQuery query;
			query.prepare("SELECT materialsId FROM meganizer_materialhasmaterials WHERE hasMaterialsId=:hasMaterialsId");
			query.bindValue(":hasMaterialsId", id());
			query.exec();
			if(query.first())
				parent.load(query.record().value("materialsId").toUInt());
			if(closeDB)
				db.close();
		}
	}
	
	QStringList personsList = materialPersons()->persons();
	QString key = publicationDate().toString("yyyy");
	if(personsList.size() > 0)
		key = personsList[0].left(3).toUpper() + key;

	// standard format is misc
	QString result;
	result  = "@article{"+ bibtexStrip(key) +",";
	result += "	title = {"+ bibtexStrip(title()) +"},\n";
	result += "	author = {"+ bibtexStrip(personsList.join(" and ")) +"},\n";
	result += "	pages = {"+ QString::number(pages()) +"},\n";
	result += "	year = {"+ publicationDate().toString("yyyy") +"},\n";
	result += "	journal = {"+ bibtexStrip(parent.title()) +"},\n";
	result += "	volume = {"+ bibtexStrip(parent.issue()) +"}\n";
	result += "}\n";
	return result;
}

void Article::setStartPage(uint page)
{
	myStartPage = page;
}

uint Article::startPage() const
{
	return myStartPage;
}

void Article::setEndPage(uint page)
{
	myEndPage = page;
}

uint Article::endPage() const
{
	return myEndPage;
}

uint Article::pages() const
{
	if(endPage() > startPage())
		return endPage() - startPage();
	return 0;
}

void Article::setPublicationDate(QDate newDate)
{
	myPubDate = newDate;
}

QDate Article::publicationDate() const
{
	return myPubDate;
}
