/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "magazine.h"
#include "magazine.moc"

#include "magazines.h"
#include <esqldatabase.h>
#include "magazineedit.h"
#include <ewidget.h>
#include "articlesmodel.h"
#include "article.h"

#include <QSqlQuery>
#include <QTextTable>
#include <QTextCursor>
#include <QTextTableCell>

#include <QDebug>

Magazine::Magazine(EQueryThread* thread, Medium* newParent)
	: Medium(tr("Magazine"), "magazine", Magazines::group() +"_viewmagazines", thread, newParent)
{
	dbTable = Magazines::group() +"_magazines";
	setChildModel(new ArticlesModel ( thread, this ));
}

#if QT_VERSION < 0x040500
Medium* Magazine::newMedium()
{
	return Magazine(thread);
}
#endif

bool Magazine::select(QSqlRecord record)
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = false;
	if ( !db.isOpen() )
		closeDB = true;

	if ( !Medium::select ( record ) )
		return false;

	if ( !record.contains ( "issue" ) ) {
		if ( !db.open() )
			return false;

		QSqlQuery replaceRecord;
		replaceRecord.exec
		(
			QString ( "SELECT * FROM %1 WHERE materialsId=%2" )
			.arg ( dbTable )
			.arg ( id() )
		);
		if ( !replaceRecord.first() ) {
			if ( closeDB )
				db.close();
			return false;
		}
		record = replaceRecord.record();
	}

	setIssue ( record.value ( "issue" ).toString() );
	setPublicationDate( record.value ( "publicationDate" ).toDate() );

	if(closeDB)
		db.close();
	return true;
}

bool Magazine::prepare()
{
	if(title().isEmpty()){
		EditMedium *edit = editWidget();
		edit->setAddMode(this, false);
		if(edit->exec() != QDialog::Accepted)
			return false;
 		return prepare();
	}

	if(checkKeys())
		return true;

	// try to fetch all stuff withhelp of the title
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		if( !db.open() )
			return false;

	QSqlQuery query;
	query.prepare("SELECT * FROM "+ mySelectTable +" WHERE materialtypesId=:materialtypesId AND title=:title AND issue=:issue");
	query.bindValue(":materialtypesId", type());
	query.bindValue(":title", title());
	query.bindValue(":issue", issue());
	query.exec();
	if(query.size() == 1){
		if(query.first()){
			// fetch all needed stuff
			select(query.record());
		}
	}
	if(closeDB)
		db.close();

	if(checkKeys())
		return true;
	
	EditMedium *edit = editWidget();
	edit->setAddMode(this, false);
	if(edit->exec() != QDialog::Accepted)
		return false;
	return prepare();
}

bool Magazine::store(bool addOwner, bool startTransaction)
{
	bool isInsert = false;
	if(id() == 0)
		isInsert = true;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		db.open();
	if ( !db.isOpen() )
		return false;

	bool transaction = false;
	if ( startTransaction )
		transaction = transactionsSupported;
	if ( transaction )
		db.transaction();

	if ( !Medium::store ( addOwner, false ) ) {
		if ( transaction )
			if ( !db.rollback() )
				emit error ( tr ( "Rolling back failed!" ) );
		if ( closeDB )
			db.close();
		return false;
	}

	// id is needed and should be set by Medium::store()
	if ( id() == 0 ) {
		if ( transaction )
			if ( !db.rollback() )
				emit error ( tr ( "Rolling back failed!" ) );
		if ( closeDB )
			db.close();
		return false;
	}

	QSqlQuery query;
	query.exec ( QString ( "SELECT id FROM %1 WHERE materialsId=%2" ).arg ( dbTable ).arg ( id() ) );
	bool isUpdate = false;
	if ( query.first() )
		isUpdate = true;

	if ( isUpdate ) {
		if
		(
			!query.prepare
			(
				QString
				(
					"UPDATE %1 SET"
					" issue=:issue, publicationDate=:publicationDate"
					" WHERE materialsId=:materialsId"
				)
				.arg ( dbTable )
			)
		) {
			if ( transaction )
				if ( !db.rollback() )
					emit error ( tr ( "Rolling back failed!" ) );
			if ( closeDB )
				db.close();
			return false;
		}
	} else {
		if
		(
			!query.prepare
			(
				QString
				(
					"INSERT INTO %1"
					" (materialsId, issue, publicationDate) "
					" VALUES(:materialsId, :issue, :publicationDate)"
				)
				.arg ( dbTable )
			)
		) {
			if ( transaction )
				if ( !db.rollback() )
					emit error ( tr ( "Rolling back failed!" ) );
			if ( closeDB )
				db.close();
			return false;
		}
	}
	query.bindValue ( ":materialsId", id() );
	query.bindValue ( ":issue", issue() );
	query.bindValue ( ":publicationDate", publicationDate() );

	if ( !query.exec() ) {
		if ( transaction )
			if ( !db.rollback() )
				emit error ( tr ( "Rolling back failed!" ) );
		if ( closeDB )
			db.close();
		return false;
	}

	if ( transaction )
		if ( !db.commit() )
			emit error ( tr ( "Could not store medium! Commit has failed!" ) );
	if ( closeDB )
		db.close();
	return true;
}

bool Magazine::setValue(QString attribute, QVariant value)
{
	if(attribute == "issue"){
		setIssue(value.toString());
		return true;
	}
	if(attribute == "publicationDate"){
		setPublicationDate(value.toDate());
		return true;
	}

	if(attribute == "article"){
		Article *article = NULL;
		QStringList items = value.toStringList();
		if(items.size() != 3)
			return false;
		if(items[0].isEmpty())
			return false;

		MediaModel *model = childs();
		if(!model)
			return false;

		article = qobject_cast<Article*>(model->addEmptyMedium());
		if(!article)
			return false;
		article->setTitle(items[0]);
		article->setStartPage(items[1].toUInt());
		article->setEndPage(items[2].toUInt());
		article->setManufacturer(manufacturer(), false);
		article->setManufacturer(manufacturerName(), false);
		article->setPublicationDate(publicationDate());
		article->setGenre(genre(), false);
		article->setGenre(genreName(), false);

		return true;
	}
	
	return Medium::setValue(attribute, value);
}

QVariant Magazine::value(QString attribute, int role)
{
	if ( role == Qt::DisplayRole ){
		if(attribute == "issue")
			return issue();
		if(attribute == "pubDate")
			return publicationDate();
	}
	return Medium::value(attribute, role);
}

QList< ColKeyTrans > Magazine::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();

	QList<ColKeyTrans> result;
	result << Medium::attrs("source", 0);
	result << Medium::attrs("title", 1);
	result << ColKeyTrans(2, "issue", tr ( "Issue" ));
	result << ColKeyTrans(3, "manufacturer", tr ( "Publisher" ));
	result << Medium::attrs("genre", 4);
	result << ColKeyTrans(5, "pubDate",  tr ( "Pub. Date" ));
	result << Medium::attrs("min", 6);
	result << Medium::attrs("owner", 7);
  return result;
}

EditMedium* Magazine::editWidget(EWidget* parent)
{
	return new MagazineEdit(parent);
}

Medium* Magazine::copyMedium()
{
	Magazine *med = qobject_cast<Magazine*>(Medium::copyMedium());
	if(!med)
		return NULL;

	med->setIssue(issue());
	med->setPublicationDate(publicationDate());
	return med;
}

bool Magazine::exportTextTable(QTextTable* table, Medium::ExportFormat format, bool titleOnly)
{
	if(format == Bibtex)
		return false;
	
	if ( !table )
		return false;

	if(compatibilityMode() && format == Table)
		return Medium::exportTextTable(table, format, titleOnly);

	// collecting the data which should be inserted
	QStringList values;
	if ( titleOnly ) {
		values << attrs ( "title" ).translation;
		values << attrs ( "issue" ).translation;
		values << attrs ( "manufacturer" ).translation;
		values << attrs ( "genre" ).translation;
		values << attrs ( "pubDate" ).translation;
		values << attrs ( "min" ).translation;
		values << tr ( "Wish" );
	} else {
		values << title();
		values << issue();
		values << manufacturerName();
		values << genreName();
		values << publicationDate().toString(tr("MM/yyyy"));
		values << minsCombined();
		if ( wish() )
			values << tr ( "Yes" );
	}

  // insert the place at the table we need
  if ( !titleOnly ) {
    // the first row is present at the table, because empty table can not be created
#if QT_VERSION < 0x040500
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
  }
  if ( table->columns() < values.size() ) {
#if QT_VERSION < 0x040500
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
  }
  int row = table->rows() - 1;
  bool result = true;
  for ( int i = 0; i < values.size(); i++ ) {
    QTextTableCell cell = table->cellAt ( row, i );
    QTextCursor cursor = cell.firstCursorPosition();
    cursor.insertText ( values[i] );
  }
  return result;
}

void Magazine::setIssue(QString issue)
{
	myIssue = issue;
}

QString Magazine::issue() const
{
	return myIssue;
}

void Magazine::setPublicationDate(QDate newDate)
{
	myPubDate = newDate;
}

QDate Magazine::publicationDate() const
{
	return myPubDate;
}
