/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "movie.h"
#include "movie.moc"

#include "movies.h"
#include "movieedit.h"
#include <materialpersonsmodel.h>
#include <esqldatabase.h>
#include <ewidget.h>
#include <equerythread.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QTextTable>
#include <QTextCursor>

Movie::Movie ( EQueryThread *thread, Medium *newParent )
	: Medium ( tr("Movie"), "movie", Movies::group() +"_viewmovies", thread, newParent )
{
}

#if QT_VERSION < 0x040500
Medium* Movie::newMedium()
{
  return new Movie();
}
#endif

bool Movie::select ( QSqlRecord record )
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = false;
  if ( !db.isOpen() )
    closeDB = true;

  if ( !Medium::select ( record ) )
    return false;

  if ( !record.contains ( "pages" ) ) {
    if ( !db.open() )
      return false;

    QSqlQuery replaceRecord;
    replaceRecord.exec
    (
      QString ( "SELECT * FROM %1_movies WHERE materialsId=%2" )
      .arg ( Movies::group() )
      .arg ( id() )
    );
    if ( !replaceRecord.first() ) {
      if ( closeDB )
        db.close();
      return false;
    }
    record = replaceRecord.record();
  }

  setPublicationDate ( record.value ( "publicationDate" ).toDate() );
	setDevice( record.value( "devicesId").toUInt() );
	
	if(closeDB)
		db.close();
	return true;
}

bool Movie::prepare()
{
	if(Medium::prepare())
		return false;

	// prepare your custom stuff below
	if(device() > 0)
		return true;

	EditMedium *edit = editWidget();
	edit->setAddMode(this, false);
	if(edit->exec() != QDialog::Accepted)
		return false;
	return prepare();
}

bool Movie::store ( bool addOwner, bool startTransaction )
{
	ESqlDatabase db = ESqlDatabase::database();
  bool closeDB = !db.isOpen();
  if ( closeDB )
    db.open();
  if ( !db.isOpen() )
    return false;

  bool transaction = false;
  if ( startTransaction )
    transaction = transactionsSupported;
  if ( transaction )
    db.transaction();

  if ( !Medium::store ( addOwner, false ) ) {
    if ( transaction )
      if ( !db.rollback() )
        emit error ( tr ( "Rolling back failed!" ) );
    if ( closeDB )
      db.close();
    return false;
  }

  // id is needed and should be set by Medium::store()
  if ( id() == 0 ) {
    if ( transaction )
      if ( !db.rollback() )
        emit error ( tr ( "Rolling back failed!" ) );
    if ( closeDB )
      db.close();
    return false;
  }



  QSqlQuery query;
  query.exec ( QString ( "SELECT id FROM %1_movies WHERE materialsId=%2" ).arg ( Movies::group() ).arg ( id() ) );
  bool isUpdate = false;
  if ( query.first() )
    isUpdate = true;

  if ( isUpdate ) {
    if
    (
      !query.prepare
      (
        QString
        (
          "UPDATE %1_movies SET "
          "publicationDate=:publicationDate, devicesId=:devicesId "
          "WHERE materialsId=:materialsId"
        )
        .arg ( Movies::group() )
      )
    ) {
      if ( transaction )
        if ( !db.rollback() )
          emit error ( tr ( "Rolling back failed!" ) );
      if ( closeDB )
        db.close();
      return false;
    }
  } else {
    if
    (
      !query.prepare
      (
        QString
        (
          "INSERT INTO %1_movies "
          "(materialsId, publicationDate, devicesId) "
          "VALUES(:materialsId, :publicationDate, :devicesId)"
        )
        .arg ( Movies::group() )
      )
    ) {
      if ( transaction )
        if ( !db.rollback() )
          emit error ( tr ( "Rolling back failed!" ) );
      if ( closeDB )
        db.close();
      return false;
    }
  }
  query.bindValue ( ":materialsId", id() );
  query.bindValue ( ":publicationDate", publicationDate().toString ( "yyyy-MM-dd" ) );
	query.bindValue ( ":devicesId", device() );

  if ( !query.exec() ) {
    if ( transaction )
      if ( !db.rollback() )
        emit error ( tr ( "Rolling back failed!" ) );
    if ( closeDB )
      db.close();
    return false;
  }

  if ( transaction )
    if ( !db.commit() )
      emit error ( tr ( "Could not store medium! Commit has failed!" ) );
  if ( closeDB )
    db.close();
  return true;
}

bool Movie::setValue ( QString attribute, QVariant value )
{
	if(compatibilityMode())
		return Medium::setValue(attribute, value);
	
	if(attribute == "publisher"){
		setManufacturer(value.toUInt());
	}

	if(attribute == "pubDate"){
		setPublicationDate(value.toDate());
		return true;
	}
	
	if(attribute == "device"){
		bool check = false;
		value.toUInt(&check);
		if(check){
			setDevice(value.toUInt());
			return true;
		}
		setDevice(value.toString());
		return true;
	}
  return Medium::setValue(attribute, value);
}

QVariant Movie::value ( QString attribute, int role )
{
	if ( role == Qt::DisplayRole || compatibilityMode() ){
		if(attribute == "pubDate")
			return publicationDate();
		
		if(attribute == "device")
			return deviceName();
	}
  return Medium::value(attribute, role);
}

EditMedium *Movie::editWidget(EWidget* parent)
{
  return new MovieEdit ( parent );
}

void Movie::setPublicationDate ( QDate newDate )
{
  myPublicationDate = newDate;
}

QDate Movie::publicationDate()
{
  return myPublicationDate;
}

void Movie::setDevice(unsigned int deviceId, bool setName)
{
	myDeviceId = deviceId;
	if(setName){
		setDevice("", false);
		if(deviceId > 0){
			QString queryString = QString("SELECT name FROM "+ Movies::group() +"_devices WHERE id=%1").arg(deviceId);
			if(thread){
				thread->execQuery(this, "setDevice", queryString, "");
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.exec(queryString);
					if(query.first())
						setDevice(query.record());
					if(closeDB)
						db.close();
				}
			}
		}
	}
  emit changed(this);
}

void Movie::setDevice(QString name, bool setId)
{
	myDeviceName = name.trimmed();
	if(setId){
		setDevice(0, false);
		if(!name.isEmpty()){
			QString queryString = "SELECT id FROM "+ Movies::group() +"_devices WHERE name=?";
			if(thread){
				QList<QVariant> values;
				values << name;
				thread->execQuery(this, "setDevice", queryString, "", values);
			}
			else{
				ESqlDatabase db = ESqlDatabase::database();
				bool closeDB = !db.isOpen();
				if(closeDB)
					db.open();
				if(db.isOpen()){
					QSqlQuery query;
					query.prepare(queryString);
					query.addBindValue(name);
					query.exec();
					if(query.first())
						if(query.size() == 1)
							setDevice(query.record());
					if(closeDB)
						db.close();
				}
			}
		}
	}
  emit changed(this);
}

void Movie::setDevice(QSqlRecord record)
{
	if(record.contains("id"))
		setDevice(record.value("id").toUInt(), false);
	else if(record.contains("name"))
		setDevice(record.value("name").toString(), false);
}

unsigned int Movie::device() const
{
	return myDeviceId;
}

QString Movie::deviceName() const
{
	return myDeviceName;
}

Medium* Movie::copyMedium()
{
	Movie *movie = qobject_cast<Movie*>(Medium::copyMedium());
	if(!movie)
		return 0;

	movie->setDevice(device(), false);
	movie->setDevice(deviceName(), false);
	movie->setPublicationDate(publicationDate());
	return movie;
}

bool Movie::exportTextTable ( QTextTable* table, Medium::ExportFormat format, bool titleOnly )
{
	if(format == Bibtex)
		return false;
	
  if ( !table )
    return false;

	if(compatibilityMode() && format == Table)
		return Medium::exportTextTable(table, format, titleOnly);
	
  // collecting the data which should be inserted
  QStringList values;
  if ( titleOnly ) {
    values << attrs ( "title" ).translation;
    values << attrs ( "persons" ).translation;
    values << attrs ( "manufacturer" ).translation;
    values << attrs ( "min" ).translation;
		values << attrs ( "device" ).translation;
    values << tr ( "Wish" );
  } else {
    values << title();
    values << materialPersonsCombined();
    values << manufacturerName();
    values << minsCombined();
    values << deviceName();
    if ( wish() )
      values << tr ( "Yes" );
  }

  // insert the place at the table we need
  if ( !titleOnly ) {
    // the first row is present at the table, because empty table can not be created
#if QT_VERSION < 0x040500
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
  }
  if ( table->columns() < values.size() ) {
#if QT_VERSION < 0x040500
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
  }
  int row = table->rows() - 1;
  bool result = true;
  for ( int i = 0; i < values.size(); i++ ) {
    QTextTableCell cell = table->cellAt ( row, i );
    QTextCursor cursor = cell.firstCursorPosition();
    cursor.insertText ( values[i] );
  }
  return result;
}


QList<ColKeyTrans> Movie::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();
		
  QList<ColKeyTrans> result;
  result << Medium::attrs("source", 0);
	result << Medium::attrs("title", 1);
	result << Medium::attrs("persons", 2);
  result << ColKeyTrans(3, "manufacturer", tr ( "Studio" ));
	result << Medium::attrs("genre", 4);
  result << ColKeyTrans(5, "pubDate",  tr ( "Pub. Date" ));
	result << Medium::attrs("min", 6);
	result << ColKeyTrans(7, "device", tr("Device"));
	result << Medium::attrs("listPrice", 8);
	result << Medium::attrs("listPriceCurrency", 9);
	result << Medium::attrs("description", 10);
	result << Medium::attrs("owner", 11);
  return result;
}

void Movie::customQueryResult(QVariant requestId, int querySize, QSqlRecord record)
{
	if(requestId.toString() == "setDevice")
		if(querySize == 1)
			setDevice(record);
}

