/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "tracktest.h"

#include <dbtestinit.h>
#include "track.h"
#include "cds.h"
#include <esqldatabase.h>
#include "tracksmodel.h"

#include <QtTest/QtTest>
#include <QSqlQuery>
#include <QSqlError>

void TrackTest::initTestCase()
{
	testMedId = 0;
	
	testInit = new DBTestInit(this);
	if(!testInit->initTestDB())
		QSKIP("Could not create test database", SkipAll);
	
	Track *track = new Track();
	keys = testInit->prepareMediumKeys(track->group());
	delete track;
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);
}

void TrackTest::fetchMessages(QString msg)
{
	qDebug() << msg;
}

void TrackTest::testExtendedProperties()
{
	Track *med = new Track();
	connectMedium(med);
	
	baseTests(med);
	
	// testing init status
	QCOMPARE(med->length(), QTime());
	QVERIFY(med->disc() == 0);
	QVERIFY(med->trackNumber() == 0);
	
	QTime date = QTime::currentTime();
	med->setLength(date);
	QCOMPARE(med->length(), date);
	
	med->setDisc(5);
	QVERIFY(med->disc() == 5);
	
	med->setTrackNumber(5);
	QVERIFY(med->trackNumber() == 5);
	
	delete med;
}

void TrackTest::copyTrack()
{
	Track *med = new Track();
	med->setDisc(5);
	med->setTrackNumber(5);
	med->setLength(QTime::currentTime());
	
	Track *copy = qobject_cast<Track*>(med->copyMedium());
	QVERIFY(copy != 0);
	QCOMPARE(copy->disc(), med->disc());
	QCOMPARE(copy->trackNumber(), med->trackNumber());
	QCOMPARE(copy->length(), med->length());
	
	delete copy;
	delete med;
}

void TrackTest::insertAndLoadExtendedProperties()
{
	Track *med = new Track();
	connectMedium(med);
	
	med->setTitle("'SELECT * FROM user;");
	med->setDescription("This is greate show; INSERT INTO test test = 10;");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	med->setLength(QTime::fromString("01:33:22", "hh:mm:ss"));
	
	Track *checkMed = qobject_cast<Track*>(med->copyMedium());
	QVERIFY(checkMed);
	QCOMPARE(med->store(), true);
	QVERIFY(med->id() > 0);
	unsigned int id = med->id();
	delete med;
	
	med = new Track();
	QCOMPARE(med->load(id), true);
	QCOMPARE(med->id(), id);
	QCOMPARE(med->length(), checkMed->length());
	
	delete med;
	delete checkMed;
}

void TrackTest::switchModes()
{
	Track *med = new Track();
	QCOMPARE(med->mode(), Track::Standard);
	
	med->setMode(Track::CD);
	QCOMPARE(med->mode(), Track::CD);
}

void TrackTest::tracksHandling()
{
	Track *med = new Track();
	connectMedium(med);
	
	med->setTitle("Test Track with Tracks");
	med->setDescription("Contains Tracks");
	med->setGenre(keys.testGenreId);
	med->setManufacturer(keys.testManuId);
	med->setListPriceCurrency(keys.testCurId);
	med->setBoughtPriceCurrency(keys.testCurId);
	med->setType(keys.testTypeId);
	
	TracksModel *model = qobject_cast<TracksModel*>(med->childs());
	QVERIFY(model);
	
	// QStringList cd, track, title, length (may be empty, format: hh:mm:ss)
	QList<QStringList> trackStrings;
	QStringList trackString;
	trackString << "1" << "2" << "Track 1" << "00:01:01";
	trackStrings << trackString;
	trackString.clear();
	trackString << "3" << "4" << "Track 2" << "00:02:02";
	trackStrings << trackString;
	
	foreach(QStringList trackString, trackStrings){
// 		QCOMPARE(med->setValue("track", trackString), true);
		Track *track = new Track(NULL, med);
		track->setTitle(trackString[2]);
		track->setLength(QTime::fromString(trackString[3], "hh:mm:ss"));
		QCOMPARE(model->addMedium(track), true);
	}
	QCOMPARE(model->rowCount(), trackStrings.size());
	
	// we can not do more here, because we cannot use EditTrack
	// QCOMPARE(med->store(), true);
	
	// testing not inserting of removed tracks if it is a new CD
	for(int i = 0; i < model->rowCount(); i++){
		Track *track = qobject_cast<Track*>(model->mediumFromIndex(model->index(i, 0)));
		QVERIFY(track);
		track->setMarkForRemovel();
	}
	QCOMPARE(med->store(), true);
	unsigned int id = med->id();
	
	// testing of adding already present Track's
	QMap<unsigned int, Track*> trackMap;
	int num = 3;
	for(int i = 0; i < num; i++){
		Track *track = new Track();
		connectMedium(track);
	
		track->setTitle(QString("Track %1").arg(i));
		track->setGenre(keys.testGenreId);
		track->setManufacturer(keys.testManuId);
		track->setListPriceCurrency(keys.testCurId);
		track->setBoughtPriceCurrency(keys.testCurId);
		track->setType(keys.testTypeId);
		track->setTrackNumber(i);
		QCOMPARE(track->store(), true);
		QCOMPARE(model->addMedium(track->copyMedium()), true);
		trackMap.insert(track->id(), track);
	}
	// since the previous added track's are now already stored at the db,
	// store should be possible
	QCOMPARE(med->store(), true);
	delete med;
	med = new Track();
	connectMedium(med);
	QTest::qWarn("Something failed below");
	QCOMPARE(med->load(id), true);
	model = qobject_cast<TracksModel*>(med->childs());
	QCOMPARE(model->rowCount(), num);
	QCOMPARE(model->mode(), Track::Collection);
	
	// testing TracksModel->value();
	for(int i = 0; i < model->rowCount(); i++){
		Medium *tmp = model->mediumFromIndex(model->index(i, 0));
		QVERIFY(tmp);
		Track *track = trackMap.value(tmp->id(), 0);
		QVERIFY(track);
		QCOMPARE(model->data(model->index(i, 0)).toUInt(), track->trackNumber());
		QCOMPARE(model->data(model->index(i, 1)).toString(), track->materialPersonsCombined());
		QCOMPARE(model->data(model->index(i, 2)).toString(), track->title());
		QCOMPARE(model->data(model->index(i, 3)).toString(), track->length().toString("HH:mm:ss"));
	}

	QList<Track*> tracks = trackMap.values();
	trackMap.clear();
	qDeleteAll(tracks);
	delete med;
}

void TrackTest::extentedSetValueAndValue()
{
	QSKIP("Not implemented yet", SkipSingle);
}

// void MediumTest::readAndSetBasicProperties()
// {
// 	QString title = "'SELECT * FROM user;";
// 	QString description = "This is great Medium!; INSERT INTO test;";
// 	int eval =  5;
// 	
// 	Medium *medium = new Medium();
// 	connectMedium(medium);
// 	
// 	medium->setTitle(title);
// 	QCOMPARE(medium->title(), title);
// 	
// 	QPixmap image(":/images/meganizer.png");
// 	medium->setImage(image);
// 	QCOMPARE(medium->image(), image);
// 	
// 	medium->setDescription(description);
// 	QCOMPARE(medium->description(), description);
// 	
// 	medium->setListPriceCurrency(testCurShort);
// 	QCOMPARE(medium->listPriceCurrency(), testCurId);
// 	medium->setListPriceCurrency(invalid);
// 	QEXPECT_FAIL("", "Validation tests on Id are not done!", Continue);
// 	QVERIFY(medium->listPriceCurrency() == 0);
// 	medium->setListPriceCurrency(testCur);
// 	QCOMPARE(medium->listPriceCurrency(), testCurId);
// 	medium->setListPriceCurrency(invalidId);
// 	// qcompare has problems with unsigned int and int
// 	QVERIFY(medium->listPriceCurrency() == invalidId);
// 	
// 	medium->setListPrice(11.11);
// 	QCOMPARE(medium->listPrice(), 11.11);
// 	
// 	medium->setBoughtPriceCurrency(testCurShort);
// 	QCOMPARE(medium->boughtPriceCurrency(), testCurId);
// 	medium->setBoughtPriceCurrency(invalid);
// 	QEXPECT_FAIL("", "Validation tests on Id are not done!", Continue);
// 	QVERIFY(medium->boughtPriceCurrency() == 0);
// 	medium->setBoughtPriceCurrency(testCur);
// 	QCOMPARE(medium->boughtPriceCurrency(), testCurId);
// 	medium->setBoughtPriceCurrency(0);
// 	QVERIFY(medium->boughtPriceCurrency() == 0);
// 	
// 	medium->setEvaluation(eval);
// 	QCOMPARE(medium->evaluation(), eval);
// 
// 	medium->setWish(true);
// 	QCOMPARE(medium->wish(), true);
// 	medium->setWish(false);
// 	QCOMPARE(medium->wish(), false);
// 	
// 	medium->setLiteratureOnly(true);
// 	QCOMPARE(medium->literatureOnly(), true);
// 	medium->setLiteratureOnly(false);
// 	QCOMPARE(medium->literatureOnly(), false);
// 	
// 	medium->setGiftOnly(true);
// 	QCOMPARE(medium->giftOnly(), true);
// 	medium->setGiftOnly(false);
// 	QCOMPARE(medium->giftOnly(), false);
// 	
// 	medium->setType(testTypeId);
// 	QCOMPARE(medium->type(), testTypeId);
// }
// 
// void MediumTest::insertAndLoadBasicProperties()
// {
// 	// prepare medium
// 	Medium *medium = new Medium();
// 	connectMedium(medium);
// 
// 	medium->setTitle("'SELECT * FROM user;");
// 	medium->setDescription("This is greate show; INSERT INTO test test = 10;");
// 	medium->setGenre(testGenreId);
// 	medium->setManufacturer(testManuId);
// 	medium->setImage(QPixmap(":/images/meganizer.png"));
// 	medium->setListPriceCurrency(testCurId);
// 	medium->setListPrice(11.11);
// 	medium->setBoughtPriceCurrency(testCurId);
// 	medium->setBoughtPrice(22.22);
// 	medium->setEvaluation(5);
// 	
// 	// storing should not be successfull here, because we have no media type
// 	QCOMPARE(medium->store(), false);
// 	
// 	medium->setType(testTypeId);
// 	Medium *checkMed = medium->copyMedium();
// 	QCOMPARE(medium->store(), true);
// 	QVERIFY(medium->id() > 0);
// 	
// 	testMedId = medium->id();
// 	delete medium;
// 	medium = new Medium();
// 	connectMedium(medium);
// 	QCOMPARE(medium->select(testMedId), true);
// 	QCOMPARE(medium->id(), testMedId);
// 	
// 	QCOMPARE(medium->title(), checkMed->title());
// 	QCOMPARE(medium->genreName(), checkMed->genreName());
// 	QCOMPARE(medium->manufacturerName(), checkMed->manufacturerName());
// 	QCOMPARE(medium->type(), checkMed->type());
// 	QEXPECT_FAIL("", "Should work normally", Continue);
// 	QCOMPARE(medium->image(), checkMed->image());
// 	QCOMPARE(medium->evaluation(), checkMed->evaluation());
// 	QCOMPARE(medium->wish(), checkMed->wish());
// 	QCOMPARE(medium->literatureOnly(), checkMed->literatureOnly());
// 	QCOMPARE(medium->giftOnly(), checkMed->giftOnly());
// 	QCOMPARE(medium->listPriceCurrency(), checkMed->listPriceCurrency());
// 	QCOMPARE(medium->listPrice(), checkMed->listPrice());
// 	QCOMPARE(medium->boughtPriceCurrency(), checkMed->boughtPriceCurrency());
// 	QCOMPARE(medium->boughtPrice(), checkMed->boughtPrice());
// 	
// 	medium->setWish(true);
// 	medium->setGiftOnly(true);
// 	medium->setLiteratureOnly(true);
// 	QCOMPARE(medium->store(), true);
// 	delete checkMed;
// 	checkMed = medium->copyMedium();
// 	medium->setId(0);
// 	medium->select(testMedId);
// 	QCOMPARE(medium->id(), testMedId);
// 	QCOMPARE(medium->wish(), checkMed->wish());
// 	QCOMPARE(medium->giftOnly(), checkMed->giftOnly());
// 	QCOMPARE(medium->literatureOnly(), checkMed->literatureOnly());
// }
// 
// void MediumTest::addOwner()
// {
// 	if(testMedId <= 0)
// 		QSKIP("No inserted medium present", SkipSingle);
// 	
// 	Medium *med = new Medium();
// 	connectMedium(med);
// 	med->select(testMedId);
// 	QCOMPARE(med->id(), testMedId);
// 	
// 	// creating new owner
// 	ESqlDatabase db = ESqlDatabase::database();
// 	if(!db.open())
// 		QSKIP("Could not connect to database", SkipSingle);
// 	QSqlQuery query;
// 	if(!query.exec("INSERT INTO meganizer_users (name, givenName, userName) VALUES('Mustermann', 'Max', 'max')"))
// 		QSKIP("Could not create Mr. Mustermann", SkipSingle);
// 	query.exec("SELECT id FROM meganizer_users WHERE name='Mustermann'");
// 	query.first();
// 	unsigned int userId = query.record().value("id").toUInt();
// 	if(userId == 0)
// 		QSKIP("Could not fetch Mr. Mustermann's userId", SkipSingle);
// 	db.close();
// 	
// 	med->setUser(userId);
// 	QCOMPARE(med->user(), userId);
// 	QCOMPARE(med->store(true), true);
// }
// 
// void MediumTest::personsHandling()
// {
// 	if(testMedId <= 0)
// 		QSKIP("No inserted medium present", SkipSingle);
// 	
// 	Medium *med = new Medium();
// 	med->select(testMedId);
// 	QCOMPARE(med->id(), testMedId);
// 	
// 	// add test persons into database
// 	ESqlDatabase db = ESqlDatabase::database();
// 	if(!db.open())
// 		QSKIP("Could not connect to database", SkipSingle);
// 	QList<QStringList> persons;
// 	QStringList personString;
// 	personString << "Mustermann" << "Max";
// 	persons << personString;
// 	personString.clear();
// 	personString << "Mustermann" << "Erna";
// 	persons << personString;
// 	personString.clear();
// 	personString << "Mustermann" << "Manfred Maria";
// 	persons << personString;
// 	personString.clear();
// 	personString << "Dicht" << "Haucke";
// 	persons << personString;
// 	personString.clear();
// 	
// 	QSqlQuery query;
// 	QList<unsigned int> personIds;
// 	foreach(QStringList person, persons){
// 		query.prepare("INSERT INTO materialpersons (name, givenName) VALUES(:name, :givenName)");
// 		query.bindValue(":name", person[0]);
// 		query.bindValue(":givenName", person[1]);
// 		if(!query.exec())
// 			QSKIP("Could not insert test person", SkipSingle);
// 		query.prepare("SELECT id FROM materialpersons WHERE name=:name AND givenName=:givenName");
// 		query.bindValue(":name", person[0]);
// 		query.bindValue(":givenName", person[1]);
// 		query.exec();
// 		query.first();
// 		unsigned int id = query.record().value("id").toUInt();
// 		if(id <= 0)
// 			QSKIP("Could not fetch person id", SkipSingle);
// 		personIds << id;
// 	}
// 	
// 	
// 	// insert role
// 	QString testRole = "Testrole";
// 	if(!query.exec("INSERT INTO materialpersonsroles (name, description) VALUES('"+ testRole +"', 'Role for testing purposes')"))
// 		QSKIP("Could not create Testrole", SkipSingle);
// 	query.exec("SELECT id FROM materialpersonsroles WHERE name='"+ testRole +"'");
// 	query.first();
// 	unsigned int testRoleId = query.record().value("id").toUInt();
// 	QVERIFY(testRoleId > 0);
// 	
// 	db.close();
// 	
//  	
// 	MaterialPersonsModel *model = med->materialPersons();
// 	foreach(unsigned int personId, personIds){
// 		QCOMPARE(model->addPerson(personId, testRoleId), true);
// 	}
// 	
// 	// check storing and loading
// 	QCOMPARE(med->store(), true);
// 	delete med;
// 	med = new Medium();
// 	connectMedium(med);
// 	med->select(testMedId);
// 	QCOMPARE(med->id(), testMedId);
// 	model = med->materialPersons();
// 	QCOMPARE(personIds.size(), model->rowCount());
// 	for(int i = 0; i < model->rowCount(); i++){
// 		MaterialPerson *person = qobject_cast<MaterialPerson*>(model->personFromIndex(model->index(i, 0)));
// 		QVERIFY(person);
// 		personIds.removeAll(person->id());
// 	}
// 	QCOMPARE(personIds.size(), 0);
// }
// 
// void MediumTest::minHandling()
// {
// 	QSKIP("Not implemented yet", SkipSingle);
// }
// 
// void MediumTest::valueAndSetValueString_data()
// {
// 	QTest::addColumn<QString>("attribute");
// 	QTest::addColumn<QVariant>("value");
// 	QTest::addColumn<bool>("result");
// 	QTest::addColumn<QVariant>("dataResult");
// 	
// 	QTest::newRow("invalid attribute") << invalid << QVariant(invalidId) << false << QVariant();
// 	
// 	QTest::newRow("type invalidId") << "type" << QVariant(invalidId) << true << QVariant(invalidId);
// 	QTest::newRow("type invalid") << "type" << QVariant(invalid) << false << QVariant(0);
// 	
// 	QTest::newRow("manufacturer invalid") << "manufacturer" << QVariant(invalid) << true << QVariant(invalid);
// 	QTest::newRow("manufacturer invalidId") << "manufacturer" << QVariant(invalidId) << true << QVariant(QString());
// 	QTest::newRow("manufacturer testManuId") << "manufacturer" << QVariant(testManuId) << true << QVariant(testManu);
// 	QTest::newRow("manufacturer testManu") << "manufacturer" << QVariant(testManu) << true << QVariant(testManu);
// 	
// 	QTest::newRow("genre invalid") << "genre" << QVariant(invalid) << true << QVariant(0);
// 	QTest::newRow("genre invalidId") << "genre" << QVariant(invalidId) << true << QVariant(QString());
// 	QTest::newRow("genre testGenreId") << "genre" << QVariant(testGenreId) << true << QVariant(testGenre);
// 	QTest::newRow("genre testGenre") << "genre" << QVariant(testGenre) << true << QVariant(testGenre);
// 	
// 	QTest::newRow("owner invalid") << "owner" << QVariant(invalid) << false << QVariant(0);
// 	QTest::newRow("owner invalidId") << "owner" << QVariant(invalidId) << true << QVariant(invalidId);
// // 	QTest::newRow("owner testUserId") << "genre" << QVariant(testGenreId) << true << QVariant(testGenre);
// // 	QTest::newRow("owner testUser") << "genre" << QVariant(testGenre) << true << QVariant(testGenre);
// }
// 
// void MediumTest::valueAndSetValueString()
// {
// 	Medium *med = new Medium();
// 	connectMedium(med);
// 	
// 	QFETCH(QString, attribute);
// 	QFETCH(QVariant, value);
// 	QFETCH(bool, result);
// 	QFETCH(QVariant, dataResult);
// 
// 	QStringList abortRows;
// 	abortRows << "manufacturer invalid" << "manufacturer testManu" << "genre invalid" << "genre testGenre";
// 	// Abort skips this data row, not the whole test
// 	if(abortRows.contains(QTest::currentDataTag()))
// 		QEXPECT_FAIL(QTest::currentDataTag(), "Setting "+ attribute.toLatin1() +" names through setValue is not supported right now", Abort);
// 
// 	QCOMPARE(med->setValue(attribute, value), result);
// 	QCOMPARE(med->value(attribute, Qt::DisplayRole), dataResult);
// }

void TrackTest::connectMedium(Medium* med)
{
	connect(med, SIGNAL(error(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(warning(QString)), this, SLOT(fetchMessages(QString)));
	connect(med, SIGNAL(information(QString)), this, SLOT(fetchMessages(QString)));
}

QTEST_MAIN(TrackTest)

#include "tracktest.moc"
