/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "track.h"
#include "track.moc"

#include "cds.h"
#include "trackedit.h"
#include <materialpersonsmodel.h>
#include <editmedium.h>
#include <esqldatabase.h>
#include "tracksmodel.h"
#include <ewidget.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QTextTable>
#include <QTextCursor>

Track::Track ( EQueryThread *thread, Medium *newParent )
	: Medium ( tr ( "Track" ), "track", CDs::group() + "_viewtracks", thread, newParent )
{
	myMode = Standard;
	myDisc = 0;
	myTrackNumber = 0;
	mySide = NotSet;
}

#if QT_VERSION < 0x040500
Medium* Track::newMedium()
{
	return new Track();
}
#endif

bool Track::select( QSqlRecord record)
{
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
  if ( closeDB )
		if( !db.open() )
			return false;
  
  if(!Medium::select(record)){
		if(closeDB)
			db.close();
    return false;
	}
  
	if(!record.contains("length")){
    if ( closeDB )
			if( !db.open() )
				return false;
      
    QSqlQuery replaceRecord;
    replaceRecord.exec
    (
      QString("SELECT * FROM %1_tracks WHERE materialsId=%2")
				.arg(CDs::group())
				.arg(id())
    );
    if(!replaceRecord.first()){
      if(closeDB)
				db.close();
      return false;
    }
    record = replaceRecord.record();
  }
  
  setLength ( record.value ( "length" ).toTime() );

  if(closeDB)
    db.close();
  return true;
}

bool Track::store( bool addOwner, bool startTransaction)
{
	bool isInsert = false;
	if(id() == 0)
		isInsert = true;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
  if ( closeDB ){
		if( !db.open() )
			return false;
	}
	
	bool transaction = false;
	if(startTransaction)
		transaction = transactionsSupported;
	if(transaction)
		db.transaction();
	
  if ( !Medium::store(addOwner, false) ){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if(closeDB)
			db.close();
    return false;
	}

  // id is needed and should be set by Medium::store()
  if(id() == 0){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if(closeDB)
			db.close();
    return false;
	}
  
	QSqlQuery query;
	query.exec(QString("SELECT id FROM %1_tracks WHERE materialsId=%2").arg(CDs::group()).arg(id()));
	bool isUpdate = false;
	if(query.first())
		isUpdate = true;
  if(isUpdate){
    if
    (
      !query.prepare
      (
				QString
				(
					"UPDATE %1_tracks SET "
					"length=:length "
					"WHERE materialsId=:materialsId"
				)
				.arg(CDs::group())
			)
		)
    {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
			if(closeDB)
				db.close();
			emit error(tr("Could not prepare statement to update track's data at tracks table!\nQuery: %1\nError: %2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }
  }
  else{
    if
    (
      !query.prepare
      (
				QString
				(
					"INSERT INTO %1_tracks "
					"(materialsId, length) "
					"VALUES(:materialsId, :length)"
				)
				.arg(CDs::group())
      )
    )
    {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
			if(closeDB)
				db.close();
			emit error(tr("Could not prepare statement to insert track's data at tracks table!\nQuery: %1\nError: %2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }
  }
  query.bindValue(":materialsId", id());
  query.bindValue(":length", length().toString("HH:mm:ss"));
  
  if(!query.exec()){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if(closeDB)
			db.close();
		emit error(tr("Could not execute the query to modife tracks table!\nQuery: %1\nError: %2").arg(query.lastQuery()).arg(query.lastError().text()));
    return false;
  }

	if(transaction)
		if(!db.commit())
			emit error(tr("Could not store medium! Commit has failed!"));	
  if(closeDB)
		db.close();

	return true;
}

bool Track::storeChild(Medium* parent)
{
	if(mode() != Track::CD && mode() != Track::LP && mode() != Track::Collection)
		return false;
	
	if(!parent)
		return false;
	
	// insert our own genre
	if(genre() == 0){
		setGenre(parent->genre(), false);
		setGenre(parent->genreName(), false);
	}
	// insert own manufacturer
	if(manufacturer() == 0){
		setManufacturer(parent->manufacturer(), false);
		setManufacturer(parent->manufacturerName(), false);
	}

	QSqlQuery query;
	if(markForRemovel()){
		if(materialHasMaterialsId() < 1)
			return false;

		if(mode() == Track::CD || mode() == Track::LP){
			// remove the special relations between this cd and the track
			// The standard relation between two media are removed with Medium::store(Medium*) below.
			query.prepare(QString("DELETE FROM %1_ondisc WHERE materialhasmaterialsId=:materialhasmaterialsId")
										.arg(CDs::group()));
			query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
			if(!query.exec())
				return false;

			query.prepare("DELETE FROM meganizer_materialhasmaterials WHERE id=:id");
			query.bindValue(":id", materialHasMaterialsId());
			if(!query.exec())
				return false;
		}
		else{
			// its the collection mode
			// remove all entries from trackhastracks table
			query.prepare(QString("DELETE FROM %1_trackhastracks WHERE materialhasmaterialsId=:materialhasmaterialsId")
										.arg(CDs::group()));
			query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
			if(!query.exec())
				return false;
		}
	}

	// Add standard relation entries or if markForRemovel()
	// is true, remove the standard relations
	if(!Medium::storeChild(parent))
		return false;
	if(markForRemovel())
		return true;

	// add special relations for CD - Track relation or Track - Track Relation
	if(mode() == Track::CD || mode() == Track::LP){
		// check whether relation already exists
		query.prepare(QString("SELECT * FROM %1_ondisc WHERE materialhasmaterialsId=:materialhasmaterialsId")
									.arg(CDs::group()));
		query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
		if(!query.exec())
			return false;
		if(query.first()){
			// it exists and everything is up to date
			if((disc() == query.record().value("disc").toUInt()) && (trackNumber() == query.record().value("number").toUInt()))
				return true;

			// it exists, but it must be updated
			query.prepare(QString("UPDATE %1_ondisc SET disc=:disc, side=:side, number=:number WHERE materialhasmaterialsId=:materialhasmaterialsId").arg(CDs::group()));
			query.bindValue(":disc", disc());
			query.bindValue(":number", trackNumber());
			query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
			if(!query.exec())
				return false;
		}
		else{
			// it does not exists
			query.prepare(QString("INSERT INTO %1_ondisc (materialhasmaterialsId, disc, side, number) VALUES(:materialhasmaterialsId, :disc, :side, :number)").arg(CDs::group()));
			query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
			query.bindValue(":disc", disc());
			if(mode() == Track::LP){
				switch(side()){
					case Track::SideB:
						query.bindValue(":side", "A");
						break;
					default:
						query.bindValue(":side", "B");
				}
			}
			else{
				query.bindValue(":side", QVariant(QVariant::String));
			}
			query.bindValue(":number", trackNumber());
			if(!query.exec())
				return false;
		}
		return true;
	}

	// collection mode
	// check whether entry already exists
	query.prepare(QString("SELECT * FROM %1_trackhastracks WHERE materialhasmaterialsId=:materialhasmaterialsId")
								.arg(CDs::group()));
	query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
	if(!query.exec())
		return false;
	if(query.first()){
		// it exists and everything is up to date
		if(trackNumber() == query.record().value("number").toUInt())
			return true;

		// it exists, but it must be updated
		query.prepare(QString("UPDATE %1_trackhastracks SET number=:number WHERE materialhasmaterialsId=:materialhasmaterialsId").arg(CDs::group()));
		query.bindValue(":number", trackNumber());
		query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
		if(!query.exec())
			return false;
	}
	else{
		// it must be added
		query.prepare(QString("INSERT INTO %1_trackhastracks (materialhasmaterialsId, number) VALUES(:materialhasmaterialsId, :number)").arg(CDs::group()));
		query.bindValue(":materialhasmaterialsId", materialHasMaterialsId());
		query.bindValue(":number", trackNumber());
		if(!query.exec())
			return false;
	}
	return true;
}

bool Track::setValue ( QString attribute, QVariant value )
{
	if(compatibilityMode())
		return Medium::setValue(attribute, value);
	
  if(attribute == "length"){
		setLength(value.toTime());
		return true;
	}
	
	bool result = false;
	if(mode() != Standard && attribute == "nr"){
		setTrackNumber(value.toUInt(&result));
		return result;
	}
	
	if((mode() == CD || mode() == LP) && attribute == "disc"){
		setDisc(value.toUInt(&result));
		return result;
	}

	if(mode() == LP && attribute == "side"){
		setSide(value.toString());
		return true;
	}

	return Medium::setValue(attribute, value);
}

QVariant Track::value ( QString attribute, int role )
{
	if ( role != Qt::DisplayRole || compatibilityMode())
		return Medium::value(attribute, role);
	
	if(attribute == "disc")
		return disc();
	if(attribute == "side"){
		switch(side()){
			case SideA:
				return "A";
			case SideB:
				return "B";
			default:
				return QString();
		}
	}
	if(attribute == "nr")
		return trackNumber();
	if(attribute == "length")
		return length().toString("HH:mm:ss");
  return Medium::value(attribute, role);
}

EditMedium *Track::editWidget(EWidget* parent )
{
  return new TrackEdit ( parent );
}

void Track::setLength ( QTime duration )
{
  myLength = duration;
}

QTime Track::length()
{
  return myLength;
}

Medium* Track::copyMedium()
{
    Track *track = qobject_cast<Track*>(Medium::copyMedium());
		if(!track)
			return 0;
		
		track->setLength(length());
		track->setMode(mode());
		track->setDisc(disc());
		track->setTrackNumber(trackNumber());
		return track;
}

bool Track::exportTextTable(QTextTable* table, Medium::ExportFormat format, bool titleOnly)
{
	if(format == Bibtex)
		return false;
	
	if(!table)
		return false;

	if(compatibilityMode() && format == Table)
		return Medium::exportTextTable(table, format, titleOnly);
	
	// collecting the data which should be inserted	
	QStringList values;
	if(titleOnly){
		values << attrs ( "title" ).translation;
		values << attrs ( "persons" ).translation;
		values << attrs ( "length" ).translation;
		values << tr("Wish");
	}
	else{
		values << title();
		MaterialPersonsModel *model = materialPersons();
		if(model){
			QStringList authors;
			for(int i = 0; i < model->rowCount(); i++){
				authors.append
				(
					QString("%1, %2")
						.arg(model->data(model->index(i, 0)).toString())
						.arg(model->data(model->index(i, 1)).toString().left(1))
				);
			}
			values << authors.join(tr(" and "));
		}
		values << length().toString(tr("hh:mm:ss"));
		if(wish())
			values << tr("Yes");
	}
	
	// insert the place at the table we need
	if(!titleOnly){
		// the first row is present at the table, because empty table can not be created
#if QT_VERSION < 0x040500
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
	}
	if(table->columns() < values.size()){
#if QT_VERSION < 0x040500
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
	}
	int row = table->rows() - 1;
	bool result = true;
	for(int i = 0; i < values.size(); i++){
		QTextTableCell cell = table->cellAt(row, i);
		QTextCursor cursor = cell.firstCursorPosition();
		cursor.insertText(values[i]);
	}
	return result;
}

void Track::setDisc ( unsigned int num )
{
	myDisc = num;
}

unsigned int Track::disc() const
{
	return myDisc;
}

void Track::setSide(QString side)
{
	if(side.length() != 1)
		return;
	if(side.toLower() == "a")
		setSide(SideA);
	if(side.toLower() == "b")
		setSide(SideB);
}

void Track::setSide(Track::Side side)
{
	mySide = side;
}

Track::Side Track::side() const
{
	return mySide;
}

void Track::setTrackNumber ( unsigned int num )
{
	myTrackNumber = num;
}

unsigned int Track::trackNumber() const
{
 return myTrackNumber;
}

void Track::setMode ( Track::Mode mode )
{
	myMode = mode;
	emit changed(this);
}

Track::Mode Track::mode() const
{
	return myMode;
}

MediaModel* Track::childs()
{
	if(Medium::childs())
		return Medium::childs();
	TracksModel *model = new TracksModel(thread, this);
	setChildModel(model);
	model->setMode(Collection);
	return model;
}

QList<ColKeyTrans> Track::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();
		
  QList<ColKeyTrans> attrs;
	switch(mode()){
		case CD:
			attrs << ColKeyTrans(0, "disc", tr ( "Disc" ));
			attrs << ColKeyTrans(1, "nr", tr ( "Nr" ));
			attrs << Medium::attrs("persons", 2);
			attrs << Medium::attrs("title", 3);
			attrs << ColKeyTrans(4, "length", tr ( "Length" ));
			break;
		case LP:
			attrs << ColKeyTrans(0, "disc", tr ( "LP" ));
			attrs << ColKeyTrans(1, "side", tr ( "Side" ));
			attrs << ColKeyTrans(2, "nr", tr ( "Nr" ));
			attrs << Medium::attrs("persons", 3);
			attrs << Medium::attrs("title", 4);
			attrs << ColKeyTrans(5, "length", tr ( "Length" ));
			break;
		case Collection:
			attrs << ColKeyTrans(0, "nr", tr ( "Nr" ));
			attrs << Medium::attrs("persons", 1);
			attrs << Medium::attrs("title", 2);
			attrs << ColKeyTrans(3, "length", tr ( "Length" ));
			break;
		default:
			attrs << Medium::attrs("source", 0);
			attrs << Medium::attrs("title", 1);
			attrs << Medium::attrs("persons", 2);
			attrs << ColKeyTrans(3, "length", tr ( "Length" ));
			attrs << Medium::attrs("owner", 4);
	}
  return attrs;
}
