/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "checkmanufacturerworker.h"
#include "checkmanufacturerworker.moc"

#include "meganizer.h"
#include <minvalidator.h>
#include <esqldatabase.h>
#include <manufacturermodel.h>
#include <manufacturermodelrootitem.h>
#include <manufacturer.h>
#include <manufacturerprefix.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QDateTime>
#include <QMap>
#include <QVariant>
#include <QSqlDriver>
#include <QSettings>
#include <QDomDocument>
#include <QNetworkReply>

CheckManufacturerWorker::CheckManufacturerWorker ( QObject* parent )
	: EObject ( parent )
{

}

CheckManufacturerWorker::~CheckManufacturerWorker()
{
	if(!connectionName.isEmpty())
		ESqlDatabase::removeDatabase(connectionName);
}

void CheckManufacturerWorker::runChecks(bool doSendData)
{
	if(addedPrefixes.size() > 0){
		// the new prefixes are not send to the central database,
		// therefore a new run is not started until they are send.
		// If they should not be send at all, we ignore them
		QSettings settings;
		if(settings.value("sendMINRanges", false).toBool()){
			emit finished(false);
			emit log(tr("New prefixes are not send"));
			return;
		}
		addedPrefixes.clear();
	}
	abort = false;
	MINValidator validator;
	ESqlDatabase origDB = ESqlDatabase::database();
	
	// make sure that really every thread has it's own db name
	ESqlDatabase db;
	prepareDB(db);
	
	if(!db.open()){
		emit finished(false);
		return;
	}
	bool transactions = db.driver()->hasFeature(QSqlDriver::Transactions);
	
	// first get all MIN types
	QMap<uint, QString> mapTypes;
	QSqlQuery query(db);
	query.exec("SELECT id, name FROM meganizer_mintypes ORDER BY id");
	while(query.next()){
		mapTypes.insert(
			query.record().value("id").toUInt(),
			query.record().value("name").toString()
		);
		if(abort){
			emit finished(false);
			db.close();
			return;
		}
	}
	if(mapTypes.uniqueKeys().size() < 1){
		emit log(tr("No MIN types found"));
		emit finished(false);
		return;
	}
	// plus one for the manufacturer list
	emit setOverallRange(mapTypes.uniqueKeys().size() + 1);
	
	// second, find out all manufacturers
	QMap<uint, QString> mapManus;
	emit setStep(tr("Fetching all manufacturers"));
	query.exec("SELECT id, name FROM "+ Meganizer::group() +"_manufacturers ORDER BY id");
	while(query.next()){
		mapManus.insert(query.record().value("id").toUInt(),
										query.record().value("name").toString());
		if(abort){
			emit finished(false);
			db.close();
			return;
		}
	}
	if(mapManus.size() < 1){
		emit log(tr("No manufacturers found!"));
		db.close();
		emit finished(false);
		return;
	}
	emit overallIncrease();
	
	foreach(uint typeId, mapTypes.uniqueKeys()){
		emit setMINType(mapTypes.value(typeId));
		emit setStep(tr("Analyzing %1's...").arg(mapTypes.value(typeId)));
		emit setMINTypeRange(mapManus.uniqueKeys().size());

		// analyizing the product numbers of each manufacturer
		// to find out the part of the manufacturer
		foreach(uint manuId, mapManus.uniqueKeys()){
			emit log(QString("  %1").arg(mapManus.value(manuId)));
			query.exec(QString("SELECT mins.min "
				"  FROM %1_materialhasmins mins,"
				"    %1_materials mats"
				"  WHERE mins.materialsId=mats.id"
				"    AND mats.manufacturersId=%2"
				"    AND mins.mintypesId=%3")
			.arg(Meganizer::group())
			.arg(manuId)
			.arg(typeId));
			QStringList numbers;
			while(query.next())
				numbers << query.record().value("min").toString();
			
			if(numbers.size() < 2){
				emit log(tr("                                  Not enough numbers"));
				emit minTypeIncrease();
			}
			else{
				QStringList prefixes = validator.commonPrefixes(numbers, mapTypes.value(typeId));
				if(prefixes.size() < 1){
					emit log(tr("                                  No common prefix found!"));
				}
				else{
					emit log(tr("                                  New: %1").arg(prefixes.join(", ")));
					// find existing mins for current manufacturer
					QString minsSelectStatement = QString("SELECT characteristic FROM %1_manufacturersmins"
																								"  WHERE manufacturersId=%2 AND mintypesId=%3")
																								.arg(Meganizer::group())
																								.arg(manuId)
																								.arg(typeId);
					if(!query.exec(minsSelectStatement)){
						emit log(tr("Could not fetch existing characteristics"));
						emit finished(false);
						db.close();
						return;
					}
					// check whether found prefix is already stored
					// or if an stored prefix is included in a found new one
					// (e.g.: 978342311 is contained in 9783423 and only
					// the last one should be used than)
					QStringList existingPrefixes;
					QStringList newPrefixes, removePrefixes;
					while(query.next())
						existingPrefixes << query.record().value("characteristic").toString();
					emit log(tr("                                  Existing: %1").arg(existingPrefixes.join(", ")));
					foreach(QString prefix, prefixes){
						bool found = false;
						foreach(QString existingPrefix, existingPrefixes){
							if(prefix == existingPrefix){
								found = true;
							}
							else{
								// we only want to have more precise results,
								// therefore we do not extend existing prefixes
								if(existingPrefix.left(prefix.size()) == prefix)
									removePrefixes << existingPrefix;
							}
						}
						// if the prefix does not exists, we add it
						if(!found)
							newPrefixes << prefix;
					}
					
					int numNew = newPrefixes.size();
					int numRemove = removePrefixes.size();
					
					// remove prefixes (we replace them with new ones
					foreach(QString removePrefix, removePrefixes){
						if(newPrefixes.size() > 0){
							ManufacturerRange prefix(manuId, typeId, newPrefixes[0]);
							if(transactions)
								db.transaction();
							query.prepare("UPDATE "+ Meganizer::group() +"_manufacturersmins SET characteristic=?"
														"  WHERE characteristic=? AND manufacturersId=? AND mintypesId=?");
							query.addBindValue(prefix.prefix());
							query.addBindValue(removePrefix);
							query.addBindValue(prefix.manufacturerId());
							query.addBindValue(prefix.mintypesId());
							if(!query.exec()){
								emit log(tr("Could not replace %1 with %2").arg(removePrefix).arg(newPrefixes[0]));
								emit finished(false);
								db.close();
								return;
							}
							if(!db.commit()){
								emit log(tr("Could not commit changes for %1").arg(removePrefix));
								emit finished(false);
								db.close();
								return;
							}
							newPrefixes.removeAt(0);
							addedPrefixes << prefix;
						}
						else{
							if(transactions)
								db.transaction();
							query.prepare("DELETE FROM "+ Meganizer::group() +"_manufacturersmins"
														"  WHERE characteristic=? AND manufacturersId=? AND mintypesId=?");
							query.addBindValue(removePrefix);
							query.addBindValue(manuId);
							query.addBindValue(typeId);
							if(!query.exec()){
								emit log(tr("Could not remove %1").arg(removePrefix));
								emit finished(false);
								db.close();
								return;
							}
							if(!db.commit()){
								emit log(tr("Could not commit changes for %1").arg(removePrefix));
								emit finished(false);
								db.close();
								return;
							}
						}
					}
					// add new prefixes
					foreach(QString prefix, newPrefixes){
						ManufacturerRange range(manuId, typeId, prefix);
						query.prepare("INSERT INTO "+ Meganizer::group() +"_manufacturersmins "
													"  (manufacturersId, mintypesId, characteristic) VALUES(?, ?, ?)");
						query.addBindValue(range.manufacturerId());
						query.addBindValue(range.mintypesId());
						query.addBindValue(range.prefix());
						if(!query.exec()){
							emit log(tr("Could not add %1").arg(prefix));
							emit finished(false);
							db.close();
							return;
						}
						addedPrefixes << range;
					}
					
					emit log(tr("                                  Added: %1, Removed: %2").arg(numNew).arg(numRemove));
				}
				emit minTypeIncrease();
			}
			if(abort){
				emit finished(false);
				db.close();
				return;
			}
		}
		if(abort){
			emit finished(false);
			db.close();
			return;
		}
		emit overallIncrease();
	}
	
	db.close();
	
	if(!doSendData)
		emit finished(true);
	else
		sendData();
}

void CheckManufacturerWorker::sendData()
{
	emit setOverallRange(3);
	emit log(tr("Sending MIN ranges and related manufacturers"));
	if(addedPrefixes.size() < 1){
		emit log(tr("  No ranges to send"));
		emit finished(true);
		return;
	}
	
	QSettings settings;
	if(!settings.value("sendMINRanges", false).toBool()){
		emit finished(false);
		emit log(tr("  Sending and recieving MINs and manufacturers is not enabled!"));
		return;
	}
	
	emit log(tr("  Preparing XML data"));
	ManufacturerModel *model = new ManufacturerModel(this);
	ManufacturerModelRootItem *manuServer = new ManufacturerModelRootItem("manuserver");
	model->setRootItem(manuServer);
	ManufacturerModelRootItem *submitPrefixes = new ManufacturerModelRootItem("submitprefixes");
	manuServer->addChild(manuServer->rowCount(), submitPrefixes);
	
	ESqlDatabase db;
	prepareDB(db);
	if(!db.open()){
		emit finished(false);
		return;
	}
	QSqlQuery query(db);
	
	// first prepare the mapping of the min types
	QMap<uint, QString> mapMINTypes;
	query.exec("SELECT id, name FROM meganizer_mintypes ORDER BY id");
	while(query.next()){
		mapMINTypes.insert(
			query.record().value("id").toUInt(),
			query.record().value("name").toString()
		);
	}

	// build up a list of different manufacturers
	QList<uint> manufacturers;
	foreach(ManufacturerRange range, addedPrefixes){
		if(!manufacturers.contains(range.manufacturerId()))
			manufacturers << range.manufacturerId();
	}
	
	emit setMINType(tr("  Preparing XML"));
	emit setMINTypeRange(manufacturers.size());
	foreach(uint manufacturerId, manufacturers){
		// fetching the manufacturer name
		if(!query.exec(QString("SELECT * FROM meganizer_manufacturers WHERE id=%1").arg(manufacturerId))){
			emit log(tr("    Could not select manufacturer"));
			emit finished(false);
			db.close();
			return;
		}
		if(!query.first()){
			emit log(tr("    Could not select manufacturer"));
			emit finished(false);
			db.close();
			return;
		}
		
		Manufacturer *manu = new Manufacturer();
		submitPrefixes->addChild(submitPrefixes->rowCount(), manu);
		manu->setName(query.record().value("name").toString());
		manu->setStreet(query.record().value("street").toString());
		manu->setNumber(query.record().value("number").toString());
		manu->setZIP(query.record().value("postalCode").toUInt());
		manu->setCity(query.record().value("city").toString());
		manu->setPostOfficeBox(query.record().value("postOfficeBox").toUInt());
		manu->setCountry(query.record().value("country").toString());
		manu->setWebsite(query.record().value("website").toString());
		manu->setEmail(query.record().value("email").toString());
		manu->setPhone(query.record().value("phone").toString());
		manu->setFax(query.record().value("fax").toString());
		
		// insert all prefixes of this type
		for(int i = 0; i < addedPrefixes.size(); i++){
			ManufacturerRange range = addedPrefixes[i];
			if(range.manufacturerId() == manufacturerId){
				if(!mapMINTypes.contains(range.mintypesId()) || mapMINTypes.value(range.mintypesId()).isEmpty()){
					emit finished(false);
					emit log(tr("    Could not find MIN type!"));
					db.close();
					return;
				}
				ManufacturerPrefix *prefix = new ManufacturerPrefix();
				manu->addChild(manu->rowCount(), prefix);
				prefix->setType(mapMINTypes.value(range.mintypesId()));
				prefix->setPrefix(range.prefix());
				
				addedPrefixes.removeAt(i);
			}
		}

		if(abort){
			emit finished(false);
			db.close();
			return;
		}
		emit minTypeIncrease();
	}
	db.close();
	
	// validate document
	emit overallIncrease();
	emit setStep(tr("  Validate document"));
	if (!model->isValidXML()){
		emit log("    Instance document is invalid.");
		emit finished(false);
		return;
	}
	if(abort){
		emit finished(false);
		return;
	}
	
	// sending the data
	emit overallIncrease();
	emit setStep(tr("  Sending data"));

	connect ( model, SIGNAL ( sendFinished(ManufacturerModel*)), this, SLOT ( httpRequestFinished(ManufacturerModel*) ) );
	if(!model->send()){
		emit log("    Could not convert model into valid XML document.");
		emit finished(false);
		delete model;
		return;
	}
}

void CheckManufacturerWorker::cancel()
{
	abort = true;
}

void CheckManufacturerWorker::httpRequestFinished(ManufacturerModel *model)
{
	emit log(tr("  Processing answer"));
	if(sender())
		sender()->deleteLater();
	if(!model){
		emit finished(false);
		return;
	}
	if(model->result()){
		emit log(tr("    Finished successfull"));
		emit finished(true);
		delete model;
		return;
	}
	
	emit log("    "+ model->message());
	emit overallIncrease();
	emit finished(false);
	delete model;
}

void CheckManufacturerWorker::prepareDB(ESqlDatabase &db)
{
	ESqlDatabase origDB = ESqlDatabase::database();
	
	// make sure that really every thread has it's own db name
	connectionName = "CheckManufacturer-"+ QDateTime::currentDateTime().toString("yyyyMMddhhmmzzz");
	db = ESqlDatabase::addDatabase(origDB.driverName(), connectionName);
	db.setDatabaseName(origDB.databaseName());
	db.setUserName(origDB.userName());
	db.setPassword(origDB.password());
	db.setHostName(origDB.hostName());
}

