/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "project.h"
#include "project.moc"

#include <medium.h>
#include <mediamodel.h>
#include <esqldatabase.h>
#include "meganizer.h"

#include <QSqlQuery>
#include <QSqlDriver>
#include <QSqlError>

Project::Project ( QObject* parent ) : QObject ( parent )
{
	myId = 0;
	myUsersId = 0;
	myResultMedium = 0;
	myMaterials = new MediaModel(0, this);
}

Project::~Project()
{
	delete myResultMedium;
	delete myMaterials;
}

bool Project::store(QString *errorMsg)
{
	if(usersId() == 0){
		if(errorMsg)
			errorMsg->append(tr("No owner given!"));
		return false;
	}
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(!db.isOpen()){
		if(errorMsg)
			errorMsg->append(tr("Could not connect to database!"));
		return false;
	}
	bool transaction = db.driver()->hasFeature(QSqlDriver::Transactions);
	
	if(transaction)
		db.transaction();
	
	QSqlQuery query;
	if(id() == 0){
		// it's a new project
		query.prepare("INSERT INTO "+ Meganizer::group() +"_projects (name, usersId, resultId, description)"
									"  VALUES(:name, :usersId, :resultId, :description)");
		query.bindValue(":usersId", usersId());
	}
	else{
		// update existing project
		query.prepare("UPDATE "+ Meganizer::group() +"_projects SET name=:name, resultId=:resultId, description=:description WHERE id=:id");
		query.bindValue(":id", id());
	}
	query.bindValue(":name", name());
	if(resultMedium()){
		query.bindValue(":resultId", resultMedium()->id());
	}
	else{
		query.bindValue(":resultId", QVariant(QVariant::UInt));
	}
	query.bindValue(":description", description());
	if(!query.exec()){
		if(errorMsg){
			errorMsg->append(tr("Could not update or insert project!"));
		}
		if(transaction)
			db.rollback();
		if(closeDB)
			db.close();
		return false;
	}
	
	if(id() == 0){
		// fetch id now
		query.prepare("SELECT id FROM "+ Meganizer::group() +"_projects"
									"  WHERE name=:name AND usersId=:usersId"
									"    AND description=:description");
		query.bindValue(":usersId", usersId());
		query.bindValue(":name", name());
		query.bindValue(":description", description());
		query.exec();
		if(query.first())
			setId(query.record().value("id").toUInt());
		if(id() == 0){
			if(errorMsg)
				errorMsg->append(tr("Could not create project"));
			if(transaction)
				db.rollback();
			if(closeDB)
				db.close();
			return false;
		}
	}

	// store materials
	MediaModel *model = materials();
	if(!model){
		if(transaction)
			db.commit();
		if(closeDB)
			db.close();
		return true;
	}
		
	for(int i = 0; i < model->rowCount(); i++){
		Medium *med = model->mediumFromIndex(model->index(i, 0));
		if(!med)
			continue;
		// check wether material is already assigned to project
		query.prepare("SELECT id, comment FROM "+ Meganizer::group() +"_projectscollection WHERE projectsId=:projectsId AND materialsId=:materialsId");
		query.bindValue(":projectsId", id());
		query.bindValue(":materialsId", med->id());
		query.exec();
		if(query.size() == 1){
			// check wether update or a delete is needed and if so, do it
			query.first();
			unsigned int rowId = query.record().value("id").toUInt();
			if(rowId == 0){
				if(errorMsg)
					errorMsg->append(tr("Could not store materials"));
				if(transaction)
					db.rollback();
				if(closeDB)
					db.close();
				return false;
			}
			if(med->markForRemovel()){
				query.prepare("DELETE FROM "+ Meganizer::group() +"_projectscollection WHERE id=:id");
				query.bindValue(":id", rowId);
				if(!query.exec()){
					if(errorMsg)
						errorMsg->append(tr("Could not store materials"));
					if(transaction)
						db.rollback();
					if(closeDB)
						db.close();
					return false;
				}
			}
			else if(comment(med) != query.record().value("comment").toString()){
				// do an update

				query.prepare("UPDATE "+ Meganizer::group() +"_projectscollection SET comment=:comment WHERE id=:id");
				query.bindValue(":comment", comment(med));
				query.bindValue(":id", rowId);
				if(!query.exec()){
					if(errorMsg){
						errorMsg->append(tr("Could not update material!"));
					}
					if(transaction)
						db.rollback();
					if(closeDB)
						db.close();
					return false;
				}
			}
		}
		else{
			// insert
			query.prepare("INSERT INTO "+ Meganizer::group() +"_projectscollection (projectsId, materialsId, comment)"
										"  VALUES(:projectsId, :materialsId, :comment)");
			query.bindValue(":projectsId", id());
			query.bindValue(":materialsId", med->id());
			query.bindValue(":comment", comment(med));
			if(!query.exec()){
				if(errorMsg){
					errorMsg->append(tr("Could not insert material!"));
				}
				if(transaction)
					db.rollback();
				if(closeDB)
					db.close();
				return false;
			}
		}
	}
	
	bool result = true;
	if(transaction)
		result = db.commit();
	
	if(!result)
		if(errorMsg)
			errorMsg->append(tr("Could not commit changes!"));
	
	if(closeDB)
		db.close();
	
	return result;
}

bool Project::load ( QSqlRecord record )
{
	setId(record.value("id").toUInt());
	setName(record.value("name").toString());
	setUsersId(record.value("usersId").toUInt());
	setResultMedium(record.value("resultId").toUInt());
	setDescription(record.value("description").toString());
	
	if(id() == 0)
		return false;
	
	return addMaterials();
}

void Project::setId ( unsigned int projectId )
{
	myId = projectId;
	emit changed(this);
}

unsigned int Project::id() const
{
	return myId;
}

void Project::setName ( QString projectName )
{
	projectName = projectName.trimmed();
	if(projectName.length() > 120)
		projectName = projectName.left(120);
	myName = projectName;
	emit changed(this);
}

QString Project::name() const
{
	return myName;
}

void Project::setUsersId ( unsigned int userId )
{
	myUsersId = userId;
	emit changed(this);
}

unsigned int Project::usersId() const
{
	return myUsersId;
}

void Project::setResultMedium ( unsigned int materialsId )
{
	// because otherwise we can't set a NULL pointer
	// and no valid material can have materialsId 0
	if(materialsId == 0){
		myResultMedium = 0;
		return;
	}
	
	myResultMedium = new Medium();
	if((materialsId > 0) && (myUsersId > 0))
		myResultMedium->load(materialsId, myUsersId);
	emit changed(this);
}

void Project::setResultMedium ( QSqlRecord record )
{
	myResultMedium = new Medium();
	myResultMedium->select(record);
	emit changed(this);
}

void Project::setResultMedium ( Medium* medium )
{
	myResultMedium = medium;
	emit changed(this);
}

Medium* Project::resultMedium() const
{
	return myResultMedium;
}

void Project::setDescription ( QString desc )
{
	desc = desc.trimmed();
	if(desc.length() > 500)
		desc = desc.left(500);
	myDescription = desc;
	emit changed(this);
}

QString Project::description() const
{
	return myDescription;
}

Medium * Project::addMaterial ( QSqlRecord record )
{
	Medium *med = myMaterials->addEmptyMedium();
	bool result = med->select(record);
	emit changed(this);
	if(!result)
		return 0;
	return med;
}

void Project::addMaterial ( Medium *medium )
{
	myMaterials->addMedium(medium);
	emit changed(this);
}

bool Project::addMaterials()
{
	if(id() == 0)
		return false;
	
	ESqlDatabase db = ESqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(!db.isOpen())
		return false;
	
	QSqlQuery query;
	query.prepare("SELECT * FROM viewprojectmaterials WHERE projectsId=:projectsId");
	query.bindValue(":projectsId", id());
	query.exec();
	bool result = true;
	while(query.next()){
		Medium * med = addMaterial(query.record());
		if(!med){
			result = false;
			continue;
		}
		setComment(med, query.record().value("comment").toString());
	}
	
	if(closeDB)
		db.close();
	if(!result)
		return false;
	
	return true;
}

MediaModel* Project::materials() const
{
	return myMaterials;
}

void Project::setComment ( Medium* medium, QString newComment )
{
	comments.insert(medium, newComment);
	emit changed(this);
}

QString Project::comment ( Medium* medium ) const
{
	return comments.value(medium, QString());
}

bool Project::operator== ( const Project& project ) const
{
	if((id() != 0) || (project.id() != 0)){
		if(id() == project.id())
			return true;
		return false;
	}
	
	if(name() == project.name())
		if(usersId() == project.usersId())
			return true;
	return false;
}

bool Project::operator!= ( const Project& project ) const
{
	return !operator==(project);
}
