/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "projects.h"
#include "projects.moc"

#include "projectsmodel.h"
#include "project.h"
#include <medium.h>
#include <mediamodel.h>
#include <addmedium.h>

#include <QSortFilterProxyModel>
#include <QSettings>

Projects::Projects ( QWidget* widget, Qt::WindowFlags flags )
	: EWidget ( widget, flags )
{
	currentProject = 0;
	materialsModel = 0;
	model = 0;
	resultMedium = 0;
	proxyModel = 0;
	
	setupUi(this);
	
	helpButton->setIcon("help-contents", ":/icons/help.png");
	
	QSettings settings;
  settings.beginGroup ( "geometries" );
  projectsSplitter->restoreState ( settings.value ( projectsSplitter->objectName() ).toByteArray() );
  settings.endGroup();
	
	connect(filterEdit, SIGNAL(textChanged(QString)), this, SLOT(filter(QString)));
	connect(projectsList, SIGNAL(clicked(QModelIndex)), this, SLOT(projectChanged(QModelIndex)));
	connect(helpButton, SIGNAL(clicked()), this, SLOT(help()));
	connect(newButton, SIGNAL(clicked()), this, SLOT(newProject()));
	connect(saveButton, SIGNAL(clicked()), this, SLOT(store()));
	
	connect(resultSearchButton, SIGNAL(clicked()), this, SLOT(resultSearch()));
	resultSearchButton->setIcon("edit-find", ":/icons/find.png");
	connect(resultClearButton, SIGNAL(clicked()), this, SLOT(resultClear()));
	resultClearButton->setIcon("edit-clear", ":/icons/clear-left.png");

	connect(materialsView, SIGNAL(clicked(QModelIndex)), this, SLOT(materialChanged(QModelIndex)));
	connect(matAddButton, SIGNAL(clicked()), this, SLOT(materialAdd()));
	connect(matRemoveButton, SIGNAL(clicked()), this, SLOT(materialRemove()));

	proxyModel = new QSortFilterProxyModel(this);
	projectsList->setModel(proxyModel);
	
	model = new ProjectsModel(this);
	proxyModel->setSourceModel(model);
	// apply changes off the original model to the proxy model
  proxyModel->setDynamicSortFilter ( true );
  proxyModel->sort ( 0 );
  proxyModel->setFilterKeyColumn ( 0 );
  proxyModel->setFilterCaseSensitivity ( Qt::CaseInsensitive );
	
	model->load();
	
	projectChanged(QModelIndex());
	materialChanged(QModelIndex());
	filterEdit->setFocus();
}

Projects::~Projects()
{
	close();
	projectChanged(QModelIndex());
	delete proxyModel;
	delete model;
}

void Projects::closeEvent ( QCloseEvent* event )
{
	QSettings settings;
  settings.beginGroup ( "geometries" );
  settings.setValue ( projectsSplitter->objectName(), projectsSplitter->saveState() );
  settings.endGroup();

	EWidget::closeEvent ( event );
}

void Projects::newProject()
{
	projectChanged(QModelIndex());
	currentProject = new Project(model);
	
	nameEdit->setText(filterEdit->text());
	projectTab->setEnabled(true);
	saveButton->setEnabled(true);
	nameEdit->setFocus();
}

void Projects::store()
{
	if(!currentProject)
		return;
		
	if(!projIndex.isValid()){
		model->addProject(currentProject);
	}
	
	currentProject->setName(nameEdit->text());
	currentProject->setResultMedium(resultMedium);
	currentProject->setDescription(descriptionEdit->toPlainText());
	Medium *med = materialsModel->mediumFromIndex(matIndex);
	if(med)
		currentProject->setComment(med, commentEdit->toPlainText());
	
	QString errorMsg;
	if(!currentProject->store(&errorMsg)){
		emit error(tr("Could not store project!\n%1").arg(errorMsg));
		return;
	}
	QString name = currentProject->name();
	
	projectChanged(QModelIndex());
	model->reload();
	
  filter ( name );

  QModelIndex index;
  if ( proxyModel->rowCount() == 1 )
    index = proxyModel->mapToSource ( proxyModel->index ( 0, 0 ) );

	filter ( "" );
	
  // select the item
  if ( index.isValid() )
    projectChanged ( proxyModel->mapFromSource ( index ) );
}

void Projects::projectChanged ( QModelIndex index )
{
	projIndex = proxyModel->mapToSource(index);
	
	projectsList->setCurrentIndex(index);
	
	projectTab->setEnabled(false);
	saveButton->setEnabled(false);
	materialsView->setEnabled(false);
	matAddButton->setEnabled(false);
	clearProjectDetails();
	
	currentProject = model->projectFromIndex(projIndex);
	if(!currentProject)
		return;
	
	nameEdit->setText(currentProject->name());
	resultMedium = currentProject->resultMedium();
	if(resultMedium)
		resultEdit->setMedium(resultMedium);
	descriptionEdit->setPlainText(currentProject->description());
	materialsModel = currentProject->materials();
	if(materialsModel){
		materialsView->setModel(materialsModel);
		materialsModel->addItemDelegates(materialsView);
		materialsModel->hideAndResizeColumns(materialsView);
		materialsView->setEnabled(true);
	}
	projectTab->setEnabled(true);
	matAddButton->setEnabled(true);
	saveButton->setEnabled(true);
	nameEdit->setFocus();
}

void Projects::resultSearch()
{
	AddMedium *addMedium = new AddMedium(this);
	connectByPass(addMedium);
	if(addMedium->exec() != QDialog::Accepted)
		return;
	
	resultMedium = addMedium->selectedMedium();
	if(!resultMedium)
		return;
	resultEdit->setMedium(resultMedium);
	delete addMedium;
}

void Projects::resultClear()
{
	resultMedium = 0;
	resultEdit->setMedium(resultMedium);
}

void Projects::materialAdd()
{
	if(!materialsModel)
		return;
	
	AddMedium *addMedium = new AddMedium(materialsModel, this);
	connectByPass(addMedium);
	addMedium->exec();
	delete addMedium;
}

void Projects::materialRemove()
{
	if(!materialsModel)
		return;
	
	Medium *med = materialsModel->mediumFromIndex(matIndex);
	if(!med)
		return;
	
	med->setMarkForRemovel();
}

void Projects::materialChanged ( QModelIndex index )
{
	// save comment changes to old index
	if((currentProject) && (materialsModel)){
		Medium *oldMed = materialsModel->mediumFromIndex(matIndex);
		if(oldMed)
			currentProject->setComment(oldMed, commentEdit->toPlainText());
	}
	
	matIndex = index;
	
	materialsView->setCurrentIndex(index);
	
	matRemoveButton->setEnabled(false);
	commentEdit->setEnabled(false);
	commentEdit->setPlainText("");
	
	if(!currentProject)
		return;
	
	if(!materialsModel)
		return;
		
	Medium *med = materialsModel->mediumFromIndex(matIndex);
	if(!med)
		return;
	
	// only enable the button and the commentEdit, if the editButton was pressed
	// this is the case if projectTab is enabled
	if(projectTab->isEnabled()){
		matRemoveButton->setEnabled(true);
		commentEdit->setEnabled(true);
	}
	commentEdit->setPlainText(currentProject->comment(med));
}

void Projects::filter ( QString fitlerString )
{
	projectChanged ( QModelIndex() );

  filterEdit->setText ( fitlerString );

  if ( fitlerString.isEmpty() ) {
    proxyModel->setFilterRegExp ( "" );
  } else {
    proxyModel->setFilterRegExp ( QRegExp::escape(fitlerString) );
  }

  if ( proxyModel->rowCount() == 1 ) {
		QModelIndex index = proxyModel->index ( 0, 0 );
    projectsList->setCurrentIndex ( index );
    projectChanged ( index );
	}
}

void Projects::clearProjectDetails()
{
	nameEdit->setText("");
	resultMedium = 0;
	resultEdit->setMedium(resultMedium);
	descriptionEdit->setPlainText("");
	materialChanged(QModelIndex());
	materialsView->setModel(0);
	commentEdit->setPlainText(0);
	currentProject = 0;
}

void Projects::help()
{
	emit showHelp("meganizer/sect0010.html");
}
