/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "shoppingcart.h"
#include "shoppingcart.moc"

#include <medium.h>
#include <mediamodel.h>
#include <delegateshoppingcart.h>
#include <searchplugin.h>
#include <eopenfileexternal.h>
#include <buydata.h>
#include "shoppingcartlistview.h"
#include "insertmanualmedia.h"
#include <epushbutton.h>

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTimer>
#include <QDockWidget>
#include <QLabel>
#include <QMultiMap>
#include <QMenu>
#include <QAction>
#include <QSettings>

ShoppingCart::ShoppingCart ( QList<SearchPlugin*> searchPlugins, QWidget* parent, Qt::WindowFlags flags )
	: EWidget ( parent, flags )
{
  setObjectName ( "shoppingCart" );
  model = 0;
	dock = 0;
  listView = 0;
  delegate = 0;
  viewAction = 0;
  totalLabel = 0;
  buyButton= 0;
  resizeTimer = new QTimer ( this );
	minUsedBorder = 0;
	usedBorder = 100;

  this->searchPlugins = searchPlugins;

  foreach ( SearchPlugin *plugin, searchPlugins ) {
    connect ( plugin, SIGNAL ( foundBuyData ( BuyData ) ), this, SLOT ( foundBuyData ( BuyData ) ) );
  }
}

ShoppingCart::~ShoppingCart()
{
  delete model;
  delete viewAction;
}

void ShoppingCart::init( bool startHidden )
{
	dock = new QDockWidget ( tr ( "Shopping Cart" ) );
  dock->setObjectName ( objectName() +"DockWidget" );
  dock->setWidget ( this );
  emit addNewDockWidget ( dock, Qt::RightDockWidgetArea );
  viewAction = new QAction ( tr ( "Shopping Cart" ), this );
  emit addAction ( viewAction, "viewMenu" );
  viewAction->setCheckable ( true );
  connect ( dock, SIGNAL ( visibilityChanged ( bool ) ), viewAction, SLOT ( setChecked ( bool ) ) );
  connect ( viewAction, SIGNAL ( triggered ( bool ) ), dock, SLOT ( setVisible ( bool ) ) );
	
	if(startHidden)
		dock->setVisible(false);
}

void ShoppingCart::addMedium ( Medium* medium )
{
  if ( !model ) {
    // if we have no model, the shopping cart isn't used right now,
    // so we must setup all stuff now
    setupUi();
  }
  
  if(dock)
		dock->setVisible(true);

  // check if medium already exist's on shopping card, if so, we need to increase the buying number only
  for ( int i = 0; i < model->rowCount(); i++ ) {
    QModelIndex tmpIndex = model->index ( i, 0 );
    Medium *tmp = model->mediumFromIndex ( tmpIndex );
    if ( !tmp )
      continue;
    if ( medium->min("EAN") == tmp->min("EAN") ) {
      // it's the same as the one at row i
      index = tmpIndex;
      increaseQuantity();
      return;
    }
  }

  Medium * newMed = medium->copyMedium();

  model->addMedium ( newMed );
  startSearchBuyData ( newMed );
  index = model->indexFromMedium ( newMed );
  // add to quantity
  increaseQuantity();
}

void ShoppingCart::resizeEvent ( QResizeEvent* event )
{
  if ( listView ) {
    resizeTimer->stop();
    resizeTimer->singleShot ( 1000, this, SLOT ( resizeDelegate() ) );
  }

  QWidget::resizeEvent ( event );
}

void ShoppingCart::buy()
{
  if ( !delegate )
    return;

  QMap<Medium*, BuyData> optimizedDatas = delegate->buyDataList();
  if ( optimizedDatas.uniqueKeys().size() < 1 ) {
    emit error ( tr ( "No shop to buy items found" ) );
    return;
  }

  // let the user change the combinations manually


  // process buying
  // building shop list
  QMultiMap<SearchPlugin *, BuyData> shops;
  foreach ( Medium *med, optimizedDatas.uniqueKeys() ) {
    BuyData data = optimizedDatas.value ( med );
    shops.insert ( data.shop, data );
  }

  // sort shops
  QMap<int, SearchPlugin*> sortShops;
  foreach ( SearchPlugin *shop, shops.uniqueKeys() ) {
    int count = shops.values ( shop ).size();
    sortShops.insert ( count, shop );
  }

	QList<Medium*> boughtMedia;
	QMap<unsigned int, QString> mapCurrencies;
	if(delegate)
		mapCurrencies = delegate->mapCurrenciesList();
  // start with shop with most items
  QMap<int, SearchPlugin*>::const_iterator i;
  i = sortShops.constEnd();
  do {
    --i; // setting it to the first valid position (therefore size() must be greater 0)
    SearchPlugin* shop = i.value();
    if ( shop->purchase ( shops.values ( shop ), mapQuantity ) ) {
      foreach ( BuyData data, shops.values ( shop ) ) {
        removeMedium ( data.medium );
				boughtMedia.append(data.medium);
				data.medium->setBoughtPrice(data.price);
				data.medium->setListPriceCurrency(mapCurrencies.key(data.currencyShortName, data.medium->boughtPriceCurrency()));
				if(data.medium->id() > 0)
					if(!data.medium->store())
						emit warning(tr("Could not set bought price for medium:\n%1").arg(data.medium->title()));
      }
    }
  } while ( i != sortShops.constBegin() );
	
	// save unsaved bought media
	InsertManualMedia *insertMedia = new InsertManualMedia(this);
	connectByPass(insertMedia);
	insertMedia->setUnsavedOrigins(Medium::Manual | Medium::SearchPlugin);
	insertMedia->setupMedia(boughtMedia);
	insertMedia->exec();
	delete insertMedia;
}

void ShoppingCart::resizeDelegate()
{
  if ( !listView )
    return;

  delegate->setMaxWidth ( listView->width() - 20 );

  reloadView();
}

void ShoppingCart::startSearchBuyData ( Medium* medium )
{
  foreach ( SearchPlugin *plugin, searchPlugins ) {
    plugin->recieveBuyData ( medium );
  }
}

void ShoppingCart::foundBuyData ( const BuyData& data )
{
  if ( !model )
    return;

  QModelIndex index = model->indexFromMedium ( data.medium );

  if ( !index.isValid() )
    return;

  mapBuyData.insert ( data.medium, data );
  optimizeBasket();
}

void ShoppingCart::calcSum()
{
  if ( !model )
    return;

  if ( !delegate )
    return;

  if ( !totalLabel )
    return;

  float price = 0;

  QString currency;

  int items = 0;

  QMap<Medium*, BuyData> optimizedBuyData = delegate->buyDataList();
  foreach ( Medium *med, mapQuantity.uniqueKeys() ) {
    float quantity = mapQuantity.value ( med );
    // ignore warning cause float to int, getQuantity returns int's,
    // it is here a float, to allow the multiplication below
    items += ( int ) quantity;
    BuyData buyData = optimizedBuyData.value ( med );
    price += quantity * buyData.price;

    if ( currency.isEmpty() )
      currency = buyData.currencyShortName;
  }

  totalLabel->setText ( tr ( "Total: %1 Items for %2 %3" )

                        .arg ( items )
                        .arg ( QString::number ( price, 'f', 2 ) )
                        .arg ( currency ) );
}

void ShoppingCart::reloadView()
{
  if ( !listView )
    return;

  if ( !delegate )
    return;

  if ( !buyButton )
    return;

  Medium *med = model->mediumFromIndex(index);
	index = QModelIndex();

  listView->reset();

  calcSum();

  buyButton->setEnabled ( false );

  if ( model->rowCount() > 0 )
    buyButton->setEnabled ( true );

	if(med){
		QModelIndex curIndex = model->indexFromMedium(med);
		if(curIndex.isValid()){
			listView->scrollTo(curIndex);
			return;
		}
	}
	listView->scrollToBottom();
}

void ShoppingCart::reloadData()
{
  if ( !delegate )
    return;

  if ( !model )
    return;

  mapBuyData.clear();
	customShop.clear();
	customOffer.clear();
  delegate->clearBuyData();

  reloadView();

  for ( int i = 0; i < model->rowCount(); i++ ) {
    Medium *med = model->mediumFromIndex ( model->index ( i, 0 ) );
    startSearchBuyData ( med );
  }
}

void ShoppingCart::contextMenuRequested ( QPoint pos )
{
  index = listView->indexAt ( pos );

  if ( !index.isValid() )
    return;
	
	if(!model)
		return;
	
	Medium *med = model->mediumFromIndex(index);
	if(!med)
		return;

  QMenu *menu = new QMenu ( this );

  menu->addAction ( QIcon(), tr ( "Increase Quantity" ), this, SLOT ( increaseQuantity() ) );

  menu->addAction ( QIcon(), tr ( "Decrease Quantity" ), this, SLOT ( decreaseQuantity() ) );

  menu->addAction ( QIcon(), tr ( "Remove Medium" ), this, SLOT ( removeMedium() ) );

	QMenu *offersMenu = new QMenu(tr("Offers"));
	menu->addMenu(offersMenu);
  QAction *tmp = offersMenu->addAction ( QIcon(), tr ( "Auto" ), this, SLOT ( setOffer() ) );
	tmp->setObjectName("shop:NoShop offer:Auto");
	tmp->setCheckable(true);
	if((!customShop.contains(med)) && (!customOffer.contains(med)))
		tmp->setChecked(true);

	// creating offers menu structure
	// sort offers to shops
	QList<BuyData> datas = mapBuyData.values(med);
	QMultiMap<SearchPlugin*, BuyData> shops;
	foreach(BuyData data, datas){
		shops.insert(data.shop, data);
	}
	foreach(SearchPlugin *shop, shops.uniqueKeys()){
		QMenu *shopMenu = new QMenu(shop->name());
		offersMenu->addMenu(shopMenu);
		
		shopMenu->setIcon(shop->icon());
		QAction *autoAction = shopMenu->addAction(tr("Auto"), this, SLOT(setOffer()));
		autoAction->setObjectName(QString("shop:%1 offer:Auto").arg(shop->name()));
		autoAction->setCheckable(true);
		if((customShop.contains(med)) && (!customOffer.contains(med))){
			if(customShop.value(med) == shop)
				autoAction->setChecked(true);
		}
		foreach(BuyData data, shops.values(shop)){
			QString usedString;
			if(data.isUsed)
				usedString = tr("(Used)");
			QString title = QString("%1 x %2 %3 %4")
											.arg(data.available)
											.arg(QString::number(data.price, 'f', 2))
											.arg(data.currencyShortName)
											.arg(usedString);
			QAction *offerAction = shopMenu->addAction(title, this, SLOT(setOffer()));
			offerAction->setObjectName(QString("shop:%1 offer:%2;%3").arg(shop->name()).arg(data.available).arg(QString::number(data.price, 'f', 2)));
			offerAction->setCheckable(true);
			if((customShop.contains(med)) && (customOffer.contains(med))){
				if(data == customOffer.value(med))
					offerAction->setChecked(true);
			}
		}
	}
	
  menu->exec ( QCursor::pos() );
}

void ShoppingCart::increaseQuantity(QModelIndex newIndex)
{
	if(!newIndex.isValid()){
		if ( !index.isValid() )
			return;
	}
	else{
		index = newIndex;
	}

  if ( !model )
    return;

  if ( !delegate )
    return;

  Medium *med = model->mediumFromIndex ( index );
  if ( !med )
    return;
  mapQuantity.insert ( med, mapQuantity.value ( med ) + 1 );
  delegate->setQuantity ( index );

  optimizeBasket();
}

void ShoppingCart::decreaseQuantity(QModelIndex newIndex)
{
	if(!newIndex.isValid()){
		if ( !index.isValid() )
			return;
	}
	else{
		index = newIndex;
	}

  if ( !model )
    return;

  if ( !delegate )
    return;

  Medium *med = model->mediumFromIndex ( index );
  if ( !med )
    return;
  int quantity = mapQuantity.value ( med );

  if ( quantity <= 1 ){
    removeMedium ( med );
		return;
	}

  mapQuantity.insert ( med, mapQuantity.value ( med ) - 1 );
  delegate->setQuantity ( index, quantity - 1 );

  optimizeBasket();
}

void ShoppingCart::removeMedium(QModelIndex newIndex)
{
	if(!newIndex.isValid()){
		if ( !index.isValid() )
			return;
	}
	else{
		index = newIndex;
	}

  if ( !model )
    return;

  removeMedium ( model->mediumFromIndex ( index ) );
}

void ShoppingCart::removeMedium ( Medium *med )
{
  if ( !model )
    return;

  if ( !delegate )
    return;

  if ( !med )
    return;
	
  mapQuantity.remove ( med );
  mapBuyData.remove ( med );
	customShop.remove ( med );
	customOffer.remove ( med );
  delegate->remove ( med );
  model->removeMedium ( med );
  optimizeBasket();
}

void ShoppingCart::setOffer()
{
	if(!index.isValid())
		return;
	
	if(!model)
		return;
	
	Medium *med = model->mediumFromIndex(index);
	if(!med)
		return;
	
	QAction *senderAction = qobject_cast<QAction*>(sender());
	if(!senderAction)
		return;
	
	QRegExp parts("shop:([\\w\\W]+) offer:([0-9]+);([0-9]+[\\\\.,]{1}[0-9]{2})");
	if(parts.indexIn(senderAction->objectName()) < 0){
		parts = QRegExp("shop:([\\w\\W]+) offer:Auto");
		if(parts.indexIn(senderAction->objectName()) < 0)
			return;
		// it must be an auto action
		QString shop = parts.cap(1);
		if(shop == "NoShop"){
			customShop.remove(med);
			customOffer.remove(med);
			optimizeBasket();
			return;
		}
		
		QList<BuyData> datas = mapBuyData.values(med);
		foreach(BuyData data, datas){
			if(data.shop->name() == shop){
				customShop.insert(med, data.shop);
				customOffer.remove(med);
				optimizeBasket();
				return;
			}
		}
		optimizeBasket();
		return;
	}
	
	if(parts.capturedTexts().size() != 4)
		return;
	
	QString shop = parts.cap(1);
	bool check;
	int available = parts.cap(2).toInt(&check);
	if(!check)
		return;
#if QT_VERSION < 0x040600
	float price = parts.cap(3).toDouble(&check);
#else
	float price = parts.cap(3).toFloat(&check);
#endif

	if(!check)
		return;
	
	QList<BuyData> datas = mapBuyData.values(med);
	foreach(BuyData data, datas){
		if(data.shop->name() != shop)
			continue;
		if((data.available == available) && (data.price == price)){
			customOffer.insert(med, data);
			customShop.insert(med, data.shop);
			optimizeBasket();
			return;
		}
	}
}

void ShoppingCart::optimizeBasket()
{
  if ( !model )
    return;

  if ( !delegate )
    return;
	
	QSettings settings;
#if QT_VERSION < 0x040600
	usedBorder = settings.value ( "usedBorder", 30 ).toDouble() / 100.0;
	minUsedBorder = settings.value ( "minUsedBorder", 80 ).toDouble() / 100.0;
#else
  usedBorder = settings.value ( "usedBorder", 30 ).toFloat() / 100.0;
	minUsedBorder = settings.value ( "minUsedBorder", 80 ).toFloat() / 100.0;
#endif
	
  // filter out not useable BuyData's and build the shop list
  foreach ( Medium *med, mapBuyData.uniqueKeys() ) {
    if ( !med ){
			foreach(BuyData data, mapBuyData.values(med)){
				mapBuyData.remove(med, data);
			}
      continue;
		}
    // filter out not valid buy data
    foreach ( BuyData data, mapBuyData.values ( med ) ) {
      if (( !data.isValid() ) || ( data.medium != med ))
				mapBuyData.remove(med, data);
    }
  }

	// from here on, we work on our internal BuyData list, so that
	// the user can choose manual from all fetched valid offers during whole process
	QMultiMap<Medium*, BuyData> localBuyData = mapBuyData;
	
  // try to get all items with choosen quantity
  foreach ( Medium *med, localBuyData.uniqueKeys() ) {
		if((customShop.contains(med)) || (customOffer.contains(med)))
			continue;
		
    // remove all buy datas, which have not enough quantities
    // (if there are more than one shop for this item)
    int quantity = mapQuantity.value ( med );
    QList<BuyData> datas = localBuyData.values ( med );

		// check wether at least one shop has enough items
		bool checkPossible = false;
		foreach ( BuyData data, datas ) {
			if ( quantity >= data.available ) {
				checkPossible = true;
				break;
			}
		}
		if(checkPossible){
			foreach ( BuyData data, datas ) {
				if ( quantity > data.available )
					localBuyData.remove ( med, data );
			}
		}
  }

  // get as much items as possible from one shop
  // Strategy:
  //   1. choose the cheapest shop for each item
  //   2. Walk through the shops (start at shop with most items) and stop, if the first shop has all items
  QMultiMap<SearchPlugin*, BuyData> minShops;
  SearchPlugin *onlyShop = 0;

  // 1. choose cheapest shop for each item
  QMultiMap<SearchPlugin*, BuyData> newShops;
  foreach ( Medium *med, localBuyData.uniqueKeys() ) {
    QList<BuyData> datas = localBuyData.values ( med );

    if ( datas.size() < 1 ) {
      emit error ( tr ( "Could not found any shop for this item: %1 (EAN: %2)" ).arg ( med->title() ).arg ( med->min("EAN") ) );
			continue;
    }

		BuyData data = selectBuyData(datas, med);
		if(!data.isValid())
			continue;

		newShops.insert ( data.shop, data );
  }

  // 2. walk through the shops
  // 2.1 sort shops from most items to less items
  QMap<int, SearchPlugin*> sortShops;
  foreach ( SearchPlugin *shop, newShops.uniqueKeys() ) {
    int count = newShops.values ( shop ).size();
    sortShops.insert ( count, shop );
  }

  // 2.2 walking from biggest shop to smallest, means, walking the map from end to start
  if ( sortShops.size() > 0 ) {
    QMap<int, SearchPlugin*>::const_iterator i;
    i = sortShops.constEnd();
    do {
      --i; // setting it to the first valid position (therefore size() must be greater 0)
      SearchPlugin* shop = i.value();
      // try to get all items from this shop
      foreach ( Medium *med, localBuyData.uniqueKeys() ) {
				QList<BuyData> datas;
				foreach(BuyData data, localBuyData.values(med)){
					if(data.shop == shop)
						datas.append(data);
				}
				BuyData data = selectBuyData(datas);
				if(!data.isValid())
					continue;
        minShops.insert ( shop, data );
      }

      // may we have found a shop, wich has all items?
      if ( minShops.values ( shop ).size() == mapQuantity.size() ) {
        onlyShop = shop;
        break;
      }
    } while ( i != sortShops.constBegin() );
  }

  if ( !onlyShop ) {
    // strip out double items
    QList<Medium*> presentMedia;

    // sort again from smallest to biggest shop and walk from biggest to smallest
    sortShops.clear();
    foreach ( SearchPlugin *shop, minShops.uniqueKeys() ) {
      int count = minShops.values ( shop ).size();
      sortShops.insert ( count, shop );
    }

    // 2.2 walking from biggest shop to smallest, means, walking the map from end to start
    if ( sortShops.size() > 0 ) {
      QMultiMap<int, SearchPlugin*>::const_iterator i;
      i = sortShops.constEnd();
      do {
        --i; // setting it to the first valid position (therefore size() must be greater 0)
        SearchPlugin* shop = i.value();
        QList<BuyData> buyDatas = minShops.values ( shop );
        foreach ( BuyData data, buyDatas ) {
          if ( presentMedia.contains ( data.medium ) ) {
            minShops.remove ( shop, data );
          } else {
            presentMedia.append ( data.medium );
          }
        }

      } while ( i != sortShops.constBegin() );
    }
  }

  // check wether another combination is cheaper as one shop minus splitting border


  // transfer choosen buydatas to the delegate
  foreach ( SearchPlugin *shop, minShops.uniqueKeys() ) {
    foreach ( BuyData data, minShops.values ( shop ) ) {
      delegate->setBuyData ( data.medium, data );
    }
  }
  reloadView();
}

void ShoppingCart::setupUi()
{
  QVBoxLayout *mainLayout = new QVBoxLayout ( this );
  setLayout ( mainLayout );

  QHBoxLayout *reloadLayout = new QHBoxLayout();
  mainLayout->addLayout ( reloadLayout );
  reloadLayout->addStretch();

	reloadButton = new EPushButton ("view-refresh", ":/icons/reload.png", tr ( "Reload" ), this );
  connect ( reloadButton,SIGNAL ( clicked() ), this, SLOT ( reloadData() ) );
  reloadLayout->addWidget ( reloadButton );

  model = new MediaModel ( 0, this );
  listView = new ShoppingCartListView ( this );
  listView->setModel ( model );
  listView->setModelColumn ( 0 );
  delegate = new DelegateShoppingCart ( model, listView );
  resizeDelegate();
  listView->setItemDelegate ( delegate );
  listView->setContextMenuPolicy ( Qt::CustomContextMenu );
  connect ( listView, SIGNAL ( customContextMenuRequested ( QPoint ) ), this, SLOT ( contextMenuRequested ( QPoint ) ) );
	connect ( listView, SIGNAL ( increase(QModelIndex)), this, SLOT(increaseQuantity(QModelIndex)));
	connect ( listView, SIGNAL ( decrease(QModelIndex)), this, SLOT(decreaseQuantity(QModelIndex)));
	connect ( listView, SIGNAL ( remove(QModelIndex)), this, SLOT(removeMedium(QModelIndex)));
  mainLayout->addWidget ( listView );

  totalLabel = new QLabel ( this );
  mainLayout->addWidget ( totalLabel );

  buyButton = new EPushButton ( tr ( "Buy" ), this );
  buyButton->setEnabled ( false );
  connect ( buyButton, SIGNAL ( clicked() ), this, SLOT ( buy() ) );
  mainLayout->addWidget ( buyButton );
}

BuyData ShoppingCart::selectBuyData(QList< BuyData > datas, Medium *med)
{
	if(!med){
		if(datas.size() < 1){
			return BuyData();
		}
		else{
			med = datas[0].medium;
			if(!med)
				return BuyData();
		}
	}
	// first - check wether a custom shop or offer was selected
	if(customOffer.contains(med)){
		return customOffer.value(med);
	}
	if(customShop.contains(med)){
		// strip out all buydatas, which are from another shop
		SearchPlugin *shop = customShop.value(med);
		if(!shop)
			return BuyData();
		
		foreach(BuyData data, datas){
			if(data.shop != shop)
				datas.removeAll(data);
		}
	}
	
	// second - strip out all invalid datas
	foreach(BuyData data, datas){
		if(!data.isValid())
			datas.removeAll(data);
	}
	if(datas.size() < 1)
		return BuyData();
	
	// third - find cheapest new item
	BuyData result;
	bool enoughItems = false;
	foreach(BuyData data, datas){
		if(mapQuantity.value(med) <= data.available){
			enoughItems = true;
			break;
		}
	}
	foreach(BuyData data, datas){
		if(data.isUsed)
			continue;
		if((enoughItems) && (mapQuantity.value(med) > data.available))
			continue;
		
		if ( (data.price < result.price) || (result.price == 0) )
			result = data;
	}
	
	// forth - check if we have cheaper used items
	float priceBorder = 0;
	float minPriceBorder = 0;
	if(result.isValid()){
		priceBorder = (1.0 - usedBorder) * result.price;
		minPriceBorder = (1.0 - minUsedBorder) * result.price;
	}
	foreach ( BuyData data, datas ) {
		if(!data.isUsed)
			continue;
		if((enoughItems) && (mapQuantity.value(med) > data.available))
			continue;

		if((priceBorder != 0) && (minPriceBorder != 0)){
			if((minPriceBorder < data.price) && (data.price < priceBorder))
				// the following check must be done, because cheapest can be used item too
				if(data.price < result.price)
					result = data;
		}
		else{
			if((data.price < result.price) || (result.price == 0))
				result = data;
		}
	}
	return result;
}
