/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010 - 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "eanvalidator.h"
#include "eanvalidator.moc"

#include <QObject>
#include <QtPlugin>
#include <QRegExp>

EANValidator::EANValidator()
{

}

QString EANValidator::type() const
{
	return "EAN";
}

QString EANValidator::description() const
{
	return tr("EAN + ISBN-13 - European Product Number is used for many kind of products. It can have 8 or 13 numbers. The ISBN-13 is also an EAN");
}

bool EANValidator::canConvert(QString toType) const
{
	if(toType == "ISBN-10")
		return true;
	if(toType == "ISSN")
		return true;
	return false;
}

QString EANValidator::convert(QString min, QString toType) const
{
	if(!canConvert(toType))
		return QString();

	min = strip(min);
	
	if(!isValid(min))
		return QString();

	if(toType == "ISBN-10"){
		if(min.length() < 13 || (min.left(3) != "978" && min.left(3) != "979"))
			return QString();
		return min.mid(3, min.length() - 1);
	}

	if(toType == "ISSN"){
		if(min.length() < 13 || min.left(3) != "977")
			return QString();
		// we do not return the special purpose characters 11 and 12
		return min.mid(3, min.length() - 6);
	}
	return QString();
}

bool EANValidator::isValid(QString min) const
{
	min.replace(QRegExp("[\\-\\s]"), "");

	if(min != strip(min))
		return false;
	
	if(min.length() != 8 && min.length() != 13)
		return false;

	if(checksum(min) == min.right(1).toInt())
		return true;
	
	return false;
}

QString EANValidator::validate(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return min;
	
	if(min.length() < 7)
		return QString();
	
	if((min.length() > 8) && (min.length() < 12))
		return QString();
	
	if(min.length() > 13)
		return QString();
	
	if((min.length() == 8) || (min.length() == 13))
		min = min.left(min.length() - 1);
	
	min += QString::number(checksum(min));
	if(isValid(min))
		return min;
	return QString();
}

bool EANValidator::isType(QString min) const
{
	if(isValid(min))
		return true;
	return false;
}

bool EANValidator::mayType(QString min) const
{
	min = strip(min);
	
	if(isValid(min))
		return true;
	
	if(min.length() < 13)
		return true;
	
	if(isValid(validate(min)))
		return true;
	
	return false;
}

QString EANValidator::strip(QString min) const
{
	min = min.trimmed();
	min.replace(QRegExp("[^\\d]"), "");
	return min;
}

QStringList EANValidator::commonPrefixes(QStringList mins) const
{
	if(mins.size() < 2)
		return QStringList();
	
	QStringList prefixes;
	while(mins.size() > 0){
		// get first prefix
		QString prefix = commonPrefix(mins);
		// if there is no possible prefix for the mins
		// at the list, there is nothing to be done anymore
		// and we can exit this loop
		if(prefix.isEmpty())
			break;
		// add the found prefix to the list of found prefixes
		prefixes << prefix;
		// remove all mins with same prefix
		foreach(QString min, mins)
			if(min.left(prefix.size()) == prefix)
				mins.removeAll(min);
	}
	
	// check whether a prefix is contained in another one
	// (e.g.: 978342311 is contained in 9783423 and only
	// the last one should be used than).
	QStringList strippedPrefixes;
	while(prefixes.size() > 0){
		QString prefix = commonPrefix(prefixes);
		if(prefix.isEmpty())
			break;
		strippedPrefixes << prefix;
		foreach(QString pref, prefixes)
			if(pref.left(prefix.size()) == prefix)
				prefixes.removeAll(pref);
	}
	//  add the prefixes, which are unique
	strippedPrefixes << prefixes;
	return strippedPrefixes;
}

QStringList EANValidator::minMaxPrefixes(QString min) const
{
	QStringList result;
	result << min.left(7) << min.left(9);
	return result;
}

bool EANValidator::operator==(const MINValidatorPlugin& plugin) const
{
	if(type() == plugin.type())
		return true;
	return false;
}

bool EANValidator::operator!=(const MINValidatorPlugin& plugin) const
{
	return !operator==(plugin);
}

int EANValidator::checksum(QString min) const
{
	min = strip(min);
	
	// remove last digest, because checksum isn't needed
	if((min.length() == 8) || (min.length() == 13))
		min = min.left(min.length() - 1);
		
	if((min.length() != 7) && (min.length() != 12))
		return -1;
	
	int sumEven = 0;
	int sumOdd = 0;
	for(int i = 0; i < min.length(); i++){
		// because of i starts at 0, even numbers are not devideable by 2
		if(i % 2 != 0){
			sumEven += min.mid(i, 1).toInt();
		}
		else{
			sumOdd += min.mid(i, 1).toInt();
		}
	}
	int sum = sumOdd + 3 * sumEven;
	if(sum < 1)
		return -1;
	int result = 0;
	if(sum % 10 != 0)
		result = 10 - (sum % 10);

	return result;
}

QString EANValidator::commonPrefix(QStringList mins) const
{
	for(int i = 0; i < mins.size(); i++){
		for(int j = i + 1; j < mins.size(); j++){
			QString prefix = commonPrefix(mins[i], mins[j]);
			if(!prefix.isEmpty())
				return prefix;
		}
	}
	return QString();
}

QString EANValidator::commonPrefix(QString min1, QString min2) const
{
	// The basic number must be 7 to 9 characters long,
	// but we start to test with 9 to get the longest possible
	// part. Therefore we count from i = 9 to i > 6 (means i == 7).
	// But first we take care, the the two mins have the needed minimum
	// size and we set the start to the minimum length of both strings.
	int start = 9;
	if(min1.length() < 7)
		return QString();
	if(min2.length() < 7)
		return QString();
	if(min1.length() < start)
		start = min1.length();
	if(min2.length() < start)
		start = min2.length();
	for(int i = start; i > 6; i--){
		if(min1.left(i) == min2.left(i))
			return min1.left(i);
	}
	return QString();
}

Q_EXPORT_PLUGIN2 ( Meganizer_EAN, EANValidator )
