/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2011 EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "issnvalidator.h"
#include "issnvalidator.moc"

#include <QObject>
#include <QtPlugin>
#include <QRegExp>

ISSNValidator::ISSNValidator()
{

}

QString ISSNValidator::type() const
{
	return "ISSN";
}

QString ISSNValidator::description() const
{
	return tr("ISSN - International Standard Serial Number is used to identify Magazines."
						" But not a special issue of the identified magazin."
						" For example: ALL Newsweek magazines are identified by 0028-9604."
						" Regardless whether they are published in May 2009 or in December 1993."
					);
}

bool ISSNValidator::canConvert(QString toType) const
{
	if(toType == "EAN")
		return true;
	return false;
}

QString ISSNValidator::convert(QString min, QString toType) const
{
	if(!canConvert(toType))
		return QString();

	min = strip(min);

	if(!isValid(min))
		return QString();

	// it's a ISSN, so prepend 977, append 00 and compute a new check number to repace last number
	// computing new checknumber is done by MINValidator and the proper plugin
  return "977" + min.left ( min.length() - 1 ) + "00";
}

bool ISSNValidator::isValid(QString min) const
{
	min.replace(QRegExp("[\\-\\s]"), "");

	if(min != strip(min))
		return false;
	
	if(min.length() != 8)
		return false;

	// Only the checksum can be an X
	if(min.left(7).toUpper().contains("X"))
		return false;
	
	if(checksum(min).toUpper() == min.right(1).toUpper())
		return true;

	return false;
}

QString ISSNValidator::validate(QString min) const
{
	min = strip(min);

	if(isValid(min))
		return min;

	// A MIN of 7 (ISSN without checksum)
	// or 8 characters may be a ISSN
	if(min.length() < 7 || 8 < min.length())
		return QString();

	if(min.length() == 8)
		min = min.left(min.length() - 1);
	min += checksum(min);
	if(isValid(min))
		return min;

	return QString();
}

bool ISSNValidator::isType(QString min) const
{
	if(isValid(min))
		return true;
	return false;
}

bool ISSNValidator::mayType(QString min) const
{
	min = strip(min);

	if(isValid(min))
		return true;

	if(min.length() < 8)
		return true;

	if(isValid(validate(min)))
		return true;

	return false;
}

QString ISSNValidator::strip(QString min) const
{
	min = min.trimmed();
	min.replace(QRegExp("[^\\dXx]"), "");
	return min;
}

QStringList ISSNValidator::commonPrefixes(QStringList mins) const
{
	return QStringList();
}

QStringList ISSNValidator::minMaxPrefixes(QString min) const
{
	return QStringList();
}

bool ISSNValidator::operator==(const MINValidatorPlugin& plugin) const
{
	if(type() == plugin.type())
		return true;
	return false;
}

bool ISSNValidator::operator!=(const MINValidatorPlugin& plugin) const
{
	return !operator==(plugin);
}

QString ISSNValidator::checksum(QString min) const
{
	min = strip(min);

	if(min.length() == 8)
		min = min.left(min.length() - 1);

	if(min.length() != 7)
		return QString();

	int sum= 0;
	for(int i = 0; i < min.length(); i++){
		sum += (8 - i) * min.mid(i, 1).toInt();
	}
	QString result = "0";
	sum = 11 - sum % 11;
	if(sum == 10){
		result = "X";
	}
	else{
		result = QString::number(sum);
	}
	return result;
}

Q_EXPORT_PLUGIN2 ( Meganizer_ISSN, ISSNValidator )
