/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "purchase.h"

#include "amazon.h"
#include "itemsearch.h"

#include <mediamodel.h>
#include <delegateshoppingcart.h>
#include <eopenfileexternal.h>

#include <QSettings>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QApplication>

Purchase::Purchase ( QList< BuyData > items, QMap< Medium*, int > quantities, QWidget* parent, Qt::WindowFlags flags )
		: EWidget ( parent, flags )
{
  setupUi ( this );

  this->items = items;
  this->quantities = quantities;

  inProgress = false;
  abort = false;
  checkedOut = false;
  canceled = false;

// 	reply = 0;
  model = new MediaModel ( this );
  delegate = new DelegateShoppingCart ( model, this );
  shoppingCart->setModel ( model );
  shoppingCart->setModelColumn ( 0 );
  shoppingCart->setItemDelegate ( delegate );

  statusProgress->setMaximum ( items.size() );
  connect ( buyButton, SIGNAL ( clicked() ), this, SLOT ( buyButtonClicked() ) );
  connect ( cancelButton, SIGNAL ( clicked() ), this, SLOT ( cancelButtonClicked() ) );

  errorStyleSheet = "color: #FF0000;";
}

Purchase::~Purchase()
{
  delete model;
  delete delegate;
}

bool Purchase::prepareShoppingCart()
{
  buyButton->setEnabled ( false );
  show();
  QString msg = tr ( "Preparing remote shopping card" );
  statusLabel->setText ( msg );
  emit logMsg ( msg );

  ItemSearch *itemSearch = new ItemSearch();

  if ( !itemSearch->checkQCA() ) {
    emit error ( tr ( "The signature can not be created due to missing crypting features!\n"
                      "Check your QCA2 installation (e.g. with qcatool2) and espacially the qca-ossl plugin." ) );
    statusLabel->setText ( msg +tr ( " - failed!" ) );
    statusLabel->setStyleSheet ( errorStyleSheet );
    return false;
  }

  QNetworkAccessManager *http = new QNetworkAccessManager ( this );

  connect ( http, SIGNAL ( finished ( QNetworkReply* ) ), this, SLOT ( finishCartAdd ( QNetworkReply* ) ) );

  itemSearch->setCartCreateMode();
  foreach ( BuyData data, items ) {
    QRegExp reg ( "<OfferListingId>(.*)<\\/OfferListingId>" );

    if ( reg.indexIn ( data.customShopData ) < 0 ) {
      statusLabel->setText ( msg +tr ( " - failed!" ) );
      statusLabel->setStyleSheet ( errorStyleSheet );
      return false;
    }

    if ( !itemSearch->addItem ( reg.cap ( 1 ), quantities.value ( data.medium ) ) ) {
      // so we have already the maximum, so let send the cart add or create request
      inProgress = true;
      QNetworkRequest netRequest = itemSearch->netRequest();
      QByteArray content;
      content.append ( itemSearch->getRequest().toUtf8() );
      emit logMsg ( tr ( "Request:\n%1" ).arg ( QString ( content ) ) );
      http->post ( netRequest, content );

      while ( inProgress ) {
        if ( abort ) {
          statusLabel->setText ( msg +tr ( " - failed!" ) );
          statusLabel->setStyleSheet ( errorStyleSheet );
          return false;
        }

        QApplication::processEvents();
      }

      itemSearch->setCartAddMode ( cartId, cartHmac );

      itemSearch->addItem ( reg.cap ( 1 ), quantities.value ( data.medium ) );
    }

    statusProgress->setValue ( statusProgress->value() + 1 );

    QApplication::processEvents();
  }

  inProgress = true;
  QNetworkRequest netRequest = itemSearch->netRequest();
  QByteArray content;
  content.append ( itemSearch->getRequest ( 1 ).toUtf8() );
  emit logMsg ( tr ( "Request:\n%1" ).arg ( QString ( content ) ) );

  http->post ( netRequest, content );

  while ( inProgress ) {
    if ( abort ) {
      statusLabel->setText ( msg +tr ( " - failed!" ) );
      statusLabel->setStyleSheet ( errorStyleSheet );
      return false;
    }

    QApplication::processEvents();
  }

  delete http;

  return true;
}

bool Purchase::checkShoppingCart()
{
  QString msg = tr ( "Checking remote shopping cart" );
  statusLabel->setText ( msg );
  emit logMsg ( msg );
  statusProgress->setValue ( 0 );

  QNetworkAccessManager *http = new QNetworkAccessManager ( this );
  connect ( http, SIGNAL ( finished ( QNetworkReply* ) ),	this, SLOT ( finishCartCheck ( QNetworkReply* ) ) );

  ItemSearch *itemSearch = new ItemSearch();;
  itemSearch->setCartGetMode ( cartId, cartHmac );

  inProgress = true;
  QNetworkRequest netRequest = itemSearch->netRequest();
  QByteArray content;
  content.append ( itemSearch->getRequest().toUtf8() );
  emit logMsg ( tr ( "Request:\n%1" ).arg ( QString ( content ) ) );
  http->post ( netRequest, content );

  while ( inProgress ) {
    if ( abort ) {
      statusLabel->setText ( msg +tr ( " - failed!" ) );
      statusLabel->setStyleSheet ( errorStyleSheet );
      return false;
    }

    QApplication::processEvents();
  }

  delete http;

  buyButton->setEnabled ( true );
  return true;
}

bool Purchase::checkoutClicked()
{
  while ( !canceled ) {
    if ( checkedOut ) {
      close();
      return true;
    }

    QApplication::processEvents();
  }

  close();

  return false;
}

void Purchase::closeEvent ( QCloseEvent* event )
{
  if ( !checkedOut )
    canceled = true;

	EWidget::closeEvent ( event );
}


void Purchase::finishCartAdd ( QNetworkReply *reply )
{
  emit logMsg ( tr ( "Checking cart add" ) );
  // get cartId and hmac if not already set

  if ( ( !cartId.isEmpty() ) && ( !cartHmac.isEmpty() ) ) {
    inProgress = false;
    return;
  }

  if ( !reply ) {
    abort = true;
    return;
  }

  QNetworkReply::NetworkError netError = reply->error();

  if ( netError != QNetworkReply::NoError ) {
    httpRequestError ( netError );
    abort = true;
    return;
  }

  // convert Data into a qdomdocument
  QByteArray data = reply->readAll();

  reply->deleteLater();

  QDomDocument result;

  QDomNodeList node;

  QString errorStr;

  int errorLine;

  int errorColumn;

  if ( !result.setContent ( data, true, &errorStr, &errorLine, &errorColumn ) ) {
    abort = true;
    return;
  }

  emit logMsg ( result.toString() );

  // first check wether the request was valid
  node = result.elementsByTagName ( "IsValid" );

  if ( node.size() > 1 ) {
    abort = true;
    return;
  }

  if ( node.item ( 0 ).toElement().text() != "True" ) {
    abort = true;
    return;
  }

  node = result.elementsByTagName ( "CartId" );

  if ( node.size() != 1 ) {
    abort = true;
    return;
  }

  QDomElement elem = node.item ( 0 ).toElement();

  cartId = elem.text();

  if ( cartId.isEmpty() ) {
    abort = true;
    return;
  }

  elem = elem.nextSiblingElement ( "HMAC" );

  cartHmac = elem.text();

  if ( cartHmac.isEmpty() ) {
    abort = true;
    return;
  }

  elem = elem.nextSiblingElement ( "PurchaseURL" );

  purchaseUrl = elem.text();

  if ( purchaseUrl.isEmpty() ) {
    abort = true;
    return;
  }

  inProgress = false;
}

void Purchase::finishCartCheck ( QNetworkReply *reply )
{
  emit logMsg ( tr ( "checking cart" ) );

  if ( !reply ) {
    abort = true;
    return;
  }

  QNetworkReply::NetworkError netError = reply->error();

  if ( netError != QNetworkReply::NoError ) {
    httpRequestError ( netError );
    abort = true;
    return;
  }

  // convert Data into a qdomdocument
  QByteArray data = reply->readAll();

  reply->deleteLater();

  QDomDocument result;

  QDomNodeList node;

  QString errorStr;

  int errorLine;

  int errorColumn;

  if ( !result.setContent ( data, true, &errorStr, &errorLine, &errorColumn ) ) {
    abort = true;
    return;
  }

  emit logMsg ( result.toString() );

  // first check wether the request was valid
  node = result.elementsByTagName ( "IsValid" );

  if ( node.size() > 1 ) {
    abort = true;
    return;
  }

  if ( node.item ( 0 ).toElement().text() != "True" ) {
    abort = true;
    return;
  }

  node = result.elementsByTagName ( "Cart" );

  if ( node.size() != 1 ) {
    abort = true;
    return;
  }

  QDomElement elem;

  QDomElement cart = node.item ( 0 ).toElement();

  elem = cart.firstChildElement ( "CartId" );

  if ( cartId != elem.text() ) {
    abort = true;
    return;
  }

  elem = elem.nextSiblingElement ( "HMAC" );

  if ( cartHmac != elem.text() ) {
    abort = true;
    return;
  }

  elem = elem.nextSiblingElement ( "PurchaseURL" );

  if ( purchaseUrl != elem.text() ) {
    abort = true;
    return;
  }

  // getting totals
  elem = cart.firstChildElement ( "SubTotal" );

  float total = elem.firstChildElement ( "Amount" ).text().toFloat() / 100;

  totalLabel->setText ( tr ( "Total: %1 %2 (without shipping)" ).arg ( total ).arg ( elem.firstChildElement ( "CurrencyCode" ).text() ) );

  // walking through all items
  node = cart.elementsByTagName ( "CartItem" );

  statusProgress->setMaximum ( node.size() );

  for ( int i = 0; i < node.size(); i++ ) {
    QDomElement item = node.item ( i ).toElement();

    // find corresponding BuyData
    QString asinCart = item.firstChildElement ( "ASIN" ).text();
    QRegExp reg ( "<ASIN>(.*)<\\/ASIN>" );
    int itemData = 0;

    for ( int j = 0; j < items.size(); j++ ) {
      if ( reg.indexIn ( items[j].customShopData ) < 0 ) {
        emit error ( "customshopdata" );
        abort = true;
        return;
      }

      if ( reg.cap ( 1 ) == asinCart ) {
        itemData = j;
      }
    }

    // if inCatalog is false, we haven't found a BuyData corresponding to the given ASIN
    // therefore the shopping cart is not valid

    if ( !items[itemData].inCatalog ) {
      emit error ( "in catalog" );
      abort = true;
      return;
    }

    int quantity = item.firstChildElement ( "Quantity" ).text().toInt();

    if ( quantity != quantities.value ( items[itemData].medium ) ) {
      // if it is not the quantity choosen by the user,
      // it must be the maximum which was available
      if ( quantity != items[itemData].available ) {
        emit error ( "quantity" );
        abort = true;
        return;
      }
    }

    QDomElement price = item.firstChildElement ( "Price" );

    float itemPrice = price.firstChildElement ( "Amount" ).text().toFloat() / 100.0;

		// we use QString here to compare rounded values to 2 decimals after the point
    if ( QString::number(itemPrice, 'f', 2) != QString::number(items[itemData].price, 'f', 2) ) {
			emit error ( tr("Wrong amount! (Actual: %1; Expected: %2)").arg(itemPrice).arg(items[itemData].price) );
      abort = true;
      return;
    }

    if ( items[itemData].currencyShortName != price.firstChildElement ( "CurrencyCode" ).text() ) {
      emit error (
				QString("Wrong currency code (Actual: %1; Expected: %2)")
					.arg(price.firstChildElement ( "CurrencyCode" ).text())
					.arg(items[itemData].currencyShortName)
			);
      abort = true;
      return;
    }

    // everything seems to be alright, so add the item to the listView,
    // but we must add a copy of the medium, so that we can delete our model afterwards without problems
    Medium *newMed = items[itemData].medium->copyMedium();

    model->addMedium ( newMed );

    QModelIndex newMedIndex = model->indexFromMedium ( newMed );

    delegate->setQuantity ( newMedIndex, quantities.value ( items[itemData].medium ) );

    items[itemData].medium = newMed;

    delegate->setBuyData ( newMedIndex, items[itemData] );

    statusProgress->setValue ( statusProgress->value() + 1 );

    QApplication::processEvents();
  }

  inProgress = false;
}

void Purchase::buyButtonClicked()
{
  checkedOut = EOpenFileExternal::openFile(purchaseUrl, "", this);
}

void Purchase::cancelButtonClicked()
{
  canceled = true;
  close();
}

void Purchase::httpRequestError ( QNetworkReply::NetworkError netErr )
{
  QString msg = tr ( "Unknown" );

  if ( netErr == QNetworkReply::ConnectionRefusedError ) {
    msg = tr ( "Connection Refused" );
  }

  if ( netErr == QNetworkReply::RemoteHostClosedError ) {
    msg = tr ( "Remote host has closed the connection" );
  }

  if ( netErr == QNetworkReply::HostNotFoundError ) {
    msg = tr ( "Host not found" );
  }

  if ( netErr == QNetworkReply::TimeoutError ) {
    msg = tr ( "Timeout" );
  }

  if ( netErr == QNetworkReply::OperationCanceledError ) {
    msg = tr ( "Operation canceled" );
  }

  if ( netErr == QNetworkReply::SslHandshakeFailedError ) {
    msg = tr ( "SSL Handshake failed" );
  }

  if ( netErr == QNetworkReply::ContentAccessDenied ) {
    msg = tr ( "Access denied" );
  }

  if ( netErr == QNetworkReply::ContentOperationNotPermittedError ) {
    msg = tr ( "Operation not permitted" );
  }

  if ( netErr == QNetworkReply::ContentNotFoundError ) {
    msg = tr ( "Page not found" );
  }

  if ( netErr == QNetworkReply::AuthenticationRequiredError ) {
    msg = tr ( "Authorisation required" );
  }

  if ( netErr == QNetworkReply::ProtocolUnknownError ) {
    msg = tr ( "Unknown protocol" );
  }

  if ( netErr == QNetworkReply::ProtocolInvalidOperationError ) {
    msg = tr ( "Protocol: Operation invalid" );
  }

  if ( netErr == QNetworkReply::UnknownNetworkError ) {
    msg = tr ( "Unknown network error" );
  }

  if ( netErr == QNetworkReply::UnknownProxyError ) {
    msg = tr ( "Unknown proxy error" );
  }

  if ( netErr == QNetworkReply::UnknownContentError ) {
    msg = tr ( "Unknown content error" );
  }

  if ( netErr == QNetworkReply::ProtocolFailure ) {
    msg = tr ( "protocol failure" );
  }

  // emit error produces a segfault, don't know why.
// 	emit logMsg ( tr ( "Request error\n%1\nat request\n%2." ).arg ( msg ).arg ( reply->url().toString() ) );
  emit logMsg ( tr ( "Request error\n%1." ).arg ( msg ) );
}

#include "purchase.moc"
